@@ -3,7 +3,7 @@
 set -e
 
 # Preparation
-_FLAGS="-I/opt/kxstudio/include -Werror"
+_FLAGS="-Werror"
 export CFLAGS="${_FLAGS}"
 export CXXFLAGS="${_FLAGS}"
 export LDFLAGS="-L/opt/kxstudio/lib"
@@ -23,5 +23,5 @@ make all posix32 posix64
 make wine32 wine64
 
 # Build windows binaries for bridges
-env PATH=/opt/mingw32/bin:${PATH} make win32 CC=i686-w64-mingw32-gcc CXX=i686-w64-mingw32-g++
-env PATH=/opt/mingw64/bin:${PATH} make win64 CC=x86_64-w64-mingw32-gcc CXX=x86_64-w64-mingw32-g++
+env PATH=/opt/mingw32/bin:${PATH} make win32 USING_JUCE=false CC=i686-w64-mingw32-gcc CXX=i686-w64-mingw32-g++
+env PATH=/opt/mingw64/bin:${PATH} make win64 USING_JUCE=false CC=x86_64-w64-mingw32-gcc CXX=x86_64-w64-mingw32-g++
