@@ -5,25 +5,24 @@
 
 source data/macos/common.env
 
-export CC=clang
-export CXX=clang++
-
-export MACOS="true"
-
-if [ $(clang -v  2>&1 | sed -n 's/.*version \([0-9]\).*/\1/p') -lt 9 ]; then
-  export MACOS_OLD="true"
+if [ $(clang -v  2>&1 | grep version | cut -d' ' -f4 | cut -d'.' -f1) -lt 11 ]; then
+  export MACOS_VERSION_MIN="10.8"
+else
+  export MACOS_VERSION_MIN="10.12"
 fi
 
 export CC=clang
 export CXX=clang++
-export CFLAGS="-I${TARGETDIR}/carla64/include -m64"
-export CXXFLAGS="${CFLAGS}"
-export LDFLAGS="-L${TARGETDIR}/carla64/lib -m64"
+export CFLAGS="-I${TARGETDIR}/carla64/include -m64 -mmacosx-version-min=${MACOS_VERSION_MIN}"
+export CXXFLAGS="${CFLAGS} -stdlib=libc++"
+export LDFLAGS="-L${TARGETDIR}/carla64/lib -m64 -stdlib=libc++"
 unset CPPFLAGS
 
+export MACOS="true"
+
 export PATH=${TARGETDIR}/carla/bin:${TARGETDIR}/carla64/bin:/usr/bin:/bin:/usr/sbin:/sbin:/usr/local/bin
 export PKG_CONFIG_PATH=${TARGETDIR}/carla/lib/pkgconfig:${TARGETDIR}/carla64/lib/pkgconfig
 
-#export DEFAULT_QT=5
-#export PYRCC5=$TARGETDIR/carla/bin/pyrcc5
-#export PYUIC5=$TARGETDIR/carla/bin/pyuic5
+export MOC_QT5=moc
+export RCC_QT5=rcc
+export UIC_QT5=uic
