@@ -1,6 +1,6 @@
 /*
  * Carla Plugin Host
- * Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,9 @@
 
 #include "jackbridge/JackBridge.hpp"
 
+#include <ctime>
+#include <sys/time.h>
+
 CARLA_BACKEND_START_NAMESPACE
 
 // -----------------------------------------------------------------------
@@ -359,6 +362,14 @@ void EngineNextAction::clearAndReset() noexcept
     mutex.unlock();
 }
 
+// -----------------------------------------------------------------------
+// Helper functions
+
+EngineEvent* CarlaEngine::getInternalEventBuffer(const bool isInput) const noexcept
+{
+    return isInput ? pData->events.in : pData->events.out;
+}
+
 // -----------------------------------------------------------------------
 // CarlaEngine::ProtectedData
 
@@ -366,16 +377,16 @@ CarlaEngine::ProtectedData::ProtectedData(CarlaEngine* const engine) noexcept
     : thread(engine),
 #if defined(HAVE_LIBLO) && !defined(BUILD_BRIDGE)
       osc(engine),
-      oscData(nullptr),
 #endif
       callback(nullptr),
       callbackPtr(nullptr),
       fileCallback(nullptr),
       fileCallbackPtr(nullptr),
+      actionCanceled(false),
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
       loadingProject(false),
+      currentProjectFilename(),
 #endif
-      hints(0x0),
       bufferSize(0),
       sampleRate(0.0),
       aboutToClose(false),
@@ -390,6 +401,8 @@ CarlaEngine::ProtectedData::ProtectedData(CarlaEngine* const engine) noexcept
       timeInfo(),
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
       plugins(nullptr),
+      xruns(0),
+      dspLoad(0.0f),
 #endif
       events(),
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
@@ -419,9 +432,6 @@ CarlaEngine::ProtectedData::~ProtectedData() noexcept
 bool CarlaEngine::ProtectedData::init(const char* const clientName)
 {
     CARLA_SAFE_ASSERT_RETURN_INTERNAL_ERR(name.isEmpty(), "Invalid engine internal data (err #1)");
-#if defined(HAVE_LIBLO) && !defined(BUILD_BRIDGE)
-    CARLA_SAFE_ASSERT_RETURN_INTERNAL_ERR(oscData == nullptr, "Invalid engine internal data (err #2)");
-#endif
     CARLA_SAFE_ASSERT_RETURN_INTERNAL_ERR(events.in  == nullptr, "Invalid engine internal data (err #4)");
     CARLA_SAFE_ASSERT_RETURN_INTERNAL_ERR(events.out == nullptr, "Invalid engine internal data (err #5)");
     CARLA_SAFE_ASSERT_RETURN_INTERNAL_ERR(clientName != nullptr && clientName[0] != '\0', "Invalid client name");
@@ -472,13 +482,15 @@ bool CarlaEngine::ProtectedData::init(const char* const clientName)
     timeInfo.clear();
 
 #if defined(HAVE_LIBLO) && !defined(BUILD_BRIDGE)
-    osc.init(clientName);
-    oscData = osc.getControlData();
+    if (options.oscEnabled)
+        osc.init(clientName, options.oscPortTCP, options.oscPortUDP);
 #endif
 
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
     plugins = new EnginePluginData[maxPluginNumber];
     carla_zeroStructs(plugins, maxPluginNumber);
+    xruns = 0;
+    dspLoad = 0.0f;
 #endif
 
     nextAction.clearAndReset();
@@ -502,7 +514,6 @@ void CarlaEngine::ProtectedData::close()
 
 #if defined(HAVE_LIBLO) && !defined(BUILD_BRIDGE)
     osc.close();
-    oscData = nullptr;
 #endif
 
     aboutToClose    = false;
@@ -555,21 +566,15 @@ void CarlaEngine::ProtectedData::doPluginRemove(const uint pluginId) noexcept
 
         plugin->setId(i);
 
-        plugins[i].plugin      = plugin;
-        plugins[i].insPeak[0]  = 0.0f;
-        plugins[i].insPeak[1]  = 0.0f;
-        plugins[i].outsPeak[0] = 0.0f;
-        plugins[i].outsPeak[1] = 0.0f;
+        plugins[i].plugin = plugin;
+        carla_zeroFloats(plugins[i].peaks, 4);
     }
 
     const uint id(curPluginCount);
 
     // reset last plugin (now removed)
-    plugins[id].plugin      = nullptr;
-    plugins[id].insPeak[0]  = 0.0f;
-    plugins[id].insPeak[1]  = 0.0f;
-    plugins[id].outsPeak[0] = 0.0f;
-    plugins[id].outsPeak[1] = 0.0f;
+    plugins[id].plugin = nullptr;
+    carla_zeroFloats(plugins[id].peaks, 4);
 }
 
 void CarlaEngine::ProtectedData::doPluginsSwitch(const uint idA, const uint idB) noexcept
@@ -640,8 +645,30 @@ void CarlaEngine::ProtectedData::doNextPluginAction() noexcept
 // -----------------------------------------------------------------------
 // PendingRtEventsRunner
 
-PendingRtEventsRunner::PendingRtEventsRunner(CarlaEngine* const engine, const uint32_t frames) noexcept
-    : pData(engine->pData)
+static int64_t getTimeInMicroseconds() noexcept
+{
+#if defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN)
+    struct timeval tv;
+    gettimeofday(&tv, nullptr);
+
+    return (tv.tv_sec * 1000000) + tv.tv_usec;
+#else
+    struct timespec ts;
+# ifdef CLOCK_MONOTONIC_RAW
+    clock_gettime(CLOCK_MONOTONIC_RAW, &ts);
+# else
+    clock_gettime(CLOCK_MONOTONIC, &ts);
+# endif
+
+    return (ts.tv_sec * 1000000) + (ts.tv_nsec / 1000);
+#endif
+}
+
+PendingRtEventsRunner::PendingRtEventsRunner(CarlaEngine* const engine,
+                                             const uint32_t frames,
+                                             const bool calcDSPLoad) noexcept
+    : pData(engine->pData),
+      prevTime(calcDSPLoad ? getTimeInMicroseconds() : 0)
 {
     pData->time.preProcess(frames);
 }
@@ -649,6 +676,25 @@ PendingRtEventsRunner::PendingRtEventsRunner(CarlaEngine* const engine, const ui
 PendingRtEventsRunner::~PendingRtEventsRunner() noexcept
 {
     pData->doNextPluginAction();
+
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+    if (prevTime > 0)
+    {
+        const int64_t newTime = getTimeInMicroseconds();
+
+        if (newTime < prevTime)
+            return;
+
+        const double timeDiff = static_cast<double>(newTime - prevTime) / 1000000.0;
+        const double maxTime = pData->bufferSize / pData->sampleRate;
+        const float dspLoad = static_cast<float>(timeDiff / maxTime) * 100.0f;
+
+        if (dspLoad > pData->dspLoad)
+            pData->dspLoad = std::min(100.0f, dspLoad);
+        else
+            pData->dspLoad *= static_cast<float>(1.0 - maxTime) + 1e-12f;
+    }
+#endif
 }
 
 // -----------------------------------------------------------------------
