@@ -1,6 +1,6 @@
 /*
  * Carla Plugin Host
- * Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,9 +25,26 @@
 
 #include "jackbridge/JackBridge.hpp"
 
+#if defined(__clang__)
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wconversion"
+# pragma clang diagnostic ignored "-Weffc++"
+#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wconversion"
+# pragma GCC diagnostic ignored "-Weffc++"
+# pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+#endif
+
 #include "rtaudio/RtAudio.h"
 #include "rtmidi/RtMidi.h"
 
+#if defined(__clang__)
+# pragma clang diagnostic pop
+#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+# pragma GCC diagnostic pop
+#endif
+
 CARLA_BACKEND_START_NAMESPACE
 
 // -------------------------------------------------------------------------------------------------------------------
@@ -315,7 +332,7 @@ public:
         if (fAudioOutCount > 0)
             fAudioIntBufOut = new float[fAudioOutCount*bufferFrames];
 
-        pData->graph.create(fAudioInCount, fAudioOutCount);
+        pData->graph.create(fAudioInCount, fAudioOutCount, 0, 0);
 
         try {
             fAudio.startStream();
@@ -327,14 +344,19 @@ public:
             return false;
         }
 
-        patchbayRefresh(false);
+        patchbayRefresh(true, false, false);
 
         if (pData->options.processMode == ENGINE_PROCESS_MODE_PATCHBAY)
-            refreshExternalGraphPorts<PatchbayGraph>(pData->graph.getPatchbayGraph(), false);
+            refreshExternalGraphPorts<PatchbayGraph>(pData->graph.getPatchbayGraph(), false, false);
 
-        callback(ENGINE_CALLBACK_ENGINE_STARTED, 0,
-                 pData->options.processMode, pData->options.transportMode,
-                 pData->sampleRate, getCurrentDriverName());
+        callback(true, true,
+                 ENGINE_CALLBACK_ENGINE_STARTED,
+                 0,
+                 pData->options.processMode,
+                 pData->options.transportMode,
+                 static_cast<int>(pData->bufferSize),
+                 static_cast<float>(pData->sampleRate),
+                 getCurrentDriverName());
         return true;
     }
 
@@ -441,11 +463,11 @@ public:
     // Patchbay
 
     template<class Graph>
-    bool refreshExternalGraphPorts(Graph* const graph, const bool sendCallback)
+    bool refreshExternalGraphPorts(Graph* const graph, const bool sendHost, const bool sendOSC)
     {
         CARLA_SAFE_ASSERT_RETURN(graph != nullptr, false);
 
-        char strBuf[STR_MAX+1];
+        char strBuf[STR_MAX+1U];
         strBuf[STR_MAX] = '\0';
 
         ExternalGraph& extGraph(graph->extGraph);
@@ -511,8 +533,8 @@ public:
         // ---------------------------------------------------------------
         // now refresh
 
-        if (sendCallback)
-            graph->refresh(fDeviceName.buffer());
+        if (sendHost || sendOSC)
+            graph->refresh(sendHost, sendOSC, true, fDeviceName.buffer());
 
         // ---------------------------------------------------------------
         // add midi connections
@@ -534,8 +556,11 @@ public:
 
             extGraph.connections.list.append(connectionToId);
 
-            if (sendCallback)
-                callback(ENGINE_CALLBACK_PATCHBAY_CONNECTION_ADDED, connectionToId.id, 0, 0, 0.0f, strBuf);
+            callback(sendHost, sendOSC,
+                      ENGINE_CALLBACK_PATCHBAY_CONNECTION_ADDED,
+                      connectionToId.id,
+                      0, 0, 0, 0.0f,
+                      strBuf);
         }
 
         fMidiOutMutex.lock();
@@ -557,8 +582,11 @@ public:
 
             extGraph.connections.list.append(connectionToId);
 
-            if (sendCallback)
-                callback(ENGINE_CALLBACK_PATCHBAY_CONNECTION_ADDED, connectionToId.id, 0, 0, 0.0f, strBuf);
+            callback(sendHost, sendOSC,
+                      ENGINE_CALLBACK_PATCHBAY_CONNECTION_ADDED,
+                      connectionToId.id,
+                      0, 0, 0, 0.0f,
+                      strBuf);
         }
 
         fMidiOutMutex.unlock();
@@ -566,27 +594,36 @@ public:
         return true;
     }
 
-    bool patchbayRefresh(const bool external) override
+    bool patchbayRefresh(const bool sendHost, const bool sendOSC, const bool external) override
     {
         CARLA_SAFE_ASSERT_RETURN(pData->graph.isReady(), false);
 
         if (pData->options.processMode == ENGINE_PROCESS_MODE_CONTINUOUS_RACK)
-            return refreshExternalGraphPorts<RackGraph>(pData->graph.getRackGraph(), true);
+            return refreshExternalGraphPorts<RackGraph>(pData->graph.getRackGraph(), sendHost, sendOSC);
 
-        pData->graph.setUsingExternal(external);
+        if (sendHost)
+            pData->graph.setUsingExternalHost(external);
+        if (sendOSC)
+            pData->graph.setUsingExternalOSC(external);
 
         if (external)
-            return refreshExternalGraphPorts<PatchbayGraph>(pData->graph.getPatchbayGraph(), true);
+            return refreshExternalGraphPorts<PatchbayGraph>(pData->graph.getPatchbayGraph(), sendHost, sendOSC);
 
-        return CarlaEngine::patchbayRefresh(false);
+        return CarlaEngine::patchbayRefresh(sendHost, sendOSC, false);
     }
 
     // -------------------------------------------------------------------
 
 protected:
-    void handleAudioProcessCallback(void* outputBuffer, void* inputBuffer, uint nframes, double streamTime, RtAudioStreamStatus status)
+    void handleAudioProcessCallback(void* outputBuffer, void* inputBuffer,
+                                    uint nframes, double streamTime, RtAudioStreamStatus status)
     {
-        const PendingRtEventsRunner prt(this, nframes);
+        const PendingRtEventsRunner prt(this, nframes, true);
+
+        if (status & RTAUDIO_INPUT_OVERFLOW)
+            ++pData->xruns;
+        if (status & RTAUDIO_OUTPUT_UNDERFLOW)
+            ++pData->xruns;
 
         // get buffers from RtAudio
         const float* const insPtr  = (const float*)inputBuffer;
@@ -715,7 +752,7 @@ protected:
                         mdataTmp[0] = static_cast<uint8_t>(midiEvent.data[0] | (engineEvent.channel & MIDI_CHANNEL_BIT));
 
                         // copy rest
-                        carla_copy<uint8_t>(mdataTmp+1, midiEvent.data+1, size-1);
+                        carla_copy<uint8_t>(mdataTmp+1, midiEvent.data+1, size-1U);
 
                         // done
                         mdataPtr = mdataTmp;
@@ -753,7 +790,7 @@ protected:
         }
 
         return; // unused
-        (void)streamTime; (void)status;
+        (void)streamTime;
     }
 
     void handleBufferSizeCallback(const uint newBufferSize)
@@ -1101,7 +1138,7 @@ CarlaEngine* CarlaEngine::newRtAudio(const AudioApi api)
 {
     initRtAudioAPIsIfNeeded();
 
-    RtAudio::Api rtApi(RtAudio::UNSPECIFIED);
+    RtAudio::Api rtApi = RtAudio::UNSPECIFIED;
 
     switch (api)
     {
