@@ -1,6 +1,6 @@
 /*
  * Carla Plugin
- * Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -142,12 +142,20 @@ void PluginCVData::initBuffers() const noexcept
 
 PluginEventData::PluginEventData() noexcept
     : portIn(nullptr),
-      portOut(nullptr) {}
+      portOut(nullptr)
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+    , cvSourcePorts(nullptr)
+#endif
+    {
+    }
 
 PluginEventData::~PluginEventData() noexcept
 {
     CARLA_SAFE_ASSERT(portIn == nullptr);
     CARLA_SAFE_ASSERT(portOut == nullptr);
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+    CARLA_SAFE_ASSERT(cvSourcePorts == nullptr);
+#endif
 }
 
 void PluginEventData::clear() noexcept
@@ -163,6 +171,14 @@ void PluginEventData::clear() noexcept
         delete portOut;
         portOut = nullptr;
     }
+
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+    if (cvSourcePorts != nullptr)
+    {
+        cvSourcePorts->cleanup();
+        cvSourcePorts = nullptr;
+    }
+#endif
 }
 
 void PluginEventData::initBuffers() const noexcept
@@ -206,7 +222,9 @@ void PluginParameterData::createNew(const uint32_t newCount, const bool withSpec
     {
         data[i].index  = PARAMETER_NULL;
         data[i].rindex = PARAMETER_NULL;
-        data[i].midiCC = -1;
+        data[i].mappedControlIndex = CONTROL_INDEX_NONE;
+        data[i].mappedMinimum = -1.0f;
+        data[i].mappedMaximum = 1.0f;
     }
 
     ranges = new ParameterRanges[newCount];
@@ -244,7 +262,7 @@ void PluginParameterData::clear() noexcept
     count = 0;
 }
 
-float PluginParameterData::getFixedValue(const uint32_t parameterId, const float& value) const noexcept
+float PluginParameterData::getFixedValue(const uint32_t parameterId, float value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(parameterId < count, 0.0f);
 
@@ -266,6 +284,67 @@ float PluginParameterData::getFixedValue(const uint32_t parameterId, const float
     return paramRanges.getFixedValue(value);
 }
 
+// copied from ParameterRanges::getUnnormalizedValue
+static float _getUnnormalizedValue(const float min, const float max, const float value) noexcept
+{
+    if (value <= 0.0f)
+        return min;
+    if (value >= 1.0f)
+        return max;
+
+    return value * (max - min) + min;
+}
+
+// copied from ParameterRanges::getUnnormalizedLogValue
+static float _getUnnormalizedLogValue(const float min, const float max, const float value) noexcept
+{
+    if (value <= 0.0f)
+        return min;
+    if (value >= 1.0f)
+        return max;
+
+    float rmin = min;
+
+    if (std::abs(min) < std::numeric_limits<float>::epsilon())
+        rmin = 0.00001f;
+
+    return rmin * std::pow(max/rmin, value);
+}
+
+float PluginParameterData::getFinalUnnormalizedValue(const uint32_t parameterId, float value) const noexcept
+{
+    float min, max;
+
+    if (data[parameterId].mappedControlIndex != CONTROL_INDEX_CV
+        && (data[parameterId].hints & PARAMETER_MAPPED_RANGES_SET) != 0x0)
+    {
+        min = data[parameterId].mappedMinimum;
+        max = data[parameterId].mappedMaximum;
+    }
+    else
+    {
+        min = ranges[parameterId].min;
+        max = ranges[parameterId].max;
+    }
+
+    if (data[parameterId].hints & PARAMETER_IS_BOOLEAN)
+    {
+        value = (value < 0.5f) ? min : max;
+    }
+    else
+    {
+        if (data[parameterId].hints & PARAMETER_IS_LOGARITHMIC)
+            value = _getUnnormalizedLogValue(min, max, value);
+        else
+            value = _getUnnormalizedValue(min, max, value);
+
+        if (data[parameterId].hints & PARAMETER_IS_INTEGER)
+            value = std::rint(value);
+    }
+
+    return value;
+}
+
 // -----------------------------------------------------------------------
 // PluginProgramData
 
@@ -501,27 +580,27 @@ void CarlaPlugin::ProtectedData::Latency::recreateBuffers(const uint32_t newChan
 // ProtectedData::PostRtEvents
 
 CarlaPlugin::ProtectedData::PostRtEvents::PostRtEvents() noexcept
-    : dataPool(128, 128),
-      dataPendingRT(dataPool),
+    : dataPool(512, 512),
       data(dataPool),
+      dataPendingRT(dataPool),
       dataMutex(),
       dataPendingMutex() {}
 
 CarlaPlugin::ProtectedData::PostRtEvents::~PostRtEvents() noexcept
 {
     dataMutex.lock();
-    data.clear();
-    dataMutex.unlock();
-
     dataPendingMutex.lock();
+
+    data.clear();
     dataPendingRT.clear();
+
+    dataMutex.unlock();
     dataPendingMutex.unlock();
 }
 
 void CarlaPlugin::ProtectedData::PostRtEvents::appendRT(const PluginPostRtEvent& e) noexcept
 {
     CARLA_SAFE_ASSERT_INT2_RETURN(dataPendingMutex.tryLock(), e.type, e.value1,);
-
     dataPendingRT.append(e);
     dataPendingMutex.unlock();
 }
@@ -530,23 +609,13 @@ void CarlaPlugin::ProtectedData::PostRtEvents::trySplice() noexcept
 {
     const CarlaMutexTryLocker cmtl(dataPendingMutex);
 
-    if (cmtl.wasLocked() && dataPendingRT.count() > 0 && dataMutex.tryLock())
+    if (cmtl.wasLocked() && dataPendingRT.isNotEmpty() && dataMutex.tryLock())
     {
         dataPendingRT.moveTo(data, true);
         dataMutex.unlock();
     }
 }
 
-void CarlaPlugin::ProtectedData::PostRtEvents::clearData() noexcept
-{
-    const bool tryLockOk(dataMutex.tryLock());
-    CARLA_SAFE_ASSERT(! tryLockOk);
-    data.clear();
-
-    if (tryLockOk)
-        dataMutex.unlock();
-}
-
 // -----------------------------------------------------------------------
 // ProtectedData::PostUiEvents
 
@@ -749,11 +818,16 @@ void CarlaPlugin::ProtectedData::postponeRtEvent(const PluginPostRtEvent& rtEven
     postRtEvents.appendRT(rtEvent);
 }
 
-void CarlaPlugin::ProtectedData::postponeRtEvent(const PluginPostRtEventType type, const int32_t value1, const int32_t value2, const float value3) noexcept
+void CarlaPlugin::ProtectedData::postponeRtEvent(const PluginPostRtEventType type,
+                                                 const bool sendCallbackLater,
+                                                 const int32_t value1,
+                                                 const int32_t value2,
+                                                 const int32_t value3,
+                                                 const float valuef) noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(type != kPluginPostRtEventNull,);
 
-    PluginPostRtEvent rtEvent = { type, value1, value2, value3 };
+    PluginPostRtEvent rtEvent = { type, sendCallbackLater, value1, value2, value3, valuef };
 
     postRtEvents.appendRT(rtEvent);
 }
@@ -781,6 +855,11 @@ bool CarlaPlugin::ProtectedData::libClose() noexcept
     return ret;
 }
 
+void CarlaPlugin::ProtectedData::setCanDeleteLib(const bool canDelete) noexcept
+{
+    sLibCounter.setCanDelete(lib, canDelete);
+}
+
 bool CarlaPlugin::ProtectedData::uiLibOpen(const char* const fname, const bool canDelete) noexcept
 {
     uiLib = sLibCounter.open(fname, canDelete);
@@ -804,7 +883,10 @@ void CarlaPlugin::ProtectedData::tryTransient() noexcept
 }
 #endif
 
-void CarlaPlugin::ProtectedData::updateParameterValues(CarlaPlugin* const plugin, const bool sendOsc, const bool sendCallback, const bool useDefault) noexcept
+void CarlaPlugin::ProtectedData::updateParameterValues(CarlaPlugin* const plugin,
+                                                       const bool sendCallback,
+                                                       const bool sendOsc,
+                                                       const bool useDefault) noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(sendOsc || sendCallback || useDefault,);
 
@@ -815,25 +897,24 @@ void CarlaPlugin::ProtectedData::updateParameterValues(CarlaPlugin* const plugin
         if (useDefault)
             param.ranges[i].def = value;
 
-#if defined(HAVE_LIBLO) && ! defined(BUILD_BRIDGE)
-        if (sendOsc && i < 50)
-        {
-            if (useDefault)
-                engine->oscSend_control_set_default_value(id, i, value);
-            engine->oscSend_control_set_parameter_value(id, static_cast<int32_t>(i), value);
+        if (useDefault) {
+            engine->callback(sendCallback, sendOsc,
+                             ENGINE_CALLBACK_PARAMETER_DEFAULT_CHANGED,
+                             id,
+                             static_cast<int>(i),
+                             0, 0,
+                             value,
+                             nullptr);
         }
-#endif
 
-        if (sendCallback)
-        {
-            if (useDefault)
-                engine->callback(ENGINE_CALLBACK_PARAMETER_DEFAULT_CHANGED, id, static_cast<int>(i), 0, value, nullptr);
-            engine->callback(ENGINE_CALLBACK_PARAMETER_VALUE_CHANGED, id, static_cast<int>(i), 0, value, nullptr);
-        }
+        engine->callback(sendCallback, sendOsc,
+                         ENGINE_CALLBACK_PARAMETER_VALUE_CHANGED,
+                         id,
+                         static_cast<int>(i),
+                         0, 0,
+                         value,
+                         nullptr);
     }
-
-    // may be unused
-    return; (void)sendOsc;
 }
 
 void CarlaPlugin::ProtectedData::updateDefaultParameterValues(CarlaPlugin* const plugin) noexcept
