@@ -1,6 +1,6 @@
 ﻿/*
  * Carla Plugin
- * Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,7 @@
 
 #include "CarlaPlugin.hpp"
 
+#include "CarlaJuceUtils.hpp"
 #include "CarlaLibUtils.hpp"
 #include "CarlaStateUtils.hpp"
 
@@ -44,6 +45,13 @@ CARLA_BACKEND_START_NAMESPACE
 
 const ushort kPluginMaxMidiEvents = 512;
 
+// -----------------------------------------------------------------------
+// Extra parameter hints, hidden from backend
+
+const uint PARAMETER_MAPPED_RANGES_SET = 0x1000;
+const uint PARAMETER_IS_STRICT_BOUNDS  = 0x2000;
+const uint PARAMETER_IS_TRIGGER        = 0x4000;
+
 // -----------------------------------------------------------------------
 // Extra plugin hints, hidden from backend
 
@@ -67,14 +75,14 @@ enum SpecialParameterType {
  * Post-RT event type.
  * These are events postponned from within the process function,
  *
- * During process, we cannot lock, allocate memory or do UI stuff,
- * so events have to be postponned to be executed later, on a separate thread.
+ * During process, we cannot lock, allocate memory or do UI stuff.
+ * Events have to be postponned to be executed later, on a separate thread.
  * @see PluginPostRtEvent
  */
 enum PluginPostRtEventType {
     kPluginPostRtEventNull = 0,
     kPluginPostRtEventDebug,
-    kPluginPostRtEventParameterChange,   // param, SP (*), value (SP: if 1 only report change to UI, don't report to Callback and OSC)
+    kPluginPostRtEventParameterChange,   // param, (unused), (unused), value
     kPluginPostRtEventProgramChange,     // index
     kPluginPostRtEventMidiProgramChange, // index
     kPluginPostRtEventNoteOn,            // channel, note, velo
@@ -87,9 +95,11 @@ enum PluginPostRtEventType {
  */
 struct PluginPostRtEvent {
     PluginPostRtEventType type;
+    bool sendCallback;
     int32_t value1;
     int32_t value2;
-    float   value3;
+    int32_t value3;
+    float   valuef;
 };
 
 // -----------------------------------------------------------------------
@@ -113,7 +123,7 @@ struct PluginAudioData {
 
     PluginAudioData() noexcept;
     ~PluginAudioData() noexcept;
-    void createNew(const uint32_t newCount);
+    void createNew(uint32_t newCount);
     void clear() noexcept;
     void initBuffers() const noexcept;
 
@@ -134,7 +144,7 @@ struct PluginCVData {
 
     PluginCVData() noexcept;
     ~PluginCVData() noexcept;
-    void createNew(const uint32_t newCount);
+    void createNew(uint32_t newCount);
     void clear() noexcept;
     void initBuffers() const noexcept;
 
@@ -146,6 +156,9 @@ struct PluginCVData {
 struct PluginEventData {
     CarlaEngineEventPort* portIn;
     CarlaEngineEventPort* portOut;
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+    CarlaEngineCVSourcePorts* cvSourcePorts;
+#endif
 
     PluginEventData() noexcept;
     ~PluginEventData() noexcept;
@@ -165,9 +178,10 @@ struct PluginParameterData {
 
     PluginParameterData() noexcept;
     ~PluginParameterData() noexcept;
-    void createNew(const uint32_t newCount, const bool withSpecial);
+    void createNew(uint32_t newCount, bool withSpecial);
     void clear() noexcept;
-    float getFixedValue(const uint32_t parameterId, const float& value) const noexcept;
+    float getFixedValue(uint32_t parameterId, float value) const noexcept;
+    float getFinalUnnormalizedValue(uint32_t parameterId, float value) const noexcept;
 
     CARLA_DECLARE_NON_COPY_STRUCT(PluginParameterData)
 };
@@ -183,7 +197,7 @@ struct PluginProgramData {
 
     PluginProgramData() noexcept;
     ~PluginProgramData() noexcept;
-    void createNew(const uint32_t newCount);
+    void createNew(uint32_t newCount);
     void clear() noexcept;
 
     CARLA_DECLARE_NON_COPY_STRUCT(PluginProgramData)
@@ -198,7 +212,7 @@ struct PluginMidiProgramData {
 
     PluginMidiProgramData() noexcept;
     ~PluginMidiProgramData() noexcept;
-    void createNew(const uint32_t newCount);
+    void createNew(uint32_t newCount);
     void clear() noexcept;
     const MidiProgramData& getCurrent() const noexcept;
 
@@ -280,7 +294,7 @@ struct CarlaPlugin::ProtectedData {
 #ifndef BUILD_BRIDGE
         ~Latency() noexcept;
         void clearBuffers() noexcept;
-        void recreateBuffers(const uint32_t newChannels, const uint32_t newFrames);
+        void recreateBuffers(uint32_t newChannels, uint32_t newFrames);
 #endif
 
         CARLA_DECLARE_NON_COPY_STRUCT(Latency)
@@ -293,25 +307,40 @@ struct CarlaPlugin::ProtectedData {
         ~PostRtEvents() noexcept;
         void appendRT(const PluginPostRtEvent& event) noexcept;
         void trySplice() noexcept;
-        void clearData() noexcept;
 
-        inline CarlaMutex& getDataMutex() noexcept
-        {
-            return dataMutex;
-        }
+        struct Access {
+            Access(PostRtEvents& e)
+              : data2(e.dataPool)
+            {
+                const CarlaMutexLocker cml(e.dataMutex);
 
-        inline RtLinkedList<PluginPostRtEvent>::Itenerator getDataIterator() const noexcept
-        {
-            return data.begin2();
-        }
+                if (e.data.isNotEmpty())
+                    e.data.moveTo(data2, true);
+            }
+
+            ~Access()
+            {
+                data2.clear();
+            }
+
+            inline RtLinkedList<PluginPostRtEvent>::Itenerator getDataIterator() const noexcept
+            {
+                return data2.begin2();
+            }
+
+            inline std::size_t isEmpty() const noexcept
+            {
+                return data2.isEmpty();
+            }
+
+        private:
+            RtLinkedList<PluginPostRtEvent> data2;
+        };
 
     private:
         RtLinkedList<PluginPostRtEvent>::Pool dataPool;
-        RtLinkedList<PluginPostRtEvent> dataPendingRT;
-        RtLinkedList<PluginPostRtEvent> data;
+        RtLinkedList<PluginPostRtEvent> data, dataPendingRT;
         CarlaMutex dataMutex;
-
-        // TESTING for thread-safety, remove later
         CarlaMutex dataPendingMutex;
 
         CARLA_DECLARE_NON_COPY_CLASS(PostRtEvents)
@@ -346,7 +375,7 @@ struct CarlaPlugin::ProtectedData {
     } postProc;
 #endif
 
-    ProtectedData(CarlaEngine* const engine, const uint idx) noexcept;
+    ProtectedData(CarlaEngine* engine, uint idx) noexcept;
     ~ProtectedData() noexcept;
 
     // -------------------------------------------------------------------
@@ -358,27 +387,29 @@ struct CarlaPlugin::ProtectedData {
     // Post-poned events
 
     void postponeRtEvent(const PluginPostRtEvent& rtEvent) noexcept;
-    void postponeRtEvent(const PluginPostRtEventType type, const int32_t value1, const int32_t value2, const float value3) noexcept;
+    void postponeRtEvent(PluginPostRtEventType type, bool sendCallbackLater,
+                         int32_t value1, int32_t value2, int32_t value3, float valuef) noexcept;
 
     // -------------------------------------------------------------------
     // Library functions
 
-    static const char* libError(const char* const filename) noexcept;
+    static const char* libError(const char* filename) noexcept;
 
-    bool libOpen(const char* const filename) noexcept;
+    bool libOpen(const char* filename) noexcept;
     bool libClose() noexcept;
+    void setCanDeleteLib(bool canDelete) noexcept;
 
-    bool uiLibOpen(const char* const filename, const bool canDelete) noexcept;
+    bool uiLibOpen(const char* filename, bool canDelete) noexcept;
     bool uiLibClose() noexcept;
 
     template<typename Func>
-    Func libSymbol(const char* const symbol) const noexcept
+    Func libSymbol(const char* symbol) const noexcept
     {
         return lib_symbol<Func>(lib, symbol);
     }
 
     template<typename Func>
-    Func uiLibSymbol(const char* const symbol) const noexcept
+    Func uiLibSymbol(const char* symbol) const noexcept
     {
         return lib_symbol<Func>(uiLib, symbol);
     }
@@ -389,8 +420,9 @@ struct CarlaPlugin::ProtectedData {
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
     void tryTransient() noexcept;
 #endif
-    void updateParameterValues(CarlaPlugin* const plugin, const bool sendOsc, const bool sendCallback, const bool useDefault) noexcept;
-    void updateDefaultParameterValues(CarlaPlugin* const plugin) noexcept;
+    void updateParameterValues(CarlaPlugin* plugin,
+                               bool sendCallback, bool sendOsc, bool useDefault) noexcept;
+    void updateDefaultParameterValues(CarlaPlugin* plugin) noexcept;
 
     // -------------------------------------------------------------------
 
