@@ -1,6 +1,6 @@
 /*
  * Carla Plugin JACK
- * Copyright (C) 2016-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2016-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,14 +20,23 @@
 
 #ifdef CARLA_OS_LINUX
 
+#include "CarlaLibJackHints.h"
 #include "CarlaBackendUtils.hpp"
 #include "CarlaBridgeUtils.hpp"
 #include "CarlaEngineUtils.hpp"
 #include "CarlaMathUtils.hpp"
 #include "CarlaPipeUtils.hpp"
+#include "CarlaScopeUtils.hpp"
 #include "CarlaShmUtils.hpp"
 #include "CarlaThread.hpp"
 
+#ifdef HAVE_LIBLO
+# include "CarlaOscUtils.hpp"
+#else
+# warning No liblo support, NSM (session state) will not be available
+#endif
+
+#include "water/files/File.h"
 #include "water/misc/Time.h"
 #include "water/text/StringArray.h"
 #include "water/threads/ChildProcess.h"
@@ -47,6 +56,14 @@ using water::Time;
 
 CARLA_BACKEND_START_NAMESPACE
 
+static size_t safe_rand(const size_t limit)
+{
+    const int r = std::rand();
+    CARLA_SAFE_ASSERT_RETURN(r >= 0, 0);
+
+    return static_cast<uint>(r) % limit;
+}
+
 // -------------------------------------------------------------------------------------------------------------------
 // Fallback data
 
@@ -62,17 +79,22 @@ public:
           kEngine(engine),
           kPlugin(plugin),
           fShmIds(),
-          fNumPorts(),
+          fSetupLabel(),
+#ifdef HAVE_LIBLO
+          fOscClientAddress(nullptr),
+          fOscServer(nullptr),
+          fProject(),
+#endif
           fProcess() {}
 
-    void setData(const char* const shmIds, const char* const numPorts) noexcept
+    void setData(const char* const shmIds, const char* const setupLabel) noexcept
     {
         CARLA_SAFE_ASSERT_RETURN(shmIds != nullptr && shmIds[0] != '\0',);
-        CARLA_SAFE_ASSERT_RETURN(numPorts != nullptr && numPorts[0] != '\0',);
+        CARLA_SAFE_ASSERT_RETURN(setupLabel != nullptr && setupLabel[0] != '\0',);
         CARLA_SAFE_ASSERT(! isThreadRunning());
 
-        fShmIds   = shmIds;
-        fNumPorts = numPorts;
+        fShmIds     = shmIds;
+        fSetupLabel = setupLabel;
     }
 
     uintptr_t getProcessID() const noexcept
@@ -82,34 +104,216 @@ public:
         return (uintptr_t)fProcess->getPID();
     }
 
+#ifdef HAVE_LIBLO
+    void nsmSave(const char* const setupLabel)
+    {
+        if (fOscClientAddress == nullptr)
+            return;
+
+        if (fSetupLabel != setupLabel)
+            fSetupLabel = setupLabel;
+
+        maybeOpenFirstTime();
+
+        lo_send_from(fOscClientAddress, fOscServer, LO_TT_IMMEDIATE, "/nsm/client/save", "");
+    }
+
+    void nsmShowGui(const bool yesNo)
+    {
+        if (fOscClientAddress == nullptr)
+            return;
+
+        lo_send_from(fOscClientAddress, fOscServer, LO_TT_IMMEDIATE,
+                     yesNo ? "/nsm/client/show_optional_gui"
+                           : "/nsm/client/hide_optional_gui", "");
+    }
+#endif
+
+    char* getEnvVarsToExport()
+    {
+        const EngineOptions& options(kEngine->getOptions());
+        CarlaString binaryDir(options.binaryDir);
+#ifdef HAVE_LIBLO
+        const int sessionManager = fSetupLabel[4] - '0';
+#endif
+
+        CarlaString ret;
+        ret += "export LD_LIBRARY_PATH=" + binaryDir + "/jack\n";
+#ifdef HAVE_X11
+        ret += "export LD_PRELOAD=" + binaryDir + "/libcarla_interposer-jack-x11.so\n";
+#endif
+#ifdef HAVE_LIBLO
+        if (sessionManager == LIBJACK_SESSION_MANAGER_NSM)
+        {
+            for (int i=50; fOscServer == nullptr && --i>=0;)
+                carla_msleep(100);
+
+            ret += "export NSM_URL=";
+            ret += lo_server_get_url(fOscServer);
+            ret += "\n";
+        }
+#endif
+
+        if (kPlugin->getHints() & PLUGIN_HAS_CUSTOM_UI)
+            ret += "export CARLA_FRONTEND_WIN_ID=" + CarlaString(options.frontendWinId) + "\n";
+
+        ret += "export CARLA_LIBJACK_SETUP=" + fSetupLabel + "\n";
+        ret += "export CARLA_SHM_IDS=" + fShmIds + "\n";
+
+        return ret.releaseBufferPointer();
+    }
+
 protected:
+#ifdef HAVE_LIBLO
+    static void _osc_error_handler(int num, const char* msg, const char* path)
+    {
+        carla_stderr2("CarlaPluginJackThread::_osc_error_handler(%i, \"%s\", \"%s\")", num, msg, path);
+    }
+
+    static int _broadcast_handler(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* data)
+    {
+        CARLA_SAFE_ASSERT_RETURN(data != nullptr, 0);
+        carla_stdout("CarlaPluginJackThread::_broadcast_handler(%s, %s, %p, %i)", path, types, argv, argc);
+
+        return ((CarlaPluginJackThread*)data)->handleBroadcast(path, types, argv, msg);
+    }
+
+    void maybeOpenFirstTime()
+    {
+        if (fSetupLabel.length() <= 6)
+            return;
+
+        if (fProject.path.isNotEmpty() || fProject.init(kEngine->getCurrentProjectFilename(), &fSetupLabel[6]))
+        {
+            carla_stdout("Sending open signal %s %s %s",
+                         fProject.path.buffer(), fProject.display.buffer(), fProject.clientName.buffer());
+
+            lo_send_from(fOscClientAddress, fOscServer, LO_TT_IMMEDIATE, "/nsm/client/open", "sss",
+                            fProject.path.buffer(), fProject.display.buffer(), fProject.clientName.buffer());
+        }
+    }
+
+    int handleBroadcast(const char* path, const char* types, lo_arg** argv, lo_message msg)
+    {
+        if (std::strcmp(path, "/nsm/server/announce") == 0)
+        {
+            CARLA_SAFE_ASSERT_RETURN(std::strcmp(types, "sssiii") == 0, 0);
+
+            const lo_address msgAddress(lo_message_get_source(msg));
+            CARLA_SAFE_ASSERT_RETURN(msgAddress != nullptr, 0);
+
+            char* const msgURL(lo_address_get_url(msgAddress));
+            CARLA_SAFE_ASSERT_RETURN(msgURL != nullptr, 0);
+
+            if (fOscClientAddress != nullptr)
+                lo_address_free(fOscClientAddress);
+
+            fOscClientAddress = lo_address_new_from_url(msgURL);
+            CARLA_SAFE_ASSERT_RETURN(fOscClientAddress != nullptr, 0);
+
+            fProject.appName = &argv[0]->s;
+
+            static const char* const method   = "/nsm/server/announce";
+            static const char* const message  = "Howdy, what took you so long?";
+            static const char* const smName   = "Carla";
+            static const char* const features = ":server-control:optional-gui:";
+
+            lo_send_from(fOscClientAddress, fOscServer, LO_TT_IMMEDIATE, "/reply", "ssss",
+                         method, message, smName, features);
+
+            maybeOpenFirstTime();
+        }
+
+        else if (std::strcmp(path, "/reply") == 0)
+        {
+            CARLA_SAFE_ASSERT_RETURN(std::strcmp(types, "ss") == 0, 0);
+
+            const char* const method  = &argv[0]->s;
+            const char* const message = &argv[1]->s;
+
+            carla_stdout("Got reply of '%s' as '%s'", method, message);
+
+            if (std::strcmp(method, "/nsm/client/open") == 0)
+            {
+                carla_stdout("Sending 'Session is loaded' to %s", fProject.appName.buffer());
+                lo_send_from(fOscClientAddress, fOscServer, LO_TT_IMMEDIATE, "/nsm/client/session_is_loaded", "");
+            }
+        }
+
+        else if (std::strcmp(path, "/nsm/client/gui_is_shown") == 0)
+        {
+            CARLA_SAFE_ASSERT_RETURN(std::strcmp(types, "") == 0, 0);
+
+            kEngine->callback(true, true,
+                              ENGINE_CALLBACK_UI_STATE_CHANGED,
+                              kPlugin->getId(),
+                              1,
+                              0, 0, 0.0f, nullptr);
+        }
+
+        else if (std::strcmp(path, "/nsm/client/gui_is_hidden") == 0)
+        {
+            CARLA_SAFE_ASSERT_RETURN(std::strcmp(types, "") == 0, 0);
+
+            kEngine->callback(true, true,
+                              ENGINE_CALLBACK_UI_STATE_CHANGED,
+                              kPlugin->getId(),
+                              0,
+                              0, 0, 0.0f, nullptr);
+        }
+
+        return 0;
+    }
+#endif
+
     void run()
     {
-        if (fProcess == nullptr)
+#ifdef HAVE_LIBLO
+        if (fOscClientAddress != nullptr)
         {
-            fProcess = new ChildProcess();
-        }
-        else if (fProcess->isRunning())
-        {
-            carla_stderr("CarlaPluginJackThread::run() - already running");
+            lo_address_free(fOscClientAddress);
+            fOscClientAddress = nullptr;
         }
 
-        String name(kPlugin->getName());
-        String filename(kPlugin->getFilename());
-
-        if (name.isEmpty())
-            name = "(none)";
-
-        CARLA_SAFE_ASSERT_RETURN(filename.isNotEmpty(),);
-
-        StringArray arguments;
-
-        // binary
-        arguments.addTokens(filename, true);
-
-        bool started;
+        const int sessionManager = fSetupLabel[4] - '0';
 
+        if (sessionManager == LIBJACK_SESSION_MANAGER_NSM)
         {
+            // NSM support
+            fOscServer = lo_server_new_with_proto(nullptr, LO_UDP, _osc_error_handler);
+            CARLA_SAFE_ASSERT_RETURN(fOscServer != nullptr,);
+
+            lo_server_add_method(fOscServer, nullptr, nullptr, _broadcast_handler, this);
+        }
+#endif
+
+        const bool externalProcess = ((fSetupLabel[5] - '0') & LIBJACK_FLAG_EXTERNAL_START)
+                                   && ! kEngine->isLoadingProject();
+
+        if (! externalProcess)
+        {
+            if (fProcess == nullptr)
+            {
+                fProcess = new ChildProcess();
+            }
+            else if (fProcess->isRunning())
+            {
+                carla_stderr("CarlaPluginJackThread::run() - already running");
+            }
+
+            String name(kPlugin->getName());
+            String filename(kPlugin->getFilename());
+
+            if (name.isEmpty())
+                name = "(none)";
+
+            CARLA_SAFE_ASSERT_RETURN(filename.isNotEmpty(),);
+
+            StringArray arguments;
+
+            // binary
+            arguments.addTokens(filename, true);
+
             const EngineOptions& options(kEngine->getOptions());
 
             char winIdStr[STR_MAX+1];
@@ -122,57 +326,90 @@ protected:
             CarlaString ldpreload;
 #ifdef HAVE_X11
             ldpreload = (CarlaString(options.binaryDir)
-                      + "/libcarla_interposer-jack-x11.so");
+                    + "/libcarla_interposer-jack-x11.so");
 #endif
 
             const ScopedEngineEnvironmentLocker _seel(kEngine);
 
-            const ScopedEnvVar sev2("LD_LIBRARY_PATH", libjackdir.buffer());
-            const ScopedEnvVar sev1("LD_PRELOAD", ldpreload.isNotEmpty() ? ldpreload.buffer() : nullptr);
+            const CarlaScopedEnvVar sev2("LD_LIBRARY_PATH", libjackdir.buffer());
+            const CarlaScopedEnvVar sev1("LD_PRELOAD", ldpreload.isNotEmpty() ? ldpreload.buffer() : nullptr);
+#ifdef HAVE_LIBLO
+            const CarlaScopedEnvVar sev3("NSM_URL", lo_server_get_url(fOscServer));
+#endif
 
             if (kPlugin->getHints() & PLUGIN_HAS_CUSTOM_UI)
                 carla_setenv("CARLA_FRONTEND_WIN_ID", winIdStr);
             else
                 carla_unsetenv("CARLA_FRONTEND_WIN_ID");
 
-            carla_setenv("CARLA_LIBJACK_SETUP", fNumPorts.buffer());
+            carla_setenv("CARLA_LIBJACK_SETUP", fSetupLabel.buffer());
             carla_setenv("CARLA_SHM_IDS", fShmIds.buffer());
 
-            started = fProcess->start(arguments);
-        }
-
-        if (! started)
-        {
-            carla_stdout("failed!");
-            fProcess = nullptr;
-            return;
-        }
-
-        for (; fProcess->isRunning() && ! shouldThreadExit();)
-            carla_msleep(50);
-
-        // we only get here if bridge crashed or thread asked to exit
-        if (fProcess->isRunning() && shouldThreadExit())
-        {
-            fProcess->waitForProcessToFinish(2000);
-
-            if (fProcess->isRunning())
+            if (! fProcess->start(arguments))
             {
-                carla_stdout("CarlaPluginJackThread::run() - application refused to close, force kill now");
-                fProcess->kill();
+                carla_stdout("failed!");
+                fProcess = nullptr;
+                return;
             }
         }
-        else
-        {
-            // forced quit, may have crashed
-            if (fProcess->getExitCode() != 0 /*|| fProcess->exitStatus() == QProcess::CrashExit*/)
-            {
-                carla_stderr("CarlaPluginJackThread::run() - application crashed");
 
-                CarlaString errorString("Plugin '" + CarlaString(kPlugin->getName()) + "' has crashed!\n"
-                                        "Saving now will lose its current settings.\n"
-                                        "Please remove this plugin, and not rely on it from this point.");
-                kEngine->callback(CarlaBackend::ENGINE_CALLBACK_ERROR, kPlugin->getId(), 0, 0, 0.0f, errorString);
+        for (; (externalProcess || fProcess->isRunning()) && ! shouldThreadExit();)
+        {
+#ifdef HAVE_LIBLO
+            if (sessionManager == LIBJACK_SESSION_MANAGER_NSM)
+            {
+                lo_server_recv_noblock(fOscServer, 50);
+            }
+            else
+#endif
+            {
+                carla_msleep(50);
+            }
+        }
+
+#ifdef HAVE_LIBLO
+        if (sessionManager == LIBJACK_SESSION_MANAGER_NSM)
+        {
+            lo_server_free(fOscServer);
+            fOscServer = nullptr;
+
+            if (fOscClientAddress != nullptr)
+            {
+                lo_address_free(fOscClientAddress);
+                fOscClientAddress = nullptr;
+            }
+        }
+#endif
+
+        if (! externalProcess)
+        {
+            // we only get here if bridge crashed or thread asked to exit
+            if (fProcess->isRunning() && shouldThreadExit())
+            {
+                fProcess->waitForProcessToFinish(2000);
+
+                if (fProcess->isRunning())
+                {
+                    carla_stdout("CarlaPluginJackThread::run() - application refused to close, force kill now");
+                    fProcess->kill();
+                }
+            }
+            else
+            {
+                // forced quit, may have crashed
+                if (fProcess->getExitCode() != 0 /*|| fProcess->exitStatus() == QProcess::CrashExit*/)
+                {
+                    carla_stderr("CarlaPluginJackThread::run() - application crashed");
+
+                    CarlaString errorString("Plugin '" + CarlaString(kPlugin->getName()) + "' has crashed!\n"
+                                            "Saving now will lose its current settings.\n"
+                                            "Please remove this plugin, and not rely on it from this point.");
+                    kEngine->callback(true, true,
+                                    ENGINE_CALLBACK_ERROR,
+                                    kPlugin->getId(),
+                                    0, 0, 0, 0.0f,
+                                    errorString);
+                }
             }
         }
 
@@ -184,9 +421,44 @@ private:
     CarlaPlugin* const kPlugin;
 
     CarlaString fShmIds;
-    CarlaString fNumPorts;
+    CarlaString fSetupLabel;
 
-    ScopedPointer<ChildProcess> fProcess;
+#ifdef HAVE_LIBLO
+    lo_address fOscClientAddress;
+    lo_server  fOscServer;
+
+    struct ProjectData {
+        CarlaString appName;
+        CarlaString path;
+        CarlaString display;
+        CarlaString clientName;
+
+        ProjectData()
+            : appName(),
+              path(),
+              display(),
+              clientName() {}
+
+        bool init(const char* const engineProjectFilename, const char* const uniqueCodeID)
+        {
+            CARLA_SAFE_ASSERT_RETURN(engineProjectFilename != nullptr && engineProjectFilename[0] != '\0', false);
+            CARLA_SAFE_ASSERT_RETURN(uniqueCodeID != nullptr && uniqueCodeID[0] != '\0', false);
+            CARLA_SAFE_ASSERT_RETURN(appName.isNotEmpty(), false);
+
+            const File file(File(engineProjectFilename).withFileExtension(uniqueCodeID));
+
+            path = file.getFullPathName().toRawUTF8();
+            display = file.getFileNameWithoutExtension().toRawUTF8();
+            clientName = appName + "." + uniqueCodeID;
+
+            return true;
+        }
+
+        CARLA_DECLARE_NON_COPY_STRUCT(ProjectData)
+    } fProject;
+#endif
+
+    CarlaScopedPointer<ChildProcess> fProcess;
 
     CARLA_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(CarlaPluginJackThread)
 };
@@ -205,7 +477,7 @@ public:
           fProcCanceled(false),
           fBufferSize(engine->getBufferSize()),
           fProcWaitTime(0),
-          fLastPingTime(-1),
+          fSetupHints(0x0),
           fBridgeThread(engine, this),
           fShmAudioPool(),
           fShmRtClientControl(),
@@ -301,25 +573,27 @@ public:
         return fInfo.optionsAvailable;
     }
 
-    void getLabel(char* const strBuf) const noexcept override
+    bool getLabel(char* const strBuf) const noexcept override
     {
         std::strncpy(strBuf, fInfo.setupLabel, STR_MAX);
+        return true;
     }
 
-    void getMaker(char* const strBuf) const noexcept override
+    bool getMaker(char* const) const noexcept override
     {
-        nullStrBuf(strBuf);
+        return false;
     }
 
-    void getCopyright(char* const strBuf) const noexcept override
+    bool getCopyright(char* const) const noexcept override
     {
-        nullStrBuf(strBuf);
+        return false;
     }
 
-    void getRealName(char* const strBuf) const noexcept override
+    bool getRealName(char* const strBuf) const noexcept override
     {
         // FIXME
         std::strncpy(strBuf, "Carla's libjack", STR_MAX);
+        return true;
     }
 
     // -------------------------------------------------------------------
@@ -327,6 +601,13 @@ public:
 
     void prepareForSave() noexcept override
     {
+#ifdef HAVE_LIBLO
+        if (fInfo.setupLabel.length() == 6)
+            setupUniqueProjectID();
+
+        fBridgeThread.nsmSave(fInfo.setupLabel);
+#endif
+
         {
             const CarlaMutexLocker _cml(fShmNonRtClientControl.mutex);
 
@@ -403,6 +684,10 @@ public:
             CARLA_SAFE_ASSERT_RETURN(restartBridgeThread(),);
         }
 
+#ifdef HAVE_LIBLO
+        fBridgeThread.nsmShowGui(yesNo);
+#endif
+
         const CarlaMutexLocker _cml(fShmNonRtClientControl.mutex);
 
         fShmNonRtClientControl.writeOpcode(yesNo ? kPluginBridgeNonRtClientShowUI : kPluginBridgeNonRtClientHideUI);
@@ -426,14 +711,6 @@ public:
             try {
                 handleNonRtData();
             } CARLA_SAFE_EXCEPTION("handleNonRtData");
-
-            if (fLastPingTime > 0 && Time::currentTimeMillis() > fLastPingTime + 30000)
-            {
-                carla_stderr("Did not receive ping message from server for 30 secs, closing...");
-                // TODO
-                //threadShouldExit();
-                //callback(ENGINE_CALLBACK_QUIT, 0, 0, 0, 0.0f, nullptr);
-            }
         }
         else if (fInitiated)
         {
@@ -640,7 +917,8 @@ public:
         } CARLA_SAFE_EXCEPTION("deactivate - waitForClient");
     }
 
-    void process(const float** const audioIn, float** const audioOut, const float**, float**, const uint32_t frames) override
+    void process(const float* const* const audioIn, float** const audioOut, 
+                 const float* const*, float**, const uint32_t frames) override
     {
         // --------------------------------------------------------------------------------------------------------
         // Check if active
@@ -732,13 +1010,13 @@ public:
                             if (MIDI_IS_CONTROL_BREATH_CONTROLLER(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_DRYWET) != 0)
                             {
                                 value = ctrlEvent.value;
-                                setDryWetRT(value);
+                                setDryWetRT(value, true);
                             }
 
                             if (MIDI_IS_CONTROL_CHANNEL_VOLUME(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_VOLUME) != 0)
                             {
                                 value = ctrlEvent.value*127.0f/100.0f;
-                                setVolumeRT(value);
+                                setVolumeRT(value, true);
                             }
 
                             if (MIDI_IS_CONTROL_BALANCE(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_BALANCE) != 0)
@@ -762,8 +1040,8 @@ public:
                                     right = 1.0f;
                                 }
 
-                                setBalanceLeftRT(left);
-                                setBalanceRightRT(right);
+                                setBalanceLeftRT(left, true);
+                                setBalanceRightRT(right, true);
                             }
                         }
 #endif
@@ -808,7 +1086,7 @@ public:
                             if (event.channel == pData->ctrlChannel && ! allNotesOffSent)
                             {
                                 allNotesOffSent = true;
-                                sendMidiAllNotesOffToCallback();
+                                postponeRtAllNotesOff();
                             }
 #endif
 
@@ -858,9 +1136,22 @@ public:
                     fShmRtClientControl.commitWrite();
 
                     if (status == MIDI_STATUS_NOTE_ON)
-                        pData->postponeRtEvent(kPluginPostRtEventNoteOn, event.channel, midiData[1], midiData[2]);
+                    {
+                        pData->postponeRtEvent(kPluginPostRtEventNoteOn,
+                                               true,
+                                               event.channel,
+                                               midiData[1],
+                                               midiData[2],
+                                               0.0f);
+                    }
                     else if (status == MIDI_STATUS_NOTE_OFF)
-                        pData->postponeRtEvent(kPluginPostRtEventNoteOff, event.channel, midiData[1], 0.0f);
+                    {
+                        pData->postponeRtEvent(kPluginPostRtEventNoteOff,
+                                               true,
+                                               event.channel,
+                                               midiData[1],
+                                               0, 0.0f);
+                    }
                 } break;
                 }
             }
@@ -911,7 +1202,7 @@ public:
         } // End of Control and MIDI Output
     }
 
-    bool processSingle(const float** const audioIn, float** const audioOut, const uint32_t frames)
+    bool processSingle(const float* const* const audioIn, float** const audioOut, const uint32_t frames)
     {
         CARLA_SAFE_ASSERT_RETURN(! fTimedError, false);
         CARLA_SAFE_ASSERT_RETURN(frames > 0, false);
@@ -1132,9 +1423,6 @@ public:
                 carla_debug("CarlaPluginJack::handleNonRtData() - got opcode: %s", PluginBridgeNonRtServerOpcode2str(opcode));
             }
 //#endif
-            if (opcode != kPluginBridgeNonRtServerNull && fLastPingTime > 0)
-                fLastPingTime = Time::currentTimeMillis();
-
             switch (opcode)
             {
             case kPluginBridgeNonRtServerNull:
@@ -1160,6 +1448,7 @@ public:
             case kPluginBridgeNonRtServerProgramName:
             case kPluginBridgeNonRtServerMidiProgramData:
             case kPluginBridgeNonRtServerSetCustomData:
+            case kPluginBridgeNonRtServerVersion:
                 break;
 
             case kPluginBridgeNonRtServerSetChunkDataFile:
@@ -1183,7 +1472,11 @@ public:
                 break;
 
             case kPluginBridgeNonRtServerUiClosed:
-                pData->engine->callback(ENGINE_CALLBACK_UI_STATE_CHANGED, pData->id, 0, 0, 0.0f, nullptr);
+                pData->engine->callback(true, true,
+                                        ENGINE_CALLBACK_UI_STATE_CHANGED,
+                                        pData->id,
+                                        0,
+                                        0, 0, 0.0f, nullptr);
                 break;
 
             case kPluginBridgeNonRtServerError: {
@@ -1195,7 +1488,7 @@ public:
 
                 if (fInitiated)
                 {
-                    pData->engine->callback(ENGINE_CALLBACK_ERROR, pData->id, 0, 0, 0.0f, error);
+                    pData->engine->callback(true, true, ENGINE_CALLBACK_ERROR, pData->id, 0, 0, 0, 0.0f, error);
 
                     // just in case
                     pData->engine->setLastError(error);
@@ -1249,7 +1542,7 @@ public:
         // ---------------------------------------------------------------
         // check setup
 
-        if (std::strlen(label) != 6)
+        if (std::strlen(label) < 6)
         {
             pData->engine->setLastError("invalid application setup received");
             return false;
@@ -1269,7 +1562,16 @@ public:
 
         fInfo.setupLabel = label;
 
-        const int setupHints = label[5] - '0';
+        // ---------------------------------------------------------------
+        // set project unique id
+
+        if (label[6] == '\0')
+            setupUniqueProjectID();
+
+        // ---------------------------------------------------------------
+        // set icon
+
+        pData->iconName = carla_strdup_safe("application");
 
         // ---------------------------------------------------------------
         // set info
@@ -1319,6 +1621,8 @@ public:
         // ---------------------------------------------------------------
         // setup hints and options
 
+        fSetupHints = static_cast<uint>(label[5] - '0');
+
         // FIXME dryWet broken
         pData->hints  = PLUGIN_IS_BRIDGE | PLUGIN_OPTION_FIXED_BUFFERS;
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
@@ -1326,7 +1630,7 @@ public:
 #endif
         //fInfo.optionsAvailable = optionAv;
 
-        if (setupHints & 0x1)
+        if (fSetupHints & LIBJACK_FLAG_CONTROL_WINDOW)
             pData->hints |= PLUGIN_HAS_CUSTOM_UI;
 
         // ---------------------------------------------------------------
@@ -1341,7 +1645,7 @@ public:
             std::strncpy(shmIdsStr+6*2, &fShmNonRtClientControl.filename[fShmNonRtClientControl.filename.length()-6], 6);
             std::strncpy(shmIdsStr+6*3, &fShmNonRtServerControl.filename[fShmNonRtServerControl.filename.length()-6], 6);
 
-            fBridgeThread.setData(shmIdsStr, label);
+            fBridgeThread.setData(shmIdsStr, fInfo.setupLabel);
         }
 
         if (! restartBridgeThread())
@@ -1372,8 +1676,7 @@ private:
     bool fProcCanceled;
     uint fBufferSize;
     uint fProcWaitTime;
-
-    int64_t fLastPingTime;
+    uint fSetupHints;
 
     CarlaPluginJackThread fBridgeThread;
 
@@ -1408,15 +1711,21 @@ private:
 
         if (wasActive)
         {
-#if defined(HAVE_LIBLO) && ! defined(BUILD_BRIDGE)
-            if (pData->engine->isOscControlRegistered())
-                pData->engine->oscSend_control_set_parameter_value(pData->id, PARAMETER_ACTIVE, 0.0f);
-            pData->engine->callback(ENGINE_CALLBACK_PARAMETER_VALUE_CHANGED, pData->id, PARAMETER_ACTIVE, 0, 0.0f, nullptr);
-#endif
+            pData->engine->callback(true, true,
+                                    ENGINE_CALLBACK_PARAMETER_VALUE_CHANGED,
+                                    pData->id,
+                                    PARAMETER_ACTIVE,
+                                    0, 0,
+                                    0.0f,
+                                    nullptr);
         }
 
         if (pData->hints & PLUGIN_HAS_CUSTOM_UI)
-            pData->engine->callback(ENGINE_CALLBACK_UI_STATE_CHANGED, pData->id, 0, 0, 0.0f, nullptr);
+            pData->engine->callback(true, true,
+                                    ENGINE_CALLBACK_UI_STATE_CHANGED,
+                                    pData->id,
+                                    0,
+                                    0, 0, 0.0f, nullptr);
     }
 
     void resizeAudioPool(const uint32_t bufferSize)
@@ -1430,16 +1739,45 @@ private:
         waitForClient("resize-pool", 5000);
     }
 
-    void waitForClient(const char* const action, const uint msecs)
+    void setupUniqueProjectID()
     {
-        CARLA_SAFE_ASSERT_RETURN(! fTimedOut,);
-        CARLA_SAFE_ASSERT_RETURN(! fTimedError,);
+        const char* const engineProjectFilename = pData->engine->getCurrentProjectFilename();
+        carla_stdout("setupUniqueProjectID %s", engineProjectFilename);
 
-        if (fShmRtClientControl.waitForClient(msecs))
+        if (engineProjectFilename == nullptr || engineProjectFilename[0] == '\0')
             return;
 
-        fTimedOut = true;
-        carla_stderr2("waitForClient(%s) timed out", action);
+        const File file(engineProjectFilename);
+        CARLA_SAFE_ASSERT_RETURN(file.existsAsFile(),);
+        CARLA_SAFE_ASSERT_RETURN(file.getFileExtension().isNotEmpty(),);
+
+        char code[6];
+        code[5] = '\0';
+
+        for (;;)
+        {
+            static const char* const kValidChars =
+                "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                "abcdefghijklmnopqrstuvwxyz"
+                "0123456789";
+
+            static const size_t kValidCharsLen(std::strlen(kValidChars)-1U);
+
+            code[0] = kValidChars[safe_rand(kValidCharsLen)];
+            code[1] = kValidChars[safe_rand(kValidCharsLen)];
+            code[2] = kValidChars[safe_rand(kValidCharsLen)];
+            code[3] = kValidChars[safe_rand(kValidCharsLen)];
+            code[4] = kValidChars[safe_rand(kValidCharsLen)];
+
+            const File newFile(file.withFileExtension(code));
+
+            if (newFile.existsAsFile())
+                continue;
+
+            fInfo.setupLabel += code;
+            carla_stdout("new label %s", fInfo.setupLabel.buffer());
+            break;
+        }
     }
 
     bool restartBridgeThread()
@@ -1460,7 +1798,7 @@ private:
 
         // initial values
         fShmNonRtClientControl.writeOpcode(kPluginBridgeNonRtClientVersion);
-        fShmNonRtClientControl.writeUInt(CARLA_PLUGIN_BRIDGE_API_VERSION);
+        fShmNonRtClientControl.writeUInt(CARLA_PLUGIN_BRIDGE_API_VERSION_CURRENT);
 
         fShmNonRtClientControl.writeUInt(static_cast<uint32_t>(sizeof(BridgeRtClientData)));
         fShmNonRtClientControl.writeUInt(static_cast<uint32_t>(sizeof(BridgeNonRtClientData)));
@@ -1487,22 +1825,42 @@ private:
 
         fBridgeThread.startThread();
 
-        fLastPingTime = Time::currentTimeMillis();
-        CARLA_SAFE_ASSERT(fLastPingTime > 0);
-
-        static bool sFirstInit = true;
-
-        int64_t timeoutEnd = 5000;
-
-        if (sFirstInit)
-            timeoutEnd *= 2;
-        sFirstInit = false;
-
+        const bool needsCancelableAction = ! pData->engine->isLoadingProject();
         const bool needsEngineIdle = pData->engine->getType() != kEngineTypePlugin;
 
-        for (; Time::currentTimeMillis() < fLastPingTime + timeoutEnd && fBridgeThread.isThreadRunning();)
+        CarlaString actionName;
+
+        if (needsCancelableAction)
         {
-            pData->engine->callback(ENGINE_CALLBACK_IDLE, 0, 0, 0, 0.0f, nullptr);
+            if (fSetupHints & LIBJACK_FLAG_EXTERNAL_START)
+            {
+                const EngineOptions& options(pData->engine->getOptions());
+                CarlaString binaryDir(options.binaryDir);
+
+                char* const hwVars = fBridgeThread.getEnvVarsToExport();
+
+                actionName  = "Waiting for external JACK application start, please use the following environment variables:\n";
+                actionName += hwVars;
+
+                delete[] hwVars;
+            }
+            else
+            {
+                actionName = "Loading JACK application";
+            }
+
+            pData->engine->setActionCanceled(false);
+            pData->engine->callback(true, true,
+                                    ENGINE_CALLBACK_CANCELABLE_ACTION,
+                                    pData->id,
+                                    1,
+                                    0, 0, 0.0f,
+                                    actionName.buffer());
+        }
+
+        for (;fBridgeThread.isThreadRunning();)
+        {
+            pData->engine->callback(true, false, ENGINE_CALLBACK_IDLE, 0, 0, 0, 0, 0.0f, nullptr);
 
             if (needsEngineIdle)
                 pData->engine->idle();
@@ -1511,13 +1869,21 @@ private:
 
             if (fInitiated)
                 break;
-            if (pData->engine->isAboutToClose())
+            if (pData->engine->isAboutToClose() || pData->engine->wasActionCanceled())
                 break;
 
-            carla_msleep(20);
+            carla_msleep(5);
         }
 
-        fLastPingTime = -1;
+        if (needsCancelableAction)
+        {
+            pData->engine->callback(true, true,
+                                    ENGINE_CALLBACK_CANCELABLE_ACTION,
+                                    pData->id,
+                                    0,
+                                    0, 0, 0.0f,
+                                    actionName.buffer());
+        }
 
         if (fInitError || ! fInitiated)
         {
@@ -1533,6 +1899,18 @@ private:
         return true;
     }
 
+    void waitForClient(const char* const action, const uint msecs)
+    {
+        CARLA_SAFE_ASSERT_RETURN(! fTimedOut,);
+        CARLA_SAFE_ASSERT_RETURN(! fTimedError,);
+
+        if (fShmRtClientControl.waitForClient(msecs))
+            return;
+
+        fTimedOut = true;
+        carla_stderr2("waitForClient(%s) timed out", action);
+    }
+
     CARLA_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(CarlaPluginJack)
 };
 
