@@ -1,6 +1,6 @@
 /*
  * Carla VST Plugin
- * Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,9 +18,15 @@
 #include "CarlaPluginInternal.hpp"
 #include "CarlaEngine.hpp"
 
+#if defined(USING_JUCE) && (defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN))
+# define USE_JUCE_FOR_VST2
+#endif
+
+#include "CarlaBackendUtils.hpp"
+#include "CarlaMathUtils.hpp"
+#include "CarlaScopeUtils.hpp"
 #include "CarlaVstUtils.hpp"
 
-#include "CarlaMathUtils.hpp"
 #include "CarlaPluginUI.hpp"
 
 #ifdef CARLA_OS_MAC
@@ -270,39 +276,42 @@ public:
         return fEffect->getParameter(fEffect, static_cast<int32_t>(parameterId));
     }
 
-    void getLabel(char* const strBuf) const noexcept override
+    bool getLabel(char* const strBuf) const noexcept override
     {
-        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr,);
+        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr, false);
 
         strBuf[0] = '\0';
         dispatcher(effGetProductString, 0, 0, strBuf);
+        return true;
     }
 
-    void getMaker(char* const strBuf) const noexcept override
+    bool getMaker(char* const strBuf) const noexcept override
     {
-        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr,);
+        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr, false);
 
         strBuf[0] = '\0';
         dispatcher(effGetVendorString, 0, 0, strBuf);
+        return true;
     }
 
-    void getCopyright(char* const strBuf) const noexcept override
+    bool getCopyright(char* const strBuf) const noexcept override
     {
-        getMaker(strBuf);
+        return getMaker(strBuf);
     }
 
-    void getRealName(char* const strBuf) const noexcept override
+    bool getRealName(char* const strBuf) const noexcept override
     {
-        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr,);
+        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr, false);
 
         strBuf[0] = '\0';
         dispatcher(effGetEffectName, 0, 0, strBuf);
+        return true;
     }
 
-    void getParameterName(const uint32_t parameterId, char* const strBuf) const noexcept override
+    bool getParameterName(const uint32_t parameterId, char* const strBuf) const noexcept override
     {
-        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr,);
-        CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count,);
+        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr, false);
+        CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count, false);
 
         strBuf[0] = '\0';
 
@@ -313,32 +322,36 @@ public:
         {
             std::strncpy(strBuf, prop.label, 64);
             strBuf[64] = '\0';
-            return;
+            return true;
         }
 
         strBuf[0] = '\0';
         dispatcher(effGetParamName, static_cast<int32_t>(parameterId), 0, strBuf);
+        return true;
     }
 
-    void getParameterText(const uint32_t parameterId, char* const strBuf) noexcept override
+    bool getParameterText(const uint32_t parameterId, char* const strBuf) noexcept override
     {
-        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr,);
-        CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count,);
+        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr, false);
+        CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count, false);
 
         strBuf[0] = '\0';
         dispatcher(effGetParamDisplay, static_cast<int32_t>(parameterId), 0, strBuf);
 
         if (strBuf[0] == '\0')
-            std::snprintf(strBuf, STR_MAX, "%f", getParameterValue(parameterId));
+            std::snprintf(strBuf, STR_MAX, "%.12g", static_cast<double>(getParameterValue(parameterId)));
+
+        return true;
     }
 
-    void getParameterUnit(const uint32_t parameterId, char* const strBuf) const noexcept override
+    bool getParameterUnit(const uint32_t parameterId, char* const strBuf) const noexcept override
     {
-        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr,);
-        CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count,);
+        CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr, false);
+        CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count, false);
 
         strBuf[0] = '\0';
         dispatcher(effGetParamLabel, static_cast<int32_t>(parameterId), 0, strBuf);
+        return true;
     }
 
     // -------------------------------------------------------------------
@@ -375,7 +388,7 @@ public:
         CarlaPlugin::setParameterValue(parameterId, fixedValue, sendGui, sendOsc, sendCallback);
     }
 
-    void setParameterValueRT(const uint32_t parameterId, const float value) noexcept override
+    void setParameterValueRT(const uint32_t parameterId, const float value, const bool sendCallbackLater) noexcept override
     {
         CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr,);
         CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count,);
@@ -383,7 +396,7 @@ public:
         const float fixedValue(pData->param.getFixedValue(parameterId, value));
         fEffect->setParameter(fEffect, static_cast<int32_t>(parameterId), fixedValue);
 
-        CarlaPlugin::setParameterValueRT(parameterId, fixedValue);
+        CarlaPlugin::setParameterValueRT(parameterId, fixedValue, sendCallbackLater);
     }
 
     void setChunkData(const void* const data, const std::size_t dataSize) override
@@ -393,7 +406,7 @@ public:
         CARLA_SAFE_ASSERT_RETURN(data != nullptr,);
         CARLA_SAFE_ASSERT_RETURN(dataSize > 0,);
 
-        if (loadOldSaveFormat(data, dataSize))
+        if (loadJuceSaveFormat(data, dataSize))
             return;
 
         if (fLastChunk != nullptr)
@@ -406,7 +419,7 @@ public:
 
         {
             const ScopedSingleProcessLocker spl(this, true);
-            const ScopedValueSetter<pthread_t> svs(fChangingValuesThread, pthread_self(), kNullThread);
+            const CarlaScopedValueSetter<pthread_t> svs(fChangingValuesThread, pthread_self(), kNullThread);
 
             dispatcher(effSetChunk, 0 /* bank */, static_cast<intptr_t>(dataSize), fLastChunk);
         }
@@ -414,12 +427,7 @@ public:
         // simulate an updateDisplay callback
         handleAudioMasterCallback(audioMasterUpdateDisplay, 0, 0, nullptr, 0.0f);
 
-#if defined(HAVE_LIBLO) && ! defined(BUILD_BRIDGE)
-        const bool sendOsc(pData->engine->isOscControlRegistered());
-#else
-        const bool sendOsc(false);
-#endif
-        pData->updateParameterValues(this, sendOsc, true, false);
+        pData->updateParameterValues(this, true, true, false);
     }
 
     void setProgram(const int32_t index, const bool sendGui, const bool sendOsc, const bool sendCallback, const bool doingInit) noexcept override
@@ -436,7 +444,7 @@ public:
 
             {
                 const ScopedSingleProcessLocker spl(this, (sendGui || sendOsc || sendCallback));
-                const ScopedValueSetter<pthread_t> svs(fChangingValuesThread, pthread_self(), kNullThread);
+                const CarlaScopedValueSetter<pthread_t> svs(fChangingValuesThread, pthread_self(), kNullThread);
 
                 try {
                     dispatcher(effSetProgram, 0, index);
@@ -451,7 +459,7 @@ public:
         CarlaPlugin::setProgram(index, sendGui, sendOsc, sendCallback, doingInit);
     }
 
-    void setProgramRT(const uint32_t uindex) noexcept override
+    void setProgramRT(const uint32_t uindex, const bool sendCallbackLater) noexcept override
     {
         CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr,);
         CARLA_SAFE_ASSERT_RETURN(uindex < pData->prog.count,);
@@ -468,7 +476,7 @@ public:
             dispatcher(effEndSetProgram);
         } CARLA_SAFE_EXCEPTION("effEndSetProgram");
 
-        CarlaPlugin::setProgramRT(uindex);
+        CarlaPlugin::setProgramRT(uindex, sendCallbackLater);
     }
 
     // -------------------------------------------------------------------
@@ -504,7 +512,12 @@ public:
 #endif
 
                 if (fUI.window == nullptr)
-                    return pData->engine->callback(ENGINE_CALLBACK_UI_STATE_CHANGED, pData->id, -1, 0, 0.0f, msg);
+                    return pData->engine->callback(true, true,
+                                                   ENGINE_CALLBACK_UI_STATE_CHANGED,
+                                                   pData->id,
+                                                   -1,
+                                                   0, 0, 0.0f,
+                                                   msg);
 
                 fUI.window->setTitle(uiTitle.buffer());
 
@@ -537,7 +550,12 @@ public:
                     fUI.window = nullptr;
 
                     carla_stderr2("Plugin refused to open its own UI");
-                    return pData->engine->callback(ENGINE_CALLBACK_UI_STATE_CHANGED, pData->id, -1, 0, 0.0f, "Plugin refused to open its own UI");
+                    return pData->engine->callback(true, true,
+                                                   ENGINE_CALLBACK_UI_STATE_CHANGED,
+                                                   pData->id,
+                                                   -1,
+                                                   0, 0, 0.0f,
+                                                   "Plugin refused to open its own UI");
                 }
             }
 
@@ -557,7 +575,7 @@ public:
     {
         if (fNeedIdle)
         {
-            const ScopedValueSetter<pthread_t> svs(fIdleThread, pthread_self(), kNullThread);
+            const CarlaScopedValueSetter<pthread_t> svs(fIdleThread, pthread_self(), kNullThread);
             dispatcher(effIdle);
         }
 
@@ -590,7 +608,7 @@ public:
 
         // Safely disable plugin for reload
         const ScopedDisabler sd(this);
-        const ScopedValueSetter<bool> svs(fIsInitializing, fIsInitializing, true);
+        const CarlaScopedValueSetter<bool> svs(fIsInitializing, fIsInitializing, false);
 
         if (pData->active)
             deactivate();
@@ -823,8 +841,13 @@ public:
             pData->param.data[j].hints |= PARAMETER_USES_CUSTOM_TEXT;
 
             if ((pData->hints & PLUGIN_USES_OLD_VSTSDK) != 0 || dispatcher(effCanBeAutomated, ij) == 1)
+            {
                 pData->param.data[j].hints |= PARAMETER_IS_AUTOMABLE;
 
+                if ((prop.flags & (kVstParameterIsSwitch|kVstParameterUsesIntStep)) == 0x0)
+                    pData->param.data[j].hints |= PARAMETER_CAN_BE_CV_CONTROLLED;
+            }
+
             // no such thing as VST default parameters
             def = fEffect->getParameter(fEffect, ij);
 
@@ -855,6 +878,9 @@ public:
             portName.truncate(portNameSize);
 
             pData->event.portIn = (CarlaEngineEventPort*)pData->client->addPort(kEnginePortTypeEvent, portName, true, 0);
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+            pData->event.cvSourcePorts = pData->client->createCVSourcePorts();
+#endif
         }
 
         if (needsCtrlOut)
@@ -966,33 +992,22 @@ public:
             for (int32_t i=0; i < fEffect->numPrograms; ++i)
             {
                 char strBuf[STR_MAX+1] = { '\0' };
-                if (dispatcher(effGetProgramNameIndexed, i, 0, strBuf, 0.0f) != 1)
+                if (dispatcher(effGetProgramNameIndexed, i, 0, strBuf) != 1)
                 {
                     // program will be [re-]changed later
-                    dispatcher(effSetProgram, 0, i, nullptr, 0.0f);
-                    dispatcher(effGetProgramName, 0, 0, strBuf, 0.0f);
+                    dispatcher(effSetProgram, 0, i);
+                    dispatcher(effGetProgramName, 0, 0, strBuf);
                 }
                 pData->prog.names[i] = carla_strdup(strBuf);
             }
         }
 
-#if defined(HAVE_LIBLO) && ! defined(BUILD_BRIDGE)
-        // Update OSC Names
-        if (pData->engine->isOscControlRegistered() && pData->id < pData->engine->getCurrentPluginCount())
-        {
-            pData->engine->oscSend_control_set_program_count(pData->id, newCount);
-
-            for (uint32_t i=0; i < newCount; ++i)
-                pData->engine->oscSend_control_set_program_name(pData->id, i, pData->prog.names[i]);
-        }
-#endif
-
         if (doInit)
         {
             if (newCount > 0)
                 setProgram(0, false, false, false, true);
             else
-                dispatcher(effSetProgram);
+                dispatcher(effSetProgram, 0, 0);
         }
         else
         {
@@ -1037,10 +1052,10 @@ public:
             {
                 // Program was changed during update, re-set it
                 if (pData->prog.current >= 0)
-                    dispatcher(effSetProgram, 0, pData->prog.current, nullptr, 0.0f);
+                    dispatcher(effSetProgram, 0, pData->prog.current);
             }
 
-            pData->engine->callback(ENGINE_CALLBACK_RELOAD_PROGRAMS, pData->id, 0, 0, 0.0f, nullptr);
+            pData->engine->callback(true, true, ENGINE_CALLBACK_RELOAD_PROGRAMS, pData->id, 0, 0, 0, 0.0f, nullptr);
         }
     }
 
@@ -1083,9 +1098,13 @@ public:
         } catch(...) {}
     }
 
-    void process(const float** const audioIn, float** const audioOut, const float** const, float** const, const uint32_t frames) override
+    void process(const float* const* const audioIn,
+                 float** const audioOut,
+                 const float* const* const cvIn,
+                 float** const,
+                 const uint32_t frames) override
     {
-        const ScopedValueSetter<pthread_t> svs(fProcThread, pthread_self(), kNullThread);
+        const CarlaScopedValueSetter<pthread_t> svs(fProcThread, pthread_self(), kNullThread);
 
         // --------------------------------------------------------------------------------------------------------
         // Check if active
@@ -1169,8 +1188,8 @@ public:
             CARLA_SAFE_ASSERT_INT(timeInfo.bbt.bar > 0, timeInfo.bbt.bar);
             CARLA_SAFE_ASSERT_INT(timeInfo.bbt.beat > 0, timeInfo.bbt.beat);
 
-            const double ppqBar  = double(timeInfo.bbt.bar - 1) * timeInfo.bbt.beatsPerBar;
-            // const double ppqBeat = double(timeInfo.bbt.beat - 1);
+            const double ppqBar  = static_cast<double>(timeInfo.bbt.beatsPerBar) * (timeInfo.bbt.bar - 1);
+            // const double ppqBeat = static_cast<double>(timeInfo.bbt.beat - 1);
             // const double ppqTick = timeInfo.bbt.tick / timeInfo.bbt.ticksPerBeat;
 
             // PPQ Pos
@@ -1249,6 +1268,11 @@ public:
             uint32_t startTime  = 0;
             uint32_t timeOffset = 0;
 
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+            if (cvIn != nullptr && pData->event.cvSourcePorts != nullptr)
+                pData->event.cvSourcePorts->initPortBuffers(cvIn, frames, isSampleAccurate, pData->event.portIn);
+#endif
+
             for (uint32_t i=0, numEvents = pData->event.portIn->getEventCount(); i < numEvents; ++i)
             {
                 const EngineEvent& event(pData->event.portIn->getEvent(i));
@@ -1294,22 +1318,33 @@ public:
                         break;
 
                     case kEngineControlEventTypeParameter: {
+                        float value;
+
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+                        // non-midi
+                        if (event.channel == kEngineEventNonMidiChannel)
+                        {
+                            const uint32_t k = ctrlEvent.param;
+                            CARLA_SAFE_ASSERT_CONTINUE(k < pData->param.count);
+
+                            value = pData->param.getFinalUnnormalizedValue(k, ctrlEvent.value);
+                            setParameterValueRT(k, value, true);
+                            continue;
+                        }
+
                         // Control backend stuff
                         if (event.channel == pData->ctrlChannel)
                         {
-                            float value;
-
                             if (MIDI_IS_CONTROL_BREATH_CONTROLLER(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_DRYWET) != 0)
                             {
                                 value = ctrlEvent.value;
-                                setDryWetRT(value);
+                                setDryWetRT(value, true);
                             }
 
                             if (MIDI_IS_CONTROL_CHANNEL_VOLUME(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_VOLUME) != 0)
                             {
                                 value = ctrlEvent.value*127.0f/100.0f;
-                                setVolumeRT(value);
+                                setVolumeRT(value, true);
                             }
 
                             if (MIDI_IS_CONTROL_BALANCE(ctrlEvent.param) && (pData->hints & PLUGIN_CAN_BALANCE) != 0)
@@ -1333,8 +1368,8 @@ public:
                                     right = 1.0f;
                                 }
 
-                                setBalanceLeftRT(left);
-                                setBalanceRightRT(right);
+                                setBalanceLeftRT(left, true);
+                                setBalanceRightRT(right, true);
                             }
                         }
 #endif
@@ -1344,34 +1379,18 @@ public:
                         {
                             if (pData->param.data[k].midiChannel != event.channel)
                                 continue;
-                            if (pData->param.data[k].midiCC != ctrlEvent.param)
+                            if (pData->param.data[k].mappedControlIndex != ctrlEvent.param)
                                 continue;
                             if (pData->param.data[k].type != PARAMETER_INPUT)
                                 continue;
                             if ((pData->param.data[k].hints & PARAMETER_IS_AUTOMABLE) == 0)
                                 continue;
 
-                            float value;
-
-                            if (pData->param.data[k].hints & PARAMETER_IS_BOOLEAN)
-                            {
-                                value = (ctrlEvent.value < 0.5f) ? pData->param.ranges[k].min : pData->param.ranges[k].max;
-                            }
-                            else
-                            {
-                                if (pData->param.data[k].hints & PARAMETER_IS_LOGARITHMIC)
-                                    value = pData->param.ranges[k].getUnnormalizedLogValue(ctrlEvent.value);
-                                else
-                                    value = pData->param.ranges[k].getUnnormalizedValue(ctrlEvent.value);
-
-                                if (pData->param.data[k].hints & PARAMETER_IS_INTEGER)
-                                    value = std::rint(value);
-                            }
-
-                            setParameterValueRT(k, value);
+                            value = pData->param.getFinalUnnormalizedValue(k, ctrlEvent.value);
+                            setParameterValueRT(k, value, true);
                         }
 
-                        if ((pData->options & PLUGIN_OPTION_SEND_CONTROL_CHANGES) != 0 && ctrlEvent.param < MAX_MIDI_CONTROL)
+                        if ((pData->options & PLUGIN_OPTION_SEND_CONTROL_CHANGES) != 0 && ctrlEvent.param < MAX_MIDI_VALUE)
                         {
                             if (fMidiEventCount >= kPluginMaxMidiEvents*2)
                                 continue;
@@ -1396,15 +1415,23 @@ public:
                             if (fMidiEventCount >= kPluginMaxMidiEvents*2)
                                 continue;
 
-                            VstMidiEvent& vstMidiEvent(fMidiEvents[fMidiEventCount++]);
-                            carla_zeroStruct(vstMidiEvent);
+                            VstMidiEvent& vstMidiEvent_MSB(fMidiEvents[fMidiEventCount++]);
+                            carla_zeroStruct(vstMidiEvent_MSB);
+                            vstMidiEvent_MSB.type = kVstMidiType;
+                            vstMidiEvent_MSB.byteSize = kVstMidiEventSize;
+                            vstMidiEvent_MSB.deltaFrames = static_cast<int32_t>(isSampleAccurate ? startTime : event.time);
+                            vstMidiEvent_MSB.midiData[0] = char(MIDI_STATUS_CONTROL_CHANGE | (event.channel & MIDI_CHANNEL_BIT));
+                            vstMidiEvent_MSB.midiData[1] = MIDI_CONTROL_BANK_SELECT;
+                            vstMidiEvent_MSB.midiData[2] = 0;
 
-                            vstMidiEvent.type        = kVstMidiType;
-                            vstMidiEvent.byteSize    = kVstMidiEventSize;
-                            vstMidiEvent.deltaFrames = static_cast<int32_t>(isSampleAccurate ? startTime : eventTime);
-                            vstMidiEvent.midiData[0] = char(MIDI_STATUS_CONTROL_CHANGE | (event.channel & MIDI_CHANNEL_BIT));
-                            vstMidiEvent.midiData[1] = MIDI_CONTROL_BANK_SELECT;
-                            vstMidiEvent.midiData[2] = char(ctrlEvent.param);
+                            VstMidiEvent& vstMidiEvent_LSB(fMidiEvents[fMidiEventCount++]);
+                            carla_zeroStruct(vstMidiEvent_LSB);
+                            vstMidiEvent_LSB.type        = kVstMidiType;
+                            vstMidiEvent_LSB.byteSize    = kVstMidiEventSize;
+                            vstMidiEvent_LSB.deltaFrames = static_cast<int32_t>(isSampleAccurate ? startTime : eventTime);
+                            vstMidiEvent_LSB.midiData[0] = char(MIDI_STATUS_CONTROL_CHANGE | (event.channel & MIDI_CHANNEL_BIT));
+                            vstMidiEvent_LSB.midiData[1] = MIDI_CONTROL_BANK_SELECT__LSB;
+                            vstMidiEvent_LSB.midiData[2] = char(ctrlEvent.param);
                         }
                         break;
 
@@ -1413,7 +1440,7 @@ public:
                         {
                             if (ctrlEvent.param < pData->prog.count)
                             {
-                                setProgramRT(ctrlEvent.param);
+                                setProgramRT(ctrlEvent.param, true);
                                 break;
                             }
                         }
@@ -1457,7 +1484,7 @@ public:
                             if (event.channel == pData->ctrlChannel && ! allNotesOffSent)
                             {
                                 allNotesOffSent = true;
-                                sendMidiAllNotesOffToCallback();
+                                postponeRtAllNotesOff();
                             }
 #endif
 
@@ -1516,9 +1543,22 @@ public:
                     vstMidiEvent.midiData[2] = char(midiEvent.size >= 3 ? midiEvent.data[2] : 0);
 
                     if (status == MIDI_STATUS_NOTE_ON)
-                        pData->postponeRtEvent(kPluginPostRtEventNoteOn, event.channel, midiEvent.data[1], midiEvent.data[2]);
+                    {
+                        pData->postponeRtEvent(kPluginPostRtEventNoteOn,
+                                               true,
+                                               event.channel,
+                                               midiEvent.data[1],
+                                               midiEvent.data[2],
+                                               0.0f);
+                    }
                     else if (status == MIDI_STATUS_NOTE_OFF)
-                        pData->postponeRtEvent(kPluginPostRtEventNoteOff, event.channel, midiEvent.data[1], 0.0f);
+                    {
+                        pData->postponeRtEvent(kPluginPostRtEventNoteOff,
+                                               true,
+                                               event.channel,
+                                               midiEvent.data[1],
+                                               0, 0.0f);
+                    }
                 } break;
                 } // switch (event.type)
             }
@@ -1567,9 +1607,18 @@ public:
         } // End of MIDI Output
 
         fFirstActive = false;
+
+        // --------------------------------------------------------------------------------------------------------
+
+#ifdef BUILD_BRIDGE_ALTERNATIVE_ARCH
+        return;
+
+        // unused
+        (void)cvIn;
+#endif
     }
 
-    bool processSingle(const float** const inBuffer, float** const outBuffer, const uint32_t frames, const uint32_t timeOffset)
+    bool processSingle(const float* const* const inBuffer, float** const outBuffer, const uint32_t frames, const uint32_t timeOffset)
     {
         CARLA_SAFE_ASSERT_RETURN(frames > 0, false);
 
@@ -1813,7 +1862,11 @@ protected:
         carla_debug("CarlaPluginVST2::handlePluginUIClosed()");
 
         showCustomUI(false);
-        pData->engine->callback(ENGINE_CALLBACK_UI_STATE_CHANGED, pData->id, 0, 0, 0.0f, nullptr);
+        pData->engine->callback(true, true,
+                                ENGINE_CALLBACK_UI_STATE_CHANGED,
+                                pData->id,
+                                0,
+                                0, 0, 0.0f, nullptr);
     }
 
     void handlePluginUIResized(const uint width, const uint height) override
@@ -1832,7 +1885,8 @@ protected:
         CARLA_SAFE_ASSERT_RETURN(fEffect != nullptr, 0);
 #ifdef DEBUG
         if (opcode != effIdle && opcode != effEditIdle && opcode != effProcessEvents)
-            carla_debug("CarlaPluginVST2::dispatcher(%02i:%s, %i, " P_INTPTR ", %p, %f)", opcode, vstEffectOpcode2str(opcode), index, value, ptr, opt);
+            carla_debug("CarlaPluginVST2::dispatcher(%02i:%s, %i, " P_INTPTR ", %p, %f)",
+                        opcode, vstEffectOpcode2str(opcode), index, value, ptr, static_cast<double>(opt));
 #endif
 
         try {
@@ -1844,7 +1898,8 @@ protected:
     {
 #ifdef DEBUG
         if (opcode != audioMasterGetTime)
-            carla_debug("CarlaPluginVST2::handleAudioMasterCallback(%02i:%s, %i, " P_INTPTR ", %p, %f)", opcode, vstMasterOpcode2str(opcode), index, value, ptr, opt);
+            carla_debug("CarlaPluginVST2::handleAudioMasterCallback(%02i:%s, %i, " P_INTPTR ", %p, %f)",
+                        opcode, vstMasterOpcode2str(opcode), index, value, ptr, static_cast<double>(opt));
 #endif
 
         intptr_t ret = 0;
@@ -1879,19 +1934,29 @@ protected:
             else if (pthread_equal(thisThread, fProcThread))
             {
                 CARLA_SAFE_ASSERT(fIsProcessing);
-                pData->postponeRtEvent(kPluginPostRtEventParameterChange, index, 0, fixedValue);
+                pData->postponeRtEvent(kPluginPostRtEventParameterChange, true, index, 0, 0, fixedValue);
             }
             // Called from effSetChunk or effSetProgram
             else if (pthread_equal(thisThread, fChangingValuesThread))
             {
                 carla_debug("audioMasterAutomate called while setting state");
-                pData->postponeRtEvent(kPluginPostRtEventParameterChange, index, 0, fixedValue);
+                pData->postponeRtEvent(kPluginPostRtEventParameterChange, true, index, 0, 0, fixedValue);
             }
             // Called from effIdle
             else if (pthread_equal(thisThread, fIdleThread))
             {
                 carla_debug("audioMasterAutomate called from idle thread");
-                pData->postponeRtEvent(kPluginPostRtEventParameterChange, index, 0, fixedValue);
+                pData->postponeRtEvent(kPluginPostRtEventParameterChange, true, index, 0, 0, fixedValue);
+            }
+            // Called from main thread, why?
+            else if (pthread_equal(thisThread, fMainThread))
+            {
+                if (fFirstActive) {
+                    carla_stdout("audioMasterAutomate called while loading, nasty!");
+                } else {
+                    carla_debug("audioMasterAutomate called from main thread");
+                }
+                CarlaPlugin::setParameterValue(uindex, fixedValue, false, true, true);
             }
             // Called from UI?
             else if (fUI.isVisible)
@@ -1914,13 +1979,12 @@ protected:
             break;
 
         case audioMasterIdle:
-            if (pthread_equal(pthread_self(), fMainThread))
-            {
-                pData->engine->callback(ENGINE_CALLBACK_IDLE, 0, 0, 0, 0.0f, nullptr);
+            CARLA_SAFE_ASSERT_BREAK(pthread_equal(pthread_self(), fMainThread));
 
-                if (pData->engine->getType() != kEngineTypePlugin)
-                    pData->engine->idle();
-            }
+            pData->engine->callback(true, false, ENGINE_CALLBACK_IDLE, 0, 0, 0, 0, 0.0f, nullptr);
+
+            if (pData->engine->getType() != kEngineTypePlugin)
+                pData->engine->idle();
             break;
 
 #if ! VST_FORCE_DEPRECATED
@@ -2031,7 +2095,7 @@ protected:
                 effect->dispatcher(effect, effStartProcess);
             }
 
-            x_engine->callback(CALLBACK_RELOAD_ALL, m_id, 0, 0, 0.0, nullptr);
+            x_engine->callback(CALLBACK_RELOAD_ALL, m_id, 0, 0, 0, 0.0, nullptr);
 
             ret = 1;
             break;
@@ -2149,7 +2213,7 @@ protected:
 
         case audioMasterUpdateDisplay:
             // Update current program
-            if (pData->prog.count > 0)
+            if (pData->prog.count > 1)
             {
                 const int32_t current = static_cast<int32_t>(dispatcher(effGetProgram));
 
@@ -2166,20 +2230,30 @@ protected:
                     if (pData->prog.current != current)
                     {
                         pData->prog.current = current;
-                        pData->engine->callback(ENGINE_CALLBACK_PROGRAM_CHANGED, pData->id, current, 0, 0.0f, nullptr);
+                        pData->engine->callback(true, true,
+                                                ENGINE_CALLBACK_PROGRAM_CHANGED,
+                                                pData->id,
+                                                current,
+                                                0, 0, 0.0f, nullptr);
                     }
                 }
             }
 
             if (! fIsInitializing)
-                pData->engine->callback(ENGINE_CALLBACK_UPDATE, pData->id, 0, 0, 0.0f, nullptr);
+                pData->engine->callback(true, true,
+                                        ENGINE_CALLBACK_RELOAD_PARAMETERS, pData->id, 0, 0, 0, 0.0f, nullptr);
 
             ret = 1;
             break;
 
         case audioMasterBeginEdit:
+            CARLA_SAFE_ASSERT_BREAK(index >= 0);
+            pData->engine->touchPluginParameter(pData->id, static_cast<uint32_t>(index), true);
+            break;
+
         case audioMasterEndEdit:
-            // TODO
+            CARLA_SAFE_ASSERT_BREAK(index >= 0);
+            pData->engine->touchPluginParameter(pData->id, static_cast<uint32_t>(index), false);
             break;
 
         case audioMasterOpenFileSelector:
@@ -2205,7 +2279,8 @@ protected:
 #endif
 
         default:
-            carla_debug("CarlaPluginVST2::handleAudioMasterCallback(%02i:%s, %i, " P_INTPTR ", %p, %f) UNDEF", opcode, vstMasterOpcode2str(opcode), index, value, ptr, opt);
+            carla_debug("CarlaPluginVST2::handleAudioMasterCallback(%02i:%s, %i, " P_INTPTR ", %p, %f) UNDEF",
+                        opcode, vstMasterOpcode2str(opcode), index, value, ptr, static_cast<double>(opt));
             break;
         }
 
@@ -2224,12 +2299,14 @@ protected:
 
     bool hasMidiInput() const noexcept
     {
-        return (canDo("receiveVstEvents") || canDo("receiveVstMidiEvent") || (fEffect->flags & effFlagsIsSynth) > 0 || (pData->hints & PLUGIN_WANTS_MIDI_INPUT));
+        return (fEffect->flags & effFlagsIsSynth) != 0 ||
+               (pData->hints & PLUGIN_WANTS_MIDI_INPUT) != 0 ||
+               canDo("receiveVstEvents") || canDo("receiveVstMidiEvent");
     }
 
     bool hasMidiOutput() const noexcept
     {
-        return (canDo("sendVstEvents") || canDo("sendVstMidiEvent"));
+        return canDo("sendVstEvents") || canDo("sendVstMidiEvent");
     }
 
     // -------------------------------------------------------------------
@@ -2333,7 +2410,7 @@ public:
         // ---------------------------------------------------------------
         // initialize plugin (part 1)
 
-        sCurrentUniqueId     = uniqueId;
+        sCurrentUniqueId     = static_cast<intptr_t>(uniqueId);
         sLastCarlaPluginVST2 = this;
 
         try {
@@ -2367,6 +2444,43 @@ public:
         dispatcher(effSetBlockSize, 0, iBufferSize);
         dispatcher(effOpen);
 
+        const bool isShell = (dispatcher(effGetPlugCategory) == kPlugCategShell);
+
+        if (uniqueId == 0 && isShell)
+        {
+            char strBuf[STR_MAX+1];
+            carla_zeroChars(strBuf, STR_MAX+1);
+
+            sCurrentUniqueId = dispatcher(effShellGetNextPlugin, 0, 0, strBuf);
+
+            dispatcher(effClose);
+            fEffect = nullptr;
+
+            sLastCarlaPluginVST2 = this;
+
+            try {
+                fEffect = vstFn(carla_vst_audioMasterCallback);
+            } CARLA_SAFE_EXCEPTION_RETURN("Vst init", false);
+
+            sLastCarlaPluginVST2 = nullptr;
+            sCurrentUniqueId     = 0;
+
+            dispatcher(effIdentify);
+            dispatcher(effSetProcessPrecision, 0, kVstProcessPrecision32);
+            dispatcher(effSetBlockSizeAndSampleRate, 0, iBufferSize, nullptr, fSampleRate);
+            dispatcher(effSetSampleRate, 0, 0, nullptr, fSampleRate);
+            dispatcher(effSetBlockSize, 0, iBufferSize);
+            dispatcher(effOpen);
+        }
+
+        if (fEffect->uniqueID == 0 && !isShell)
+        {
+            dispatcher(effClose);
+            fEffect = nullptr;
+            pData->engine->setLastError("Plugin is not valid (no unique ID after being open)");
+            return false;
+        }
+
         // ---------------------------------------------------------------
         // get info
 
@@ -2420,30 +2534,33 @@ public:
 
         pData->options = 0x0;
 
-        if (pData->latency.frames != 0 || hasMidiOutput())
+        if (pData->latency.frames != 0 || hasMidiOutput() || isPluginOptionEnabled(options, PLUGIN_OPTION_FIXED_BUFFERS))
             pData->options |= PLUGIN_OPTION_FIXED_BUFFERS;
-        else if (options & PLUGIN_OPTION_FIXED_BUFFERS)
-            pData->options |= PLUGIN_OPTION_FIXED_BUFFERS;
-
-        if (fEffect->numPrograms > 1)
-            pData->options |= PLUGIN_OPTION_MAP_PROGRAM_CHANGES;
 
         if (fEffect->flags & effFlagsProgramChunks)
-            pData->options |= PLUGIN_OPTION_USE_CHUNKS;
+            if (isPluginOptionEnabled(options, PLUGIN_OPTION_USE_CHUNKS))
+                pData->options |= PLUGIN_OPTION_USE_CHUNKS;
 
         if (hasMidiInput())
         {
-            pData->options |= PLUGIN_OPTION_SEND_CHANNEL_PRESSURE;
-            pData->options |= PLUGIN_OPTION_SEND_NOTE_AFTERTOUCH;
-            pData->options |= PLUGIN_OPTION_SEND_PITCHBEND;
-            pData->options |= PLUGIN_OPTION_SEND_ALL_SOUND_OFF;
-
-            if (options & PLUGIN_OPTION_SEND_CONTROL_CHANGES)
+            if (isPluginOptionEnabled(options, PLUGIN_OPTION_SEND_CONTROL_CHANGES))
                 pData->options |= PLUGIN_OPTION_SEND_CONTROL_CHANGES;
-            if (options & PLUGIN_OPTION_MAP_PROGRAM_CHANGES)
-                pData->options |= PLUGIN_OPTION_MAP_PROGRAM_CHANGES;
+            if (isPluginOptionEnabled(options, PLUGIN_OPTION_SEND_CHANNEL_PRESSURE))
+                pData->options |= PLUGIN_OPTION_SEND_CHANNEL_PRESSURE;
+            if (isPluginOptionEnabled(options, PLUGIN_OPTION_SEND_NOTE_AFTERTOUCH))
+                pData->options |= PLUGIN_OPTION_SEND_NOTE_AFTERTOUCH;
+            if (isPluginOptionEnabled(options, PLUGIN_OPTION_SEND_PITCHBEND))
+                pData->options |= PLUGIN_OPTION_SEND_PITCHBEND;
+            if (isPluginOptionEnabled(options, PLUGIN_OPTION_SEND_ALL_SOUND_OFF))
+                pData->options |= PLUGIN_OPTION_SEND_ALL_SOUND_OFF;
+            if (isPluginOptionEnabled(options, PLUGIN_OPTION_SEND_PROGRAM_CHANGES))
+                pData->options |= PLUGIN_OPTION_SEND_PROGRAM_CHANGES;
         }
 
+        if (fEffect->numPrograms > 1 && (pData->options & PLUGIN_OPTION_SEND_PROGRAM_CHANGES) == 0)
+            if (isPluginOptionEnabled(options, PLUGIN_OPTION_MAP_PROGRAM_CHANGES))
+                pData->options |= PLUGIN_OPTION_MAP_PROGRAM_CHANGES;
+
         return true;
     }
 
@@ -2533,16 +2650,18 @@ private:
         return (int32_t)ByteOrder::swapIfLittleEndian ((uint32_t) x);
     }
 
-    bool loadOldSaveFormat(const void* const data, const std::size_t dataSize)
+    bool loadJuceSaveFormat(const void* const data, const std::size_t dataSize)
     {
         if (dataSize < 28)
             return false;
 
         const int32_t* const set = (const int32_t*)data;
 
+        if (set[1] != 0)
+            return false;
         if (! compareMagic(set[0], "CcnK"))
             return false;
-        if (! compareMagic(set[2], "FBCh"))
+        if (! compareMagic(set[2], "FBCh") && ! compareMagic(set[2], "FJuc"))
             return false;
         if (fxbSwap(set[3]) > 1)
             return false;
@@ -2553,6 +2672,7 @@ private:
         if (static_cast<std::size_t>(chunkSize + 160) > dataSize)
             return false;
 
+        carla_stdout("NOTE: Loading plugin state in VST2/JUCE compatibility mode");
         setChunkData(&set[40], static_cast<std::size_t>(chunkSize));
         return true;
     }
@@ -2607,7 +2727,8 @@ private:
     {
 #if defined(DEBUG) && ! defined(CARLA_OS_WIN)
         if (opcode != audioMasterGetTime && opcode != audioMasterProcessEvents && opcode != audioMasterGetCurrentProcessLevel && opcode != audioMasterGetOutputLatency)
-            carla_debug("carla_vst_audioMasterCallback(%p, %02i:%s, %i, " P_INTPTR ", %p, %f)", effect, opcode, vstMasterOpcode2str(opcode), index, value, ptr, opt);
+            carla_debug("carla_vst_audioMasterCallback(%p, %02i:%s, %i, " P_INTPTR ", %p, %f)",
+                        effect, opcode, vstMasterOpcode2str(opcode), index, value, ptr, static_cast<double>(opt));
 #endif
 
         switch (opcode)
@@ -2641,7 +2762,7 @@ private:
             return kVstLangEnglish;
         }
 
-        // Check if 'resvd1' points to us, otherwise register ourselfs if possible
+        // Check if 'resvd1' points to us, otherwise register ourselves if possible
         CarlaPluginVST2* self = nullptr;
 
         if (effect != nullptr)
@@ -2690,6 +2811,11 @@ CarlaPlugin* CarlaPlugin::newVST2(const Initializer& init)
 {
     carla_debug("CarlaPlugin::newVST2({%p, \"%s\", \"%s\", " P_INT64 "})", init.engine, init.filename, init.name, init.uniqueId);
 
+#ifdef USE_JUCE_FOR_VST2
+    if (std::getenv("CARLA_DO_NOT_USE_JUCE_FOR_VST2") == nullptr)
+        return newJuce(init, "VST2");
+#endif
+
     CarlaPluginVST2* const plugin(new CarlaPluginVST2(init.engine, init.id));
 
     if (! plugin->init(init.filename, init.name, init.uniqueId, init.options))
