@@ -1,6 +1,6 @@
 /*
  * Carla Plugin Host
- * Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -15,17 +15,36 @@
  * For a full copy of the GNU General Public License see the doc/GPL.txt file.
  */
 
+#include "CarlaHost.h"
 #include "CarlaUtils.h"
 
 #include "CarlaString.hpp"
 
-#include "rtaudio/RtAudio.h"
-#include "rtmidi/RtMidi.h"
-
 #ifdef HAVE_FLUIDSYNTH
 # include <fluidsynth.h>
 #endif
 
+#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wconversion"
+# pragma GCC diagnostic ignored "-Weffc++"
+# pragma GCC diagnostic ignored "-Wsign-conversion"
+# pragma GCC diagnostic ignored "-Wundef"
+# pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+#endif
+
+#ifdef USING_JUCE
+# include "AppConfig.h"
+# include "juce_core/juce_core.h"
+#else
+# include "rtaudio/RtAudio.h"
+# include "rtmidi/RtMidi.h"
+#endif
+
+#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+# pragma GCC diagnostic pop
+#endif
+
 #include "water/files/File.h"
 
 // -------------------------------------------------------------------------------------------------------------------
@@ -46,7 +65,14 @@ const char* carla_get_complete_license_text()
         "<li>LADSPA plugin support</li>"
         "<li>DSSI plugin support</li>"
         "<li>LV2 plugin support</li>"
-        "<li>VST2 plugin support using VeSTige header by Javier Serrano Polo</li>"
+#if defined(USING_JUCE) && (defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN))
+        "<li>VST2/3 plugin support (using Juce)</li>"
+#else
+        "<li>VST2 plugin support (using VeSTige header by Javier Serrano Polo)</li>"
+#endif
+#if defined(USING_JUCE) && defined(CARLA_OS_MAC)
+        "<li>AU plugin support (using Juce)</li>"
+#endif
 
         // Sample kit libraries
 #ifdef HAVE_FLUIDSYNTH
@@ -59,7 +85,9 @@ const char* carla_get_complete_license_text()
         "<li>liblo library for OSC support</li>"
         "<li>rtmempool library by Nedko Arnaudov"
         "<li>serd, sord, sratom and lilv libraries for LV2 discovery</li>"
+#ifndef USING_JUCE
         "<li>RtAudio v" RTAUDIO_VERSION " and RtMidi v" RTMIDI_VERSION " for native Audio and MIDI support</li>"
+#endif
 
         // Internal plugins
         "<li>MIDI Sequencer UI code by Perry Nguyen</li>"
@@ -80,6 +108,25 @@ const char* carla_get_complete_license_text()
     return retText;
 }
 
+const char* carla_get_juce_version()
+{
+    carla_debug("carla_get_juce_version()");
+
+    static CarlaString retVersion;
+
+#ifdef USING_JUCE
+    if (retVersion.isEmpty())
+    {
+        if (const char* const version = juce::SystemStats::getJUCEVersion().toRawUTF8())
+            retVersion = version+6;
+        else
+            retVersion = "Unknown";
+    }
+#endif
+
+    return retVersion;
+}
+
 const char* const* carla_get_supported_file_extensions()
 {
     carla_debug("carla_get_supported_file_extensions()");
@@ -96,12 +143,13 @@ const char* const* carla_get_supported_file_extensions()
 #ifdef HAVE_ZYN_DEPS
         "xmz", "xiz",
 #endif
-#if defined(CARLA_OS_MAC)
+#ifdef CARLA_OS_MAC
         "vst",
 #else
         "dll",
         "so",
 #endif
+        "vst3",
 
         // Audio files
 #ifdef HAVE_SNDFILE
@@ -110,7 +158,7 @@ const char* const* carla_get_supported_file_extensions()
 #endif
 #ifdef HAVE_FFMPEG
         "3g2", "3gp", "aac", "ac3", "amr", "ape", "mp2", "mp3", "mpc", "wma",
-# ifdef HAVE_SNDFILE
+# ifndef HAVE_SNDFILE
         // FFmpeg without sndfile
         "flac", "oga", "ogg", "w64", "wav",
 # endif
@@ -148,6 +196,15 @@ const char* const* carla_get_supported_features()
 #endif
 #ifdef HAVE_PYQT
         "gui",
+#endif
+#ifdef USING_JUCE
+        "juce",
+# if defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN)
+        "vst3",
+# endif
+# if defined(CARLA_OS_MAC)
+        "au",
+# endif
 #endif
         nullptr
     };
@@ -157,6 +214,8 @@ const char* const* carla_get_supported_features()
 
 // -------------------------------------------------------------------------------------------------------------------
 
-#include "../CarlaHostCommon.cpp"
+#ifndef CARLA_SKIP_HOST_COMMON
+# include "../CarlaHostCommon.cpp"
+#endif
 
 // -------------------------------------------------------------------------------------------------------------------
