@@ -1,6 +1,6 @@
 ﻿/*
  * Carla Bridge Plugin
- * Copyright (C) 2012-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,12 +23,16 @@
 #include "CarlaHost.h"
 
 #include "CarlaBackendUtils.hpp"
+#include "CarlaJuceUtils.hpp"
 #include "CarlaMainLoop.hpp"
 #include "CarlaMIDI.h"
 
+#ifdef CARLA_OS_UNIX
+# include <signal.h>
+#endif
+
 #ifdef CARLA_OS_LINUX
 # include <sched.h>
-# include <signal.h>
 # define SCHED_RESET_ON_FORK 0x40000000
 #endif
 
@@ -41,6 +45,26 @@
 # include <X11/Xlib.h>
 #endif
 
+#ifdef USING_JUCE
+# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wconversion"
+#  pragma GCC diagnostic ignored "-Weffc++"
+#  pragma GCC diagnostic ignored "-Wsign-conversion"
+#  pragma GCC diagnostic ignored "-Wundef"
+#  pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+# endif
+# include "AppConfig.h"
+# if defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN)
+#  include "juce_gui_basics/juce_gui_basics.h"
+# else
+#  include "juce_events/juce_events.h"
+# endif
+# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic pop
+# endif
+#endif
+
 #include "jackbridge/JackBridge.hpp"
 
 #include "water/files/File.h"
@@ -60,7 +84,7 @@ static bool gIsInitiated = false;
 static volatile bool gCloseNow = false;
 static volatile bool gSaveNow  = false;
 
-#if defined(CARLA_OS_LINUX)
+#if defined(CARLA_OS_UNIX)
 static void closeSignalHandler(int) noexcept
 {
     gCloseNow = true;
@@ -83,7 +107,7 @@ static BOOL WINAPI winSignalHandler(DWORD dwCtrlType) noexcept
 
 static void initSignalHandler()
 {
-#if defined(CARLA_OS_LINUX)
+#if defined(CARLA_OS_UNIX)
     struct sigaction sig;
     carla_zeroStruct(sig);
 
@@ -122,6 +146,52 @@ static void gIdle()
     }
 }
 
+ // -------------------------------------------------------------------------
+
+#if defined(USING_JUCE) && (defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN))
+class CarlaJuceApp : public juce::JUCEApplication,
+                     private juce::Timer
+{
+public:
+    CarlaJuceApp()  {}
+    ~CarlaJuceApp() {}
+
+    void initialise(const juce::String&) override
+    {
+        startTimer(8);
+    }
+
+    void shutdown() override
+    {
+        gCloseNow = true;
+        stopTimer();
+    }
+
+    const juce::String getApplicationName() override
+    {
+        return "CarlaPlugin";
+    }
+
+    const juce::String getApplicationVersion() override
+    {
+        return CARLA_VERSION_STRING;
+    }
+
+    void timerCallback() override
+    {
+        gIdle();
+
+        if (gCloseNow)
+        {
+            quit();
+            gCloseNow = false;
+        }
+    }
+};
+
+static juce::JUCEApplicationBase* juce_CreateApplication() { return new CarlaJuceApp(); }
+#endif
+
 // -------------------------------------------------------------------------
 
 class CarlaBridgePlugin
@@ -130,6 +200,9 @@ public:
     CarlaBridgePlugin(const bool useBridge, const char* const clientName, const char* const audioPoolBaseName,
                       const char* const rtClientBaseName, const char* const nonRtClientBaseName, const char* const nonRtServerBaseName)
         : fEngine(nullptr),
+#ifdef USING_JUCE
+          fJuceInitialiser(),
+#endif
           fUsingBridge(false),
           fUsingExec(false)
     {
@@ -140,9 +213,21 @@ public:
         carla_set_engine_callback(callback, this);
 
         if (useBridge)
-            carla_engine_init_bridge(audioPoolBaseName, rtClientBaseName, nonRtClientBaseName, nonRtServerBaseName, clientName);
+        {
+            carla_engine_init_bridge(audioPoolBaseName,
+                                     rtClientBaseName,
+                                     nonRtClientBaseName,
+                                     nonRtServerBaseName,
+                                     clientName);
+        }
+        else if (std::getenv("CARLA_BRIDGE_DUMMY") != nullptr)
+        {
+            carla_engine_init("Dummy", clientName);
+        }
         else
+        {
             carla_engine_init("JACK", clientName);
+        }
 
         fEngine = carla_get_engine();
     }
@@ -194,16 +279,25 @@ public:
 
         gIsInitiated = true;
 
+#if defined(USING_JUCE) && (defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN))
+# ifndef CARLA_OS_WIN
+        static const int argc = 0;
+        static const char* argv[] = {};
+# endif
+        juce::JUCEApplicationBase::createInstance = &juce_CreateApplication;
+        juce::JUCEApplicationBase::main(JUCE_MAIN_FUNCTION_ARGS);
+#else
         for (; runMainLoopOnce() && ! gCloseNow;)
         {
             gIdle();
-#if defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN)
+# if defined(CARLA_OS_MAC) || defined(CARLA_OS_WIN)
             // MacOS and Win32 have event-loops to run, so minimize sleep time
             carla_msleep(1);
-#else
+# else
             carla_msleep(5);
-#endif
+# endif
         }
+#endif
 
         carla_engine_close();
     }
@@ -211,7 +305,9 @@ public:
     // ---------------------------------------------------------------------
 
 protected:
-    void handleCallback(const EngineCallbackOpcode action, const int value1, const int, const float, const char* const)
+    void handleCallback(const EngineCallbackOpcode action,
+                        const int value1,
+                        const int, const int, const float, const char* const)
     {
         CARLA_BACKEND_USE_NAMESPACE;
 
@@ -235,16 +331,35 @@ protected:
 
 private:
     const CarlaEngine* fEngine;
-    bool               fUsingBridge;
-    bool               fUsingExec;
 
-    static void callback(void* ptr, EngineCallbackOpcode action, unsigned int pluginId, int value1, int value2, float value3, const char* valueStr)
+#ifdef USING_JUCE
+    const juce::ScopedJuceInitialiser_GUI fJuceInitialiser;
+#endif
+
+    bool fUsingBridge;
+    bool fUsingExec;
+
+    static void callback(void* ptr, EngineCallbackOpcode action, unsigned int pluginId,
+                         int value1, int value2, int value3,
+                         float valuef, const char* valueStr)
     {
-        carla_debug("CarlaBridgePlugin::callback(%p, %i:%s, %i, %i, %i, %f, \"%s\")", ptr, action, EngineCallbackOpcode2Str(action), pluginId, value1, value2, value3, valueStr);
-        CARLA_SAFE_ASSERT_RETURN(ptr != nullptr,);
-        CARLA_SAFE_ASSERT_RETURN(pluginId == 0,);
+        carla_debug("CarlaBridgePlugin::callback(%p, %i:%s, %i, %i, %i, %i, %f, \"%s\")",
+                    ptr, action, EngineCallbackOpcode2Str(action),
+                    pluginId, value1, value2, value3, static_cast<double>(valuef), valueStr);
 
-        return ((CarlaBridgePlugin*)ptr)->handleCallback(action, value1, value2, value3, valueStr);
+        // ptr must not be null
+        CARLA_SAFE_ASSERT_RETURN(ptr != nullptr,);
+
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+        // pluginId must be 0 (first), except for patchbay things
+        if (action < CarlaBackend::ENGINE_CALLBACK_PATCHBAY_CLIENT_ADDED ||
+            action > CarlaBackend::ENGINE_CALLBACK_PATCHBAY_CONNECTION_REMOVED)
+#endif
+        {
+            CARLA_SAFE_ASSERT_UINT_RETURN(pluginId == 0, pluginId,);
+        }
+
+        return ((CarlaBridgePlugin*)ptr)->handleCallback(action, value1, value2, value3, valuef, valueStr);
     }
 
     CARLA_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(CarlaBridgePlugin)
@@ -313,6 +428,11 @@ int main(int argc, char* argv[])
         return 1;
     }
 
+    // ---------------------------------------------------------------------
+    // Set file
+
+    const File file(filename != nullptr ? filename : "");
+
     // ---------------------------------------------------------------------
     // Set name
 
@@ -418,8 +538,7 @@ int main(int argc, char* argv[])
     }
     else
     {
-        const String jfilename = String(CharPointer_UTF8(filename));
-        clientName = File(jfilename).getFileNameWithoutExtension().toRawUTF8();
+        clientName = file.getFileNameWithoutExtension().toRawUTF8();
     }
 
     // if we still have no client name by now, use a dummy one
@@ -514,14 +633,13 @@ int main(int argc, char* argv[])
         // -----------------------------------------------------------------
         // Init plugin
 
-        if (carla_add_plugin(btype, itype, filename, name, label, uniqueId, extraStuff, 0x0))
+        if (carla_add_plugin(btype, itype, file.getFullPathName().toRawUTF8(), name, label, uniqueId, extraStuff, 0x0))
         {
             ret = 0;
 
             if (! useBridge)
             {
                 carla_set_active(0, true);
-                carla_set_option(0, CarlaBackend::PLUGIN_OPTION_SEND_CONTROL_CHANGES, true);
 
                 if (const CarlaPluginInfo* const pluginInfo = carla_get_plugin_info(0))
                 {
@@ -544,8 +662,16 @@ int main(int argc, char* argv[])
             const char* const lastError(carla_get_last_error());
             carla_stderr("Plugin failed to load, error was:\n%s", lastError);
 
-            //if (useBridge)
-            //    bridge.sendOscBridgeError(lastError);
+            if (useBridge)
+            {
+                // do a single idle so that we can send error message to server
+                gIdle();
+
+#ifdef CARLA_OS_UNIX
+                // kill ourselves now if we can't load plugin in bridge mode
+                ::kill(::getpid(), SIGKILL);
+#endif
+            }
         }
     }
 
