@@ -1,6 +1,6 @@
 /*
  * Carla LV2 Single Plugin
- * Copyright (C) 2017-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2017-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,24 @@
 #include "CarlaLv2Utils.hpp"
 #include "CarlaUtils.h"
 
+#ifdef USING_JUCE
+# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wconversion"
+#  pragma GCC diagnostic ignored "-Weffc++"
+#  pragma GCC diagnostic ignored "-Wsign-conversion"
+#  pragma GCC diagnostic ignored "-Wundef"
+#  pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+# endif
+
+# include "AppConfig.h"
+# include "juce_events/juce_events.h"
+
+# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic pop
+# endif
+#endif
+
 #include "water/files/File.h"
 
 template<>
@@ -50,6 +68,9 @@ public:
         : Lv2PluginBaseClass<EngineTimeInfo>(sampleRate, features),
           fPlugin(nullptr),
           fUiName()
+#ifdef USING_JUCE
+        , fJuceInitialiser()
+#endif
     {
         CARLA_SAFE_ASSERT_RETURN(pData->curPluginCount == 0,)
         CARLA_SAFE_ASSERT_RETURN(pData->plugins == nullptr,);
@@ -88,7 +109,7 @@ public:
         using water::File;
         const File pluginFile(File::getSpecialLocation(File::currentExecutableFile).withFileExtension("xml"));
 
-        if (! loadProject(pluginFile.getFullPathName().toRawUTF8()))
+        if (! loadProject(pluginFile.getFullPathName().toRawUTF8(), true))
         {
             carla_stderr2("Failed to init plugin, possible reasons: %s", getLastError());
             return;
@@ -104,6 +125,8 @@ public:
         fPorts.usesTime     = true;
         fPorts.numAudioIns  = fPlugin->getAudioInCount();
         fPorts.numAudioOuts = fPlugin->getAudioOutCount();
+        fPorts.numCVIns     = fPlugin->getCVInCount();
+        fPorts.numCVOuts    = fPlugin->getCVOutCount();
         fPorts.numMidiIns   = fPlugin->getMidiInCount();
         fPorts.numMidiOuts  = fPlugin->getMidiOutCount();
         fPorts.numParams    = fPlugin->getParameterCount();
@@ -200,7 +223,11 @@ public:
         if (fPlugin->tryLock(fIsOffline))
         {
             fPlugin->initBuffers();
-            fPlugin->process(fPorts.audioIns, fPorts.audioOuts, nullptr, nullptr, frames);
+            fPlugin->process(fPorts.audioCVIns,
+                             fPorts.audioCVOuts,
+                             fPorts.audioCVIns + fPorts.numAudioIns,
+                             fPorts.audioCVOuts + fPorts.numAudioOuts,
+                             frames);
             fPlugin->unlock();
 
             if (fPorts.numMidiOuts > 0)
@@ -245,7 +272,7 @@ public:
                             mdataTmp[0] = static_cast<uint8_t>(midiEvent.data[0] | (engineEvent.channel & MIDI_CHANNEL_BIT));
 
                             // copy rest
-                            carla_copy<uint8_t>(mdataTmp+1, midiEvent.data+1, size-1);
+                            carla_copy<uint8_t>(mdataTmp+1, midiEvent.data+1, size-1U);
 
                             // done
                             mdataPtr = mdataTmp;
@@ -264,7 +291,9 @@ public:
         else
         {
             for (uint32_t i=0; i<fPorts.numAudioOuts; ++i)
-                carla_zeroFloats(fPorts.audioOuts[i], frames);
+                carla_zeroFloats(fPorts.audioCVOuts[i], frames);
+            for (uint32_t i=0; i<fPorts.numCVOuts; ++i)
+                carla_zeroFloats(fPorts.audioCVOuts[fPorts.numAudioOuts+i], frames);
         }
 
         lv2_post_run(frames);
@@ -390,7 +419,9 @@ protected:
         return "LV2 Plugin";
     }
 
-    void engineCallback(const EngineCallbackOpcode action, const uint pluginId, const int value1, const int value2, const float value3, const char* const valueStr)
+    void engineCallback(const EngineCallbackOpcode action, const uint pluginId,
+                        const int value1, const int value2, const int value3,
+                        const float valuef, const char* const valueStr)
     {
         switch (action)
         {
@@ -402,7 +433,7 @@ protected:
             {
                 fUI.writeFunction(fUI.controller,
                                   static_cast<uint32_t>(value1)+fPorts.indexOffset,
-                                  sizeof(float), 0, &value3);
+                                  sizeof(float), 0, &valuef);
             }
             break;
 
@@ -417,7 +448,9 @@ protected:
 
         default:
             carla_stdout("engineCallback(%i:%s, %u, %i, %i, %f, %s)",
-                         action, EngineCallbackOpcode2Str(action), pluginId, value1, value2, value3, valueStr);
+                         action, EngineCallbackOpcode2Str(action), pluginId,
+                         value1, value2, value3,
+                         static_cast<double>(valuef), valueStr);
             break;
         }
     }
@@ -466,6 +499,10 @@ private:
     CarlaPlugin* fPlugin;
     CarlaString fUiName;
 
+#ifdef USING_JUCE
+    juce::SharedResourcePointer<juce::ScopedJuceInitialiser_GUI> fJuceInitialiser;
+#endif
+
     void updateParameterOutputs() noexcept
     {
         float value;
@@ -515,9 +552,11 @@ private:
 
     #define handlePtr ((CarlaEngineSingleLV2*)handle)
 
-    static void _engine_callback(void* handle, EngineCallbackOpcode action, uint pluginId, int value1, int value2, float value3, const char* valueStr)
+    static void _engine_callback(void* handle, EngineCallbackOpcode action, uint pluginId,
+                                 int value1, int value2, int value3,
+                                 float valuef, const char* valueStr)
     {
-        handlePtr->engineCallback(action, pluginId, value1, value2, value3, valueStr);
+        handlePtr->engineCallback(action, pluginId, value1, value2, value3, valuef, valueStr);
     }
 
     #undef handlePtr
