@@ -1,6 +1,6 @@
 /*
  * Carla Bridge UI
- * Copyright (C) 2011-2017 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,7 +28,8 @@
 #include <string>
 #include <vector>
 
-#define URI_CARLA_ATOM_WORKER "http://kxstudio.sf.net/ns/carla/atomWorker"
+#define URI_CARLA_ATOM_WORKER_IN   "http://kxstudio.sf.net/ns/carla/atomWorkerIn"
+#define URI_CARLA_ATOM_WORKER_RESP "http://kxstudio.sf.net/ns/carla/atomWorkerResp"
 
 using water::File;
 
@@ -39,7 +40,7 @@ CARLA_BRIDGE_UI_START_NAMESPACE
 static double gInitialSampleRate = 44100.0;
 static const char* const kNullWindowTitle = "TestUI";
 static const uint32_t kNullWindowTitleSize = 6;
-
+static const char* const kUnmapFallback = "urn:null";
 
 // LV2 URI Map Ids
 enum CarlaLv2URIDs {
@@ -75,6 +76,9 @@ enum CarlaLv2URIDs {
     kUridLogNote,
     kUridLogTrace,
     kUridLogWarning,
+    kUridPatchSet,
+    kUridPatchPoperty,
+    kUridPatchValue,
     // time base type
     kUridTimePosition,
      // time values
@@ -90,9 +94,16 @@ enum CarlaLv2URIDs {
     kUridTimeTicksPerBeat,
     kUridMidiEvent,
     kUridParamSampleRate,
+    // ui stuff
+    kUridBackgroundColor,
+    kUridForegroundColor,
+    kUridScaleFactor,
     kUridWindowTitle,
-    kUridCarlaAtomWorker,
+    // custom carla props
+    kUridCarlaAtomWorkerIn,
+    kUridCarlaAtomWorkerResp,
     kUridCarlaTransientWindowId,
+    // count
     kUridCount
 };
 
@@ -115,6 +126,7 @@ enum CarlaLv2Features {
     kFeatureIdUiParent,
     kFeatureIdUiPortMap,
     kFeatureIdUiPortSubscribe,
+    kFeatureIdUiRequestValue,
     kFeatureIdUiResize,
     kFeatureIdUiTouch,
     kFeatureCount
@@ -126,6 +138,9 @@ struct Lv2PluginOptions {
     enum OptIndex {
         SampleRate,
         TransientWinId,
+        BackgroundColor,
+        ForegroundColor,
+        ScaleFactor,
         WindowTitle,
         Null,
         Count
@@ -133,13 +148,17 @@ struct Lv2PluginOptions {
 
     float sampleRate;
     int64_t transientWinId;
-    const char* windowTitle;
+    uint32_t bgColor;
+    uint32_t fgColor;
+    float uiScale;
     LV2_Options_Option opts[Count];
 
     Lv2PluginOptions() noexcept
-        : sampleRate(gInitialSampleRate),
+        : sampleRate(static_cast<float>(gInitialSampleRate)),
           transientWinId(0),
-          windowTitle(kNullWindowTitle)
+          bgColor(0x000000ff),
+          fgColor(0xffffffff),
+          uiScale(1.0f)
     {
         LV2_Options_Option& optSampleRate(opts[SampleRate]);
         optSampleRate.context = LV2_OPTIONS_INSTANCE;
@@ -149,6 +168,30 @@ struct Lv2PluginOptions {
         optSampleRate.type    = kUridAtomFloat;
         optSampleRate.value   = &sampleRate;
 
+        LV2_Options_Option& optBackgroundColor(opts[BackgroundColor]);
+        optBackgroundColor.context = LV2_OPTIONS_INSTANCE;
+        optBackgroundColor.subject = 0;
+        optBackgroundColor.key     = kUridBackgroundColor;
+        optBackgroundColor.size    = sizeof(int32_t);
+        optBackgroundColor.type    = kUridAtomInt;
+        optBackgroundColor.value   = &bgColor;
+
+        LV2_Options_Option& optForegroundColor(opts[ForegroundColor]);
+        optForegroundColor.context = LV2_OPTIONS_INSTANCE;
+        optForegroundColor.subject = 0;
+        optForegroundColor.key     = kUridForegroundColor;
+        optForegroundColor.size    = sizeof(int32_t);
+        optForegroundColor.type    = kUridAtomInt;
+        optForegroundColor.value   = &fgColor;
+
+        LV2_Options_Option& optScaleFactor(opts[ScaleFactor]);
+        optScaleFactor.context = LV2_OPTIONS_INSTANCE;
+        optScaleFactor.subject = 0;
+        optScaleFactor.key     = kUridScaleFactor;
+        optScaleFactor.size    = sizeof(float);
+        optScaleFactor.type    = kUridAtomFloat;
+        optScaleFactor.value   = &uiScale;
+
         LV2_Options_Option& optTransientWinId(opts[TransientWinId]);
         optTransientWinId.context = LV2_OPTIONS_INSTANCE;
         optTransientWinId.subject = 0;
@@ -234,6 +277,10 @@ public:
         uiPortMapFt->handle               = this;
         uiPortMapFt->port_index           = carla_lv2_ui_port_map;
 
+        LV2UI_Request_Value* const uiRequestValueFt = new LV2UI_Request_Value;
+        uiRequestValueFt->handle                    = this;
+        uiRequestValueFt->request                   = carla_lv2_ui_request_value;
+
         LV2UI_Resize* const uiResizeFt    = new LV2UI_Resize;
         uiResizeFt->handle                = this;
         uiResizeFt->ui_resize             = carla_lv2_ui_resize;
@@ -292,6 +339,9 @@ public:
         fFeatures[kFeatureIdUiPortSubscribe]->URI  = LV2_UI__portSubscribe;
         fFeatures[kFeatureIdUiPortSubscribe]->data = nullptr;
 
+        fFeatures[kFeatureIdUiRequestValue]->URI  = LV2_UI__requestValue;
+        fFeatures[kFeatureIdUiRequestValue]->data = uiRequestValueFt;
+
         fFeatures[kFeatureIdUiResize]->URI  = LV2_UI__resize;
         fFeatures[kFeatureIdUiResize]->data = uiResizeFt;
 
@@ -323,6 +373,7 @@ public:
         delete (LV2_URID_Map*)fFeatures[kFeatureIdUridMap]->data;
         delete (LV2_URID_Unmap*)fFeatures[kFeatureIdUridUnmap]->data;
         delete (LV2UI_Port_Map*)fFeatures[kFeatureIdUiPortMap]->data;
+        delete (LV2UI_Request_Value*)fFeatures[kFeatureIdUiRequestValue]->data;
         delete (LV2UI_Resize*)fFeatures[kFeatureIdUiResize]->data;
 
         for (uint32_t i=0; i < kFeatureCount; ++i)
@@ -584,11 +635,14 @@ public:
         fCustomURIDs.push_back(uri);
     }
 
-    void uiOptionsChanged(const double sampleRate, const bool useTheme, const bool useThemeColors, const char* const windowTitle, uintptr_t transientWindowId) override
+    void uiOptionsChanged(const BridgeFormatOptions& opts) override
     {
-        carla_debug("CarlaLv2Client::uiOptionsChanged(%g, %s, %s, \"%s\", " P_UINTPTR ")", sampleRate, bool2str(useTheme), bool2str(useThemeColors), windowTitle, transientWindowId);
+        carla_debug("CarlaLv2Client::uiOptionsChanged()");
 
-        const float sampleRatef = static_cast<float>(sampleRate);
+        // ------------------------------------------------------------------------------------------------------------
+        // sample rate
+
+        const float sampleRatef = static_cast<float>(opts.sampleRate);
 
         if (carla_isNotEqual(fLv2Options.sampleRate, sampleRatef))
         {
@@ -611,23 +665,35 @@ public:
             }
         }
 
-        if (fLv2Options.windowTitle != nullptr && fLv2Options.windowTitle != kNullWindowTitle)
-            delete[] fLv2Options.windowTitle;
+        // ------------------------------------------------------------------------------------------------------------
+        // ui colors and scale
 
-        fLv2Options.transientWinId = static_cast<int64_t>(transientWindowId);
-        fLv2Options.windowTitle    = carla_strdup_safe(windowTitle);
+        fLv2Options.bgColor = opts.bgColor;
+        fLv2Options.fgColor = opts.fgColor;
+        fLv2Options.uiScale = opts.uiScale;
 
-        if (fLv2Options.windowTitle == nullptr)
-            fLv2Options.windowTitle = kNullWindowTitle;
+        // ------------------------------------------------------------------------------------------------------------
+        // window title
+
+        if (opts.windowTitle != nullptr)
+            fUiOptions.windowTitle = opts.windowTitle;
         else
-            fUiOptions.windowTitle = fLv2Options.windowTitle;
+            fUiOptions.windowTitle.clear();
 
-        fLv2Options.opts[Lv2PluginOptions::WindowTitle].size  = (uint32_t)std::strlen(fLv2Options.windowTitle);
-        fLv2Options.opts[Lv2PluginOptions::WindowTitle].value = fLv2Options.windowTitle;
+        fLv2Options.opts[Lv2PluginOptions::WindowTitle].size  = static_cast<uint32_t>(fUiOptions.windowTitle.length());
+        fLv2Options.opts[Lv2PluginOptions::WindowTitle].value = fUiOptions.windowTitle.buffer();
 
-        fUiOptions.useTheme          = useTheme;
-        fUiOptions.useThemeColors    = useThemeColors;
-        fUiOptions.transientWindowId = transientWindowId;
+        // ------------------------------------------------------------------------------------------------------------
+        // transient win id
+
+        fLv2Options.transientWinId = static_cast<int64_t>(opts.transientWindowId);
+        fUiOptions.transientWindowId = opts.transientWindowId;
+
+        // ------------------------------------------------------------------------------------------------------------
+        // other
+
+        fUiOptions.useTheme       = opts.useTheme;
+        fUiOptions.useThemeColors = opts.useThemeColors;
     }
 
     void uiResized(const uint width, const uint height) override
@@ -667,9 +733,8 @@ public:
 
     const char* getCustomURIDString(const LV2_URID urid) const noexcept
     {
-        static const char* const sFallback = "urn:null";
-        CARLA_SAFE_ASSERT_RETURN(urid != kUridNull, sFallback);
-        CARLA_SAFE_ASSERT_RETURN(urid < fCustomURIDs.size(), sFallback);
+        CARLA_SAFE_ASSERT_RETURN(urid != kUridNull, kUnmapFallback);
+        CARLA_SAFE_ASSERT_RETURN(urid < fCustomURIDs.size(), kUnmapFallback);
         carla_debug("CarlaLv2Client::getCustomURIDString(%i)", urid);
 
         return fCustomURIDs[urid].c_str();
@@ -697,6 +762,55 @@ public:
         return LV2UI_INVALID_PORT_INDEX;
     }
 
+    LV2UI_Request_Value_Status handleUiRequestValue(const LV2_URID key,
+                                                    const LV2_URID type,
+                                                    const LV2_Feature* const* features)
+    {
+        CARLA_SAFE_ASSERT_RETURN(fToolkit != nullptr, LV2UI_REQUEST_VALUE_ERR_UNKNOWN);
+        carla_debug("CarlaLv2Client::handleUIRequestValue(%u, %u, %p)", key, type, features);
+
+        if (type != kUridAtomPath)
+            return LV2UI_REQUEST_VALUE_ERR_UNSUPPORTED;
+
+        const char* const uri = getCustomURIDString(key);
+        CARLA_SAFE_ASSERT_RETURN(uri != nullptr && uri != kUnmapFallback, LV2UI_REQUEST_VALUE_ERR_UNKNOWN);
+
+        // TODO check if a file browser is already open
+
+        for (uint32_t i=0; i < fRdfDescriptor->ParameterCount; ++i)
+        {
+            if (fRdfDescriptor->Parameters[i].Type != LV2_PARAMETER_PATH)
+                continue;
+            if (std::strcmp(fRdfDescriptor->Parameters[i].URI, uri) != 0)
+                continue;
+
+            // TODO file browser filters, also label for title
+            if (isPipeRunning())
+            {
+                char tmpBuf[0xff];
+
+                const CarlaMutexLocker cml(getPipeLock());
+
+                writeMessage("requestvalue\n", 13);
+
+                std::snprintf(tmpBuf, 0xff-1, "%u\n", key);
+                tmpBuf[0xff-1] = '\0';
+                writeMessage(tmpBuf);
+
+                std::snprintf(tmpBuf, 0xff-1, "%u\n", type);
+                tmpBuf[0xff-1] = '\0';
+                writeMessage(tmpBuf);
+            }
+
+            return LV2UI_REQUEST_VALUE_SUCCESS;
+        }
+
+        return LV2UI_REQUEST_VALUE_ERR_UNSUPPORTED;
+
+        // may be unused
+        (void)features;
+    }
+
     int handleUiResize(const int width, const int height)
     {
         CARLA_SAFE_ASSERT_RETURN(fToolkit != nullptr, 1);
@@ -735,11 +849,12 @@ public:
                 const LV2_Atom* const atom((const LV2_Atom*)buffer);
 
                 // plugins sometimes fail on this, not good...
-                const uint32_t totalSize =  lv2_atom_total_size(atom);
+                const uint32_t totalSize = lv2_atom_total_size(atom);
                 const uint32_t paddedSize = lv2_atom_pad_size(totalSize);
 
                 if (bufferSize != totalSize && bufferSize != paddedSize)
-                    carla_stderr2("Warning: LV2 UI sending atom with invalid size! size: %u, padded-size: %u", totalSize, paddedSize);
+                    carla_stderr2("Warning: LV2 UI sending atom with invalid size %u! size: %u, padded-size: %u",
+                                  bufferSize, totalSize, paddedSize);
 
                 writeLv2AtomMessage(rindex, atom);
             }
@@ -990,6 +1105,14 @@ private:
         if (std::strcmp(uri, LV2_LOG__Warning) == 0)
             return kUridLogWarning;
 
+        // Patch types
+        if (std::strcmp(uri, LV2_PATCH__Set) == 0)
+            return kUridPatchSet;
+        if (std::strcmp(uri, LV2_PATCH__property) == 0)
+            return kUridPatchPoperty;
+        if (std::strcmp(uri, LV2_PATCH__value) == 0)
+            return kUridPatchValue;
+
         // Time types
         if (std::strcmp(uri, LV2_TIME__Position) == 0)
             return kUridTimePosition;
@@ -1019,12 +1142,20 @@ private:
             return kUridMidiEvent;
         if (std::strcmp(uri, LV2_PARAMETERS__sampleRate) == 0)
             return kUridParamSampleRate;
+        if (std::strcmp(uri, LV2_UI__backgroundColor) == 0)
+            return kUridBackgroundColor;
+        if (std::strcmp(uri, LV2_UI__foregroundColor) == 0)
+            return kUridForegroundColor;
+        if (std::strcmp(uri, LV2_UI__scaleFactor) == 0)
+            return kUridScaleFactor;
         if (std::strcmp(uri, LV2_UI__windowTitle) == 0)
             return kUridWindowTitle;
 
         // Custom Carla types
-        if (std::strcmp(uri, URI_CARLA_ATOM_WORKER) == 0)
-            return kUridCarlaAtomWorker;
+        if (std::strcmp(uri, URI_CARLA_ATOM_WORKER_IN) == 0)
+            return kUridCarlaAtomWorkerIn;
+        if (std::strcmp(uri, URI_CARLA_ATOM_WORKER_RESP) == 0)
+            return kUridCarlaAtomWorkerResp;
         if (std::strcmp(uri, LV2_KXSTUDIO_PROPERTIES__TransientWindowId) == 0)
             return kUridCarlaTransientWindowId;
 
@@ -1108,6 +1239,14 @@ private:
         case kUridLogWarning:
             return LV2_LOG__Warning;
 
+        // Patch types
+        case kUridPatchSet:
+            return LV2_PATCH__Set;
+        case kUridPatchPoperty:
+            return LV2_PATCH__property;
+        case kUridPatchValue:
+            return LV2_PATCH__value;
+
         // Time types
         case kUridTimePosition:
             return LV2_TIME__Position;
@@ -1137,12 +1276,20 @@ private:
             return LV2_MIDI__MidiEvent;
         case kUridParamSampleRate:
             return LV2_PARAMETERS__sampleRate;
+        case kUridBackgroundColor:
+            return LV2_UI__backgroundColor;
+        case kUridForegroundColor:
+            return LV2_UI__foregroundColor;
+        case kUridScaleFactor:
+            return LV2_UI__scaleFactor;
         case kUridWindowTitle:
             return LV2_UI__windowTitle;
 
         // Custom Carla types
-        case kUridCarlaAtomWorker:
-            return URI_CARLA_ATOM_WORKER;
+        case kUridCarlaAtomWorkerIn:
+            return URI_CARLA_ATOM_WORKER_IN;
+        case kUridCarlaAtomWorkerResp:
+            return URI_CARLA_ATOM_WORKER_RESP;
         case kUridCarlaTransientWindowId:
             return LV2_KXSTUDIO_PROPERTIES__TransientWindowId;
         }
@@ -1162,6 +1309,20 @@ private:
         return ((CarlaLv2Client*)handle)->handleUiPortMap(symbol);
     }
 
+    // ----------------------------------------------------------------------------------------------------------------
+    // UI Request Parameter Feature
+
+    static LV2UI_Request_Value_Status carla_lv2_ui_request_value(LV2UI_Feature_Handle handle,
+                                                                 LV2_URID key,
+                                                                 LV2_URID type,
+                                                                 const LV2_Feature* const* features)
+    {
+        CARLA_SAFE_ASSERT_RETURN(handle != nullptr, LV2UI_REQUEST_VALUE_ERR_UNKNOWN);
+        carla_debug("carla_lv2_ui_request_value(%p, %u, %u, %p)", handle, key, type, features);
+
+        return ((CarlaLv2Client*)handle)->handleUiRequestValue(key, type, features);
+    }
+
     // ----------------------------------------------------------------------------------------------------------------
     // UI Resize Feature
 
@@ -1207,7 +1368,10 @@ int main(int argc, const char* argv[])
 
     // try to get sampleRate value
     if (const char* const sampleRateStr = std::getenv("CARLA_SAMPLE_RATE"))
+    {
+        const CarlaScopedLocale csl;
         gInitialSampleRate = std::atof(sampleRateStr);
+    }
 
     // Init LV2 client
     CarlaLv2Client client;
