@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Common Carla code
-# Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+# Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -17,9 +17,7 @@
 # For a full copy of the GNU General Public License see the doc/GPL.txt file.
 
 # ------------------------------------------------------------------------------------------------------------
-# Imports (Config)
-
-from carla_config import *
+# Config
 
 # These will be modified during install
 X_LIBDIR_X = None
@@ -31,15 +29,12 @@ X_DATADIR_X = None
 import os
 import sys
 
-if config_UseQt5:
-    from PyQt5.Qt import PYQT_VERSION_STR
-    from PyQt5.QtCore import qFatal, qVersion, qWarning, QDir
-    from PyQt5.QtGui import QIcon
-    from PyQt5.QtWidgets import QFileDialog, QMessageBox
-else:
-    from PyQt4.Qt import PYQT_VERSION_STR
-    from PyQt4.QtCore import qFatal, qVersion, qWarning, QDir
-    from PyQt4.QtGui import QFileDialog, QIcon, QMessageBox
+from math import fmod
+
+from PyQt5.Qt import PYQT_VERSION_STR
+from PyQt5.QtCore import qFatal, QT_VERSION, QT_VERSION_STR, qWarning, QDir, QSettings
+from PyQt5.QtGui import QIcon
+from PyQt5.QtWidgets import QFileDialog, QMessageBox
 
 # ------------------------------------------------------------------------------------------------------------
 # Import Signal
@@ -66,7 +61,7 @@ if WINDOWS:
 # ------------------------------------------------------------------------------------------------------------
 # Set Version
 
-VERSION = "2.0.0"
+VERSION = "2.1.0"
 
 # ------------------------------------------------------------------------------------------------------------
 # Set TMP
@@ -126,58 +121,44 @@ del envPATH
 # Static MIDI CC list
 
 MIDI_CC_LIST = (
-    "0x01 Modulation",
-    "0x02 Breath",
-    "0x03 (Undefined)",
-    "0x04 Foot",
-    "0x05 Portamento",
-    "0x07 Volume",
-    "0x08 Balance",
-    "0x09 (Undefined)",
-    "0x0A Pan",
-    "0x0B Expression",
-    "0x0C FX Control 1",
-    "0x0D FX Control 2",
-    "0x0E (Undefined)",
-    "0x0F (Undefined)",
-    "0x10 General Purpose 1",
-    "0x11 General Purpose 2",
-    "0x12 General Purpose 3",
-    "0x13 General Purpose 4",
-    "0x14 (Undefined)",
-    "0x15 (Undefined)",
-    "0x16 (Undefined)",
-    "0x17 (Undefined)",
-    "0x18 (Undefined)",
-    "0x19 (Undefined)",
-    "0x1A (Undefined)",
-    "0x1B (Undefined)",
-    "0x1C (Undefined)",
-    "0x1D (Undefined)",
-    "0x1E (Undefined)",
-    "0x1F (Undefined)",
-    "0x46 Control 1 [Variation]",
-    "0x47 Control 2 [Timbre]",
-    "0x48 Control 3 [Release]",
-    "0x49 Control 4 [Attack]",
-    "0x4A Control 5 [Brightness]",
-    "0x4B Control 6 [Decay]",
-    "0x4C Control 7 [Vib Rate]",
-    "0x4D Control 8 [Vib Depth]",
-    "0x4E Control 9 [Vib Delay]",
-    "0x4F Control 10 [Undefined]",
-    "0x50 General Purpose 5",
-    "0x51 General Purpose 6",
-    "0x52 General Purpose 7",
-    "0x53 General Purpose 8",
-    "0x54 Portamento Control",
-    "0x5B FX 1 Depth [Reverb]",
-    "0x5C FX 2 Depth [Tremolo]",
-    "0x5D FX 3 Depth [Chorus]",
-    "0x5E FX 4 Depth [Detune]",
-    "0x5F FX 5 Depth [Phaser]"
+    "01 [0x01] Modulation",
+    "02 [0x02] Breath",
+    "04 [0x04] Foot",
+    "05 [0x05] Portamento",
+    "07 [0x07] Volume",
+    "02 [0x08] Balance",
+    "10 [0x0A] Pan",
+    "11 [0x0B] Expression",
+    "12 [0x0C] FX Control 1",
+    "13 [0x0D] FX Control 2",
+    "16 [0x10] General Purpose 1",
+    "17 [0x11] General Purpose 2",
+    "18 [0x12] General Purpose 3",
+    "19 [0x13] General Purpose 4",
+    "70 [0x46] Control 1 [Variation]",
+    "71 [0x47] Control 2 [Timbre]",
+    "72 [0x48] Control 3 [Release]",
+    "73 [0x49] Control 4 [Attack]",
+    "74 [0x4A] Control 5 [Brightness]",
+    "75 [0x4B] Control 6 [Decay]",
+    "76 [0x4C] Control 7 [Vib Rate]",
+    "77 [0x4D] Control 8 [Vib Depth]",
+    "78 [0x4E] Control 9 [Vib Delay]",
+    "79 [0x4F] Control 10 [Undefined]",
+    "80 [0x50] General Purpose 5",
+    "81 [0x51] General Purpose 6",
+    "82 [0x52] General Purpose 7",
+    "83 [0x53] General Purpose 8",
+    "84 [0x54] Portamento Control",
+    "91 [0x5B] FX 1 Depth [Reverb]",
+    "92 [0x5C] FX 2 Depth [Tremolo]",
+    "93 [0x5D] FX 3 Depth [Chorus]",
+    "94 [0x5E] FX 4 Depth [Detune]",
+    "95 [0x5F] FX 5 Depth [Phaser]"
 )
 
+MAX_MIDI_CC_LIST_ITEM = 95
+
 # ------------------------------------------------------------------------------------------------------------
 # PatchCanvas defines
 
@@ -205,6 +186,8 @@ CARLA_KEY_CANVAS_FANCY_EYE_CANDY   = "Canvas/FancyEyeCandy"   # bool
 CARLA_KEY_CANVAS_USE_OPENGL        = "Canvas/UseOpenGL"       # bool
 CARLA_KEY_CANVAS_ANTIALIASING      = "Canvas/Antialiasing"    # enum
 CARLA_KEY_CANVAS_HQ_ANTIALIASING   = "Canvas/HQAntialiasing"  # bool
+CARLA_KEY_CANVAS_INLINE_DISPLAYS   = "Canvas/InlineDisplays"  # bool
+CARLA_KEY_CANVAS_FULL_REPAINTS     = "Canvas/FullRepaints"    # bool
 
 CARLA_KEY_ENGINE_DRIVER_PREFIX         = "Engine/Driver-"
 CARLA_KEY_ENGINE_AUDIO_DRIVER          = "Engine/AudioDriver"         # str
@@ -217,12 +200,25 @@ CARLA_KEY_ENGINE_PREFER_UI_BRIDGES     = "Engine/PreferUiBridges"     # bool
 CARLA_KEY_ENGINE_MANAGE_UIS            = "Engine/ManageUIs"           # bool
 CARLA_KEY_ENGINE_UIS_ALWAYS_ON_TOP     = "Engine/UIsAlwaysOnTop"      # bool
 CARLA_KEY_ENGINE_MAX_PARAMETERS        = "Engine/MaxParameters"       # int
+CARLA_KEY_ENGINE_RESET_XRUNS           = "Engine/ResetXruns"          # bool
 CARLA_KEY_ENGINE_UI_BRIDGES_TIMEOUT    = "Engine/UiBridgesTimeout"    # int
 
+CARLA_KEY_OSC_ENABLED          = "OSC/Enabled"
+CARLA_KEY_OSC_TCP_PORT_ENABLED = "OSC/TCPEnabled"
+CARLA_KEY_OSC_TCP_PORT_NUMBER  = "OSC/TCPNumber"
+CARLA_KEY_OSC_TCP_PORT_RANDOM  = "OSC/TCPRandom"
+CARLA_KEY_OSC_UDP_PORT_ENABLED = "OSC/UDPEnabled"
+CARLA_KEY_OSC_UDP_PORT_NUMBER  = "OSC/UDPNumber"
+CARLA_KEY_OSC_UDP_PORT_RANDOM  = "OSC/UDPRandom"
+
+CARLA_KEY_PATHS_AUDIO = "Paths/Audio"
+CARLA_KEY_PATHS_MIDI  = "Paths/MIDI"
+
 CARLA_KEY_PATHS_LADSPA = "Paths/LADSPA"
 CARLA_KEY_PATHS_DSSI   = "Paths/DSSI"
 CARLA_KEY_PATHS_LV2    = "Paths/LV2"
 CARLA_KEY_PATHS_VST2   = "Paths/VST2"
+CARLA_KEY_PATHS_VST3   = "Paths/VST3"
 CARLA_KEY_PATHS_SF2    = "Paths/SF2"
 CARLA_KEY_PATHS_SFZ    = "Paths/SFZ"
 
@@ -268,6 +264,8 @@ CARLA_DEFAULT_CANVAS_FANCY_EYE_CANDY   = False
 CARLA_DEFAULT_CANVAS_USE_OPENGL        = False
 CARLA_DEFAULT_CANVAS_ANTIALIASING      = CANVAS_ANTIALIASING_SMALL
 CARLA_DEFAULT_CANVAS_HQ_ANTIALIASING   = False
+CARLA_DEFAULT_CANVAS_INLINE_DISPLAYS   = False
+CARLA_DEFAULT_CANVAS_FULL_REPAINTS     = False
 
 # Engine
 CARLA_DEFAULT_FORCE_STEREO          = False
@@ -276,6 +274,7 @@ CARLA_DEFAULT_PREFER_UI_BRIDGES     = True
 CARLA_DEFAULT_MANAGE_UIS            = True
 CARLA_DEFAULT_UIS_ALWAYS_ON_TOP     = False
 CARLA_DEFAULT_MAX_PARAMETERS        = MAX_DEFAULT_PARAMETERS
+CARLA_DEFAULT_RESET_XRUNS           = False
 CARLA_DEFAULT_UI_BRIDGES_TIMEOUT    = 4000
 
 CARLA_DEFAULT_AUDIO_BUFFER_SIZE     = 512
@@ -286,16 +285,27 @@ if WINDOWS:
     CARLA_DEFAULT_AUDIO_DRIVER = "DirectSound"
 elif MACOS:
     CARLA_DEFAULT_AUDIO_DRIVER = "CoreAudio"
-else:
+elif os.path.exists("/usr/bin/jackd") or os.path.exists("/usr/bin/jackdbus"):
     CARLA_DEFAULT_AUDIO_DRIVER = "JACK"
+else:
+    CARLA_DEFAULT_AUDIO_DRIVER = "PulseAudio"
 
-if LINUX:
+if CARLA_DEFAULT_AUDIO_DRIVER == "JACK":
     CARLA_DEFAULT_PROCESS_MODE   = ENGINE_PROCESS_MODE_MULTIPLE_CLIENTS
     CARLA_DEFAULT_TRANSPORT_MODE = ENGINE_TRANSPORT_MODE_JACK
 else:
     CARLA_DEFAULT_PROCESS_MODE   = ENGINE_PROCESS_MODE_PATCHBAY
     CARLA_DEFAULT_TRANSPORT_MODE = ENGINE_TRANSPORT_MODE_INTERNAL
 
+# OSC
+CARLA_DEFAULT_OSC_ENABLED = not WINDOWS
+CARLA_DEFAULT_OSC_TCP_PORT_ENABLED = True
+CARLA_DEFAULT_OSC_TCP_PORT_NUMBER  = 22752
+CARLA_DEFAULT_OSC_TCP_PORT_RANDOM  = False
+CARLA_DEFAULT_OSC_UDP_PORT_ENABLED = True
+CARLA_DEFAULT_OSC_UDP_PORT_NUMBER  = 22752
+CARLA_DEFAULT_OSC_UDP_PORT_RANDOM  = False
+
 # Wine
 CARLA_DEFAULT_WINE_EXECUTABLE      = "wine"
 CARLA_DEFAULT_WINE_AUTO_PREFIX     = True
@@ -312,6 +322,12 @@ CARLA_DEFAULT_EXPERIMENTAL_LV2_EXPORT            = False
 CARLA_DEFAULT_EXPERIMENTAL_PREVENT_BAD_BEHAVIOUR = False
 CARLA_DEFAULT_EXPERIMENTAL_LOAD_LIB_GLOBAL       = False
 
+# ------------------------------------------------------------------------------------------------------------
+# Default File Folders
+
+CARLA_DEFAULT_FILE_PATH_AUDIO = []
+CARLA_DEFAULT_FILE_PATH_MIDI  = []
+
 # ------------------------------------------------------------------------------------------------------------
 # Default Plugin Folders (get)
 
@@ -319,6 +335,7 @@ DEFAULT_LADSPA_PATH = ""
 DEFAULT_DSSI_PATH   = ""
 DEFAULT_LV2_PATH    = ""
 DEFAULT_VST2_PATH   = ""
+DEFAULT_VST3_PATH   = ""
 DEFAULT_SF2_PATH    = ""
 DEFAULT_SFZ_PATH    = ""
 
@@ -359,29 +376,39 @@ if WINDOWS:
     if kIs64bit:
         DEFAULT_VST2_PATH  += ";" + COMMONPROGRAMFILES + "\\VST2"
 
+    DEFAULT_VST3_PATH    = COMMONPROGRAMFILES + "\\VST3"
+
     DEFAULT_SF2_PATH     = APPDATA + "\\SF2"
     DEFAULT_SFZ_PATH     = APPDATA + "\\SFZ"
 
     if PROGRAMFILESx86:
         DEFAULT_LADSPA_PATH += ";" + PROGRAMFILESx86 + "\\LADSPA"
         DEFAULT_DSSI_PATH   += ";" + PROGRAMFILESx86 + "\\DSSI"
-        DEFAULT_VST2_PATH    += ";" + PROGRAMFILESx86 + "\\VstPlugins"
-        DEFAULT_VST2_PATH    += ";" + PROGRAMFILESx86 + "\\Steinberg\\VstPlugins"
+        DEFAULT_VST2_PATH   += ";" + PROGRAMFILESx86 + "\\VstPlugins"
+        DEFAULT_VST2_PATH   += ";" + PROGRAMFILESx86 + "\\Steinberg\\VstPlugins"
+
+    if COMMONPROGRAMFILESx86:
+        DEFAULT_VST3_PATH   += COMMONPROGRAMFILESx86 + "\\VST3"
 
 elif HAIKU:
     splitter = ":"
 
     DEFAULT_LADSPA_PATH  = HOME + "/.ladspa"
-    DEFAULT_LADSPA_PATH += ":/boot/common/add-ons/ladspa"
+    DEFAULT_LADSPA_PATH += ":/system/add-ons/media/ladspaplugins"
+    DEFAULT_LADSPA_PATH += ":/system/lib/ladspa"
 
     DEFAULT_DSSI_PATH    = HOME + "/.dssi"
-    DEFAULT_DSSI_PATH   += ":/boot/common/add-ons/dssi"
+    DEFAULT_DSSI_PATH   += ":/system/add-ons/media/dssiplugins"
+    DEFAULT_DSSI_PATH   += ":/system/lib/dssi"
 
     DEFAULT_LV2_PATH     = HOME + "/.lv2"
-    DEFAULT_LV2_PATH    += ":/boot/common/add-ons/lv2"
+    DEFAULT_LV2_PATH    += ":/system/add-ons/media/lv2plugins"
 
     DEFAULT_VST2_PATH    = HOME + "/.vst"
-    DEFAULT_VST2_PATH   += ":/boot/common/add-ons/vst"
+    DEFAULT_VST2_PATH   += ":/system/add-ons/media/vstplugins"
+
+    DEFAULT_VST3_PATH    = HOME + "/.vst3"
+    DEFAULT_VST3_PATH   += ":/system/add-ons/media/vst3plugins"
 
 elif MACOS:
     splitter = ":"
@@ -398,6 +425,9 @@ elif MACOS:
     DEFAULT_VST2_PATH    = HOME + "/Library/Audio/Plug-Ins/VST"
     DEFAULT_VST2_PATH   += ":/Library/Audio/Plug-Ins/VST"
 
+    DEFAULT_VST3_PATH    = HOME + "/Library/Audio/Plug-Ins/VST3"
+    DEFAULT_VST3_PATH   += ":/Library/Audio/Plug-Ins/VST3"
+
 else:
     splitter = ":"
 
@@ -417,8 +447,19 @@ else:
     DEFAULT_VST2_PATH   += ":/usr/lib/vst"
     DEFAULT_VST2_PATH   += ":/usr/local/lib/vst"
 
+    DEFAULT_VST2_PATH   += HOME + "/.lxvst"
+    DEFAULT_VST2_PATH   += ":/usr/lib/lxvst"
+    DEFAULT_VST2_PATH   += ":/usr/local/lib/lxvst"
+
+    DEFAULT_VST3_PATH    = HOME + "/.vst3"
+    DEFAULT_VST3_PATH   += ":/usr/lib/vst3"
+    DEFAULT_VST3_PATH   += ":/usr/local/lib/vst3"
+
     DEFAULT_SF2_PATH     = HOME + "/.sounds/sf2"
+    DEFAULT_SF2_PATH    += HOME + "/.sounds/sf3"
     DEFAULT_SF2_PATH    += ":/usr/share/sounds/sf2"
+    DEFAULT_SF2_PATH    += ":/usr/share/sounds/sf3"
+    DEFAULT_SF2_PATH    += ":/usr/share/soundfonts"
 
     DEFAULT_SFZ_PATH     = HOME + "/.sounds/sfz"
     DEFAULT_SFZ_PATH    += ":/usr/share/sounds/sfz"
@@ -431,9 +472,11 @@ if not WINDOWS:
 
     if os.path.exists(winePrefix):
         DEFAULT_VST2_PATH += ":" + winePrefix + "/drive_c/Program Files/VstPlugins"
+        DEFAULT_VST3_PATH += ":" + winePrefix + "/drive_c/Program Files/Common Files/VST3"
 
         if kIs64bit and os.path.exists(winePrefix + "/drive_c/Program Files (x86)"):
             DEFAULT_VST2_PATH += ":" + winePrefix + "/drive_c/Program Files (x86)/VstPlugins"
+            DEFAULT_VST3_PATH += ":" + winePrefix + "/drive_c/Program Files (x86)/Common Files/VST3"
 
     del winePrefix
 
@@ -463,6 +506,7 @@ if readEnvVars:
     CARLA_DEFAULT_DSSI_PATH   = os.getenv("DSSI_PATH",   DEFAULT_DSSI_PATH).split(splitter)
     CARLA_DEFAULT_LV2_PATH    = os.getenv("LV2_PATH",    DEFAULT_LV2_PATH).split(splitter)
     CARLA_DEFAULT_VST2_PATH   = os.getenv("VST_PATH",    DEFAULT_VST2_PATH).split(splitter)
+    CARLA_DEFAULT_VST3_PATH   = os.getenv("VST3_PATH",   DEFAULT_VST3_PATH).split(splitter)
     CARLA_DEFAULT_SF2_PATH    = os.getenv("SF2_PATH",    DEFAULT_SF2_PATH).split(splitter)
     CARLA_DEFAULT_SFZ_PATH    = os.getenv("SFZ_PATH",    DEFAULT_SFZ_PATH).split(splitter)
 
@@ -471,6 +515,7 @@ else:
     CARLA_DEFAULT_DSSI_PATH   = DEFAULT_DSSI_PATH.split(splitter)
     CARLA_DEFAULT_LV2_PATH    = DEFAULT_LV2_PATH.split(splitter)
     CARLA_DEFAULT_VST2_PATH   = DEFAULT_VST2_PATH.split(splitter)
+    CARLA_DEFAULT_VST3_PATH   = DEFAULT_VST3_PATH.split(splitter)
     CARLA_DEFAULT_SF2_PATH    = DEFAULT_SF2_PATH.split(splitter)
     CARLA_DEFAULT_SFZ_PATH    = DEFAULT_SFZ_PATH.split(splitter)
 
@@ -481,6 +526,7 @@ del DEFAULT_LADSPA_PATH
 del DEFAULT_DSSI_PATH
 del DEFAULT_LV2_PATH
 del DEFAULT_VST2_PATH
+del DEFAULT_VST3_PATH
 del DEFAULT_SF2_PATH
 del DEFAULT_SFZ_PATH
 
@@ -526,6 +572,23 @@ elif MACOS:
 else:
     DLL_EXTENSION = "so"
 
+# ------------------------------------------------------------------------------------------------------------
+# Find decimal points for a parameter, using step and stepSmall
+
+def countDecimalPoints(step, stepSmall):
+    if stepSmall >= 1.0:
+        return 0
+    if step >= 1.0:
+        return 2
+
+    count = 0
+    value = fmod(abs(stepSmall), 1)
+    while value > 0.001 and value < 0.99 and count < 6:
+        value = fmod(value*10, 1)
+        count += 1
+
+    return count
+
 # ------------------------------------------------------------------------------------------------------------
 # Check if a value is a number (float support)
 
@@ -594,7 +657,7 @@ def handleInitialCommandLineArguments(file):
 
             print("Using Carla version %s" % VERSION)
             print("  Python version: %s" % sys.version.split(" ",1)[0])
-            print("  Qt version:     %s" % qVersion())
+            print("  Qt version:     %s" % QT_VERSION_STR)
             print("  PyQt version:   %s" % PYQT_VERSION_STR)
             print("  Binary dir:     %s" % pathBinaries)
             print("  Resources dir:  %s" % pathResources)
@@ -698,11 +761,57 @@ def getAndSetPath(parent, lineEdit):
         lineEdit.setText(newPath)
     return newPath
 
+# ------------------------------------------------------------------------------------------------------------
+# Backwards-compatible horizontalAdvance/width call, depending on Qt version
+
+def fontMetricsHorizontalAdvance(fontMetrics, string):
+    if QT_VERSION >= 0x51100:
+        return fontMetrics.horizontalAdvance(string)
+    return fontMetrics.width(string)
+
+# ------------------------------------------------------------------------------------------------------------
+# Custom QMessageBox which resizes itself to fit text
+
+class QMessageBoxWithBetterWidth(QMessageBox):
+    def __init__(self, parent):
+        QMessageBox.__init__(self, parent)
+
+    def showEvent(self, event):
+        fontMetrics = self.fontMetrics()
+
+        lines = self.text().strip().split("\n") + self.informativeText().strip().split("\n")
+
+        if len(lines) > 0:
+            width = 0
+
+            for line in lines:
+                width = max(fontMetricsHorizontalAdvance(fontMetrics, line), width)
+
+            self.layout().setColumnMinimumWidth(2, width + 12)
+
+        QMessageBox.showEvent(self, event)
+
+# ------------------------------------------------------------------------------------------------------------
+# Safer QSettings class, which does not throw if type mismatches
+
+class QSafeSettings(QSettings):
+    def value(self, key, defaultValue, type):
+        if not isinstance(defaultValue, type):
+            print("QSafeSettings.value() - defaultValue type mismatch for key", key)
+
+        try:
+            return QSettings.value(self, key, defaultValue, type)
+        except:
+            return defaultValue
+
 # ------------------------------------------------------------------------------------------------------------
 # Custom MessageBox
 
-def CustomMessageBox(parent, icon, title, text, extraText="", buttons=QMessageBox.Yes|QMessageBox.No, defButton=QMessageBox.No):
-    msgBox = QMessageBox(parent)
+def CustomMessageBox(parent, icon, title, text,
+                     extraText="",
+                     buttons=QMessageBox.Yes|QMessageBox.No,
+                     defButton=QMessageBox.No):
+    msgBox = QMessageBoxWithBetterWidth(parent)
     msgBox.setIcon(icon)
     msgBox.setWindowTitle(title)
     msgBox.setText(text)
