@@ -1,6 +1,6 @@
 /*
  * Carla Native Plugin API (C++)
- * Copyright (C) 2012-2014 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,7 +27,7 @@
  * @{
  */
 
-// -----------------------------------------------------------------------
+// --------------------------------------------------------------------------------------------------------------------
 // Native Plugin Class
 
 class NativePluginClass
@@ -113,6 +113,17 @@ protected:
         pHost->ui_parameter_changed(pHost->handle, index, value);
     }
 
+    void uiParameterTouch(const uint32_t index, const bool touch) const
+    {
+        CARLA_SAFE_ASSERT_RETURN(pHost != nullptr,);
+
+        pHost->dispatcher(pHost->handle,
+                          NATIVE_HOST_OPCODE_UI_TOUCH_PARAMETER,
+                          static_cast<int32_t>(index),
+                          touch ? 1 : 0,
+                          nullptr, 0.0f);
+    }
+
     void uiMidiProgramChanged(const uint8_t channel, const uint32_t bank, const uint32_t program) const
     {
         CARLA_SAFE_ASSERT_RETURN(pHost != nullptr,);
@@ -214,6 +225,31 @@ protected:
         pHost->dispatcher(pHost->handle, NATIVE_HOST_OPCODE_HOST_IDLE, 0, 0, nullptr, 0.0f);
     }
 
+    void hostRequestIdle() const
+    {
+        CARLA_SAFE_ASSERT_RETURN(pHost != nullptr,);
+
+        pHost->dispatcher(pHost->handle, NATIVE_HOST_OPCODE_REQUEST_IDLE, 0, 0, nullptr, 0.0f);
+    }
+
+    void hostQueueDrawInlineDisplay()
+    {
+        CARLA_SAFE_ASSERT_RETURN(pHost != nullptr,);
+
+        pHost->dispatcher(pHost->handle, NATIVE_HOST_OPCODE_QUEUE_INLINE_DISPLAY, 0, 0, nullptr, 0.0f);
+    }
+
+    const char* hostGetFilePath(const char* const filetype) const
+    {
+        CARLA_SAFE_ASSERT_RETURN(pHost != nullptr, nullptr);
+
+        return (const char*)(uintptr_t)pHost->dispatcher(pHost->handle,
+                                                         NATIVE_HOST_OPCODE_GET_FILE_PATH,
+                                                         0, 0,
+                                                         (void*)const_cast<char*>(filetype),
+                                                         0.0f);
+    }
+
     // -------------------------------------------------------------------
     // Plugin parameter calls
 
@@ -282,7 +318,8 @@ protected:
 
     virtual void deactivate() {}
 
-    virtual void process(float** const inBuffer, float** const outBuffer, const uint32_t frames, const NativeMidiEvent* const midiEvents, const uint32_t midiEventCount) = 0;
+    virtual void process(const float* const* inBuffer, float** outBuffer, uint32_t frames,
+                         const NativeMidiEvent* midiEvents, uint32_t midiEventCount) = 0;
 
     // -------------------------------------------------------------------
     // Plugin UI calls
@@ -368,6 +405,15 @@ protected:
         CARLA_SAFE_ASSERT_RETURN(uiName != nullptr && uiName[0] != '\0',);
     }
 
+    virtual const NativeInlineDisplayImageSurface* renderInlineDisplay(const uint32_t width, const uint32_t height)
+    {
+        CARLA_SAFE_ASSERT_RETURN(width > 0 && height > 0, nullptr);
+
+        return nullptr;
+    }
+
+    virtual void idle() {}
+
     // -------------------------------------------------------------------
 
 private:
@@ -454,7 +500,10 @@ public:
         handlePtr->deactivate();
     }
 
-    static void _process(NativePluginHandle handle, float** inBuffer, float** outBuffer, const uint32_t frames, const NativeMidiEvent* midiEvents, uint32_t midiEventCount)
+    // FIXME for v3.0, use const for the input buffer
+    static void _process(NativePluginHandle handle,
+                         float** inBuffer, float** outBuffer, const uint32_t frames,
+                         const NativeMidiEvent* midiEvents, uint32_t midiEventCount)
     {
         handlePtr->process(inBuffer, outBuffer, frames, midiEvents, midiEventCount);
     }
@@ -469,7 +518,8 @@ public:
         handlePtr->setState(data);
     }
 
-    static intptr_t _dispatcher(NativePluginHandle handle, NativePluginDispatcherOpcode opcode, int32_t index, intptr_t value, void* ptr, float opt)
+    static intptr_t _dispatcher(NativePluginHandle handle,
+                                NativePluginDispatcherOpcode opcode, int32_t index, intptr_t value, void* ptr, float opt)
     {
         switch(opcode)
         {
@@ -492,6 +542,9 @@ public:
             return 0;
         case NATIVE_PLUGIN_OPCODE_GET_INTERNAL_HANDLE:
             return 0;
+        case NATIVE_PLUGIN_OPCODE_IDLE:
+            handlePtr->idle();
+            return 0;
         }
 
         return 0;
@@ -500,6 +553,11 @@ public:
         (void)index;
     }
 
+    static const NativeInlineDisplayImageSurface* _render_inline_display(NativePluginHandle handle, uint32_t width, uint32_t height)
+    {
+        return handlePtr->renderInlineDisplay(width, height);
+    }
+
     #undef handlePtr
 
     CARLA_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(NativePluginClass)
@@ -508,7 +566,24 @@ public:
 
 /**@}*/
 
-// -----------------------------------------------------------------------
+// --------------------------------------------------------------------------------------------------------------------
+// -Weffc++ compat ext widget
+
+extern "C" {
+
+typedef struct _NativeInlineDisplayImageSurfaceCompat {
+    unsigned char* data;
+    int width, height, stride;
+    size_t dataSize;
+
+  _NativeInlineDisplayImageSurfaceCompat() noexcept
+      : data(nullptr), width(0), height(0), stride(0), dataSize(0) {}
+
+} NativeInlineDisplayImageSurfaceCompat;
+
+}
+
+// --------------------------------------------------------------------------------------------------------------------
 
 #define PluginClassEND(ClassName)                                            \
 public:                                                                      \
@@ -542,8 +617,10 @@ public:                                                                      \
     ClassName::_process,                \
     ClassName::_get_state,              \
     ClassName::_set_state,              \
-    ClassName::_dispatcher
+    ClassName::_dispatcher,             \
+    ClassName::_render_inline_display,  \
+    0, 0, nullptr, nullptr
 
-// -----------------------------------------------------------------------
+// --------------------------------------------------------------------------------------------------------------------
 
 #endif // CARLA_NATIVE_HPP_INCLUDED
