@@ -1,6 +1,6 @@
 /*
  * Custom types to store LV2 information
- * Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,6 +31,15 @@ typedef const char* LV2_URI;
 typedef uint32_t LV2_Property;
 #define LV2UI_INVALID_PORT_INDEX ((uint32_t)-1)
 
+// Parameter Types
+#define LV2_PARAMETER_BOOL              1
+#define LV2_PARAMETER_INT               2
+#define LV2_PARAMETER_LONG              3
+#define LV2_PARAMETER_FLOAT             4
+#define LV2_PARAMETER_DOUBLE            5
+#define LV2_PARAMETER_PATH              6
+#define LV2_PARAMETER_STRING            7
+
 // Port Midi Map Types
 #define LV2_PORT_MIDI_MAP_CC             1
 #define LV2_PORT_MIDI_MAP_NRPN           2
@@ -215,6 +224,7 @@ typedef uint32_t LV2_Property;
 #define LV2_IS_UI_PORT_PROTOCOL_PEAK(x)  ((x) == LV2_UI_PORT_PROTOCOL_PEAK)
 
 // UI Types
+#define LV2_UI_NONE                      0
 #define LV2_UI_GTK2                      1
 #define LV2_UI_GTK3                      2
 #define LV2_UI_QT4                       3
@@ -224,6 +234,7 @@ typedef uint32_t LV2_Property;
 #define LV2_UI_X11                       7
 #define LV2_UI_EXTERNAL                  8
 #define LV2_UI_OLD_EXTERNAL              9
+#define LV2_UI_MOD                       10
 
 #define LV2_IS_UI_GTK2(x)                ((x) == LV2_UI_GTK2)
 #define LV2_IS_UI_GTK3(x)                ((x) == LV2_UI_GTK3)
@@ -234,6 +245,7 @@ typedef uint32_t LV2_Property;
 #define LV2_IS_UI_X11(x)                 ((x) == LV2_UI_X11)
 #define LV2_IS_UI_EXTERNAL(x)            ((x) == LV2_UI_EXTERNAL)
 #define LV2_IS_UI_OLD_EXTERNAL(x)        ((x) == LV2_UI_OLD_EXTERNAL)
+#define LV2_IS_UI_MOD(x)                 ((x) == LV2_UI_MOD)
 
 // Plugin Types
 #define LV2_PLUGIN_DELAY                 0x000001
@@ -391,6 +403,8 @@ struct LV2_RDF_Port {
     LV2_Property Designation;
     const char* Name;
     const char* Symbol;
+    const char* Comment;
+    const char* GroupURI;
 
     LV2_RDF_PortMidiMap MidiMap;
     LV2_RDF_PortPoints Points;
@@ -407,6 +421,8 @@ struct LV2_RDF_Port {
           Designation(0),
           Name(nullptr),
           Symbol(nullptr),
+          Comment(nullptr),
+          GroupURI(nullptr),
           MidiMap(),
           Points(),
           Unit(),
@@ -426,6 +442,16 @@ struct LV2_RDF_Port {
             delete[] Symbol;
             Symbol = nullptr;
         }
+        if (Comment != nullptr)
+        {
+            delete[] Comment;
+            Comment = nullptr;
+        }
+        if (GroupURI != nullptr)
+        {
+            delete[] GroupURI;
+            GroupURI = nullptr;
+        }
         if (ScalePoints != nullptr)
         {
             delete[] ScalePoints;
@@ -436,6 +462,85 @@ struct LV2_RDF_Port {
     CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_Port)
 };
 
+// Port
+struct LV2_RDF_PortGroup {
+    LV2_URI URI; // shared value, do not deallocate
+    const char* Name;
+    const char* Symbol;
+
+    LV2_RDF_PortGroup() noexcept
+        : URI(nullptr),
+          Name(nullptr),
+          Symbol(nullptr) {}
+
+    ~LV2_RDF_PortGroup() noexcept
+    {
+        if (Name != nullptr)
+        {
+            delete[] Name;
+            Name = nullptr;
+        }
+        if (Symbol != nullptr)
+        {
+            delete[] Symbol;
+            Symbol = nullptr;
+        }
+    }
+
+    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_PortGroup)
+};
+
+// Parameter
+struct LV2_RDF_Parameter {
+    LV2_URI URI;
+    LV2_Property Type;
+    bool Input;
+    const char* Label;
+    const char* Comment;
+    const char* GroupURI;
+
+    LV2_RDF_PortMidiMap MidiMap;
+    LV2_RDF_PortPoints Points;
+    LV2_RDF_PortUnit Unit;
+
+    LV2_RDF_Parameter() noexcept
+        : URI(nullptr),
+          Type(0),
+          Input(true),
+          Label(nullptr),
+          Comment(nullptr),
+          GroupURI(nullptr),
+          MidiMap(),
+          Points(),
+          Unit() {}
+
+    ~LV2_RDF_Parameter() noexcept
+    {
+        if (URI != nullptr)
+        {
+            delete[] URI;
+            URI = nullptr;
+        }
+        if (Label != nullptr)
+        {
+            delete[] Label;
+            Label = nullptr;
+        }
+        if (Comment != nullptr)
+        {
+            delete[] Comment;
+            Comment = nullptr;
+        }
+        if (GroupURI != nullptr)
+        {
+            delete[] GroupURI;
+            GroupURI = nullptr;
+        }
+    }
+
+    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_Parameter)
+};
+
 // Preset
 struct LV2_RDF_Preset {
     LV2_URI URI;
@@ -593,6 +698,12 @@ struct LV2_RDF_Descriptor {
     uint32_t PortCount;
     LV2_RDF_Port* Ports;
 
+    uint32_t PortGroupCount;
+    LV2_RDF_PortGroup* PortGroups;
+
+    uint32_t ParameterCount;
+    LV2_RDF_Parameter* Parameters;
+
     uint32_t PresetCount;
     LV2_RDF_Preset* Presets;
 
@@ -615,6 +726,10 @@ struct LV2_RDF_Descriptor {
           UniqueID(0),
           PortCount(0),
           Ports(nullptr),
+          PortGroupCount(0),
+          PortGroups(nullptr),
+          ParameterCount(0),
+          Parameters(nullptr),
           PresetCount(0),
           Presets(nullptr),
           FeatureCount(0),
@@ -664,6 +779,16 @@ struct LV2_RDF_Descriptor {
             delete[] Ports;
             Ports = nullptr;
         }
+        if (PortGroups != nullptr)
+        {
+            delete[] PortGroups;
+            PortGroups = nullptr;
+        }
+        if (Parameters != nullptr)
+        {
+            delete[] Parameters;
+            Parameters = nullptr;
+        }
         if (Presets != nullptr)
         {
             delete[] Presets;
