@@ -58,15 +58,18 @@ typedef void (JACKSYM_API *JackSymPropertyChangeCallback)(jack_uuid_t, const cha
 typedef void        (JACKSYM_API *jacksym_get_version)(int*, int*, int*, int*);
 typedef const char* (JACKSYM_API *jacksym_get_version_string)(void);
 
-typedef jack_client_t* (JACKSYM_API *jacksym_client_open)(const char*, jack_options_t, jack_status_t*);
+typedef jack_client_t* (JACKSYM_API *jacksym_client_open)(const char*, jack_options_t, jack_status_t*, ...);
 typedef int            (JACKSYM_API *jacksym_client_close)(jack_client_t*);
 
 typedef int   (JACKSYM_API *jacksym_client_name_size)(void);
 typedef char* (JACKSYM_API *jacksym_get_client_name)(jack_client_t*);
 
+typedef char* (JACKSYM_API *jacksym_client_get_uuid)(jack_client_t*);
 typedef char* (JACKSYM_API *jacksym_get_uuid_for_client_name)(jack_client_t*, const char*);
 typedef char* (JACKSYM_API *jacksym_get_client_name_by_uuid)(jack_client_t*, const char*);
 
+typedef int (JACKBRIDGE_API *jacksym_uuid_parse)(const char*, jack_uuid_t*);
+
 typedef int (JACKSYM_API *jacksym_activate)(jack_client_t*);
 typedef int (JACKSYM_API *jacksym_deactivate)(jack_client_t*);
 typedef int (JACKSYM_API *jacksym_is_realtime)(jack_client_t*);
@@ -187,9 +190,12 @@ struct JackBridge {
     jacksym_client_name_size client_name_size_ptr;
     jacksym_get_client_name get_client_name_ptr;
 
+    jacksym_client_get_uuid client_get_uuid_ptr;
     jacksym_get_uuid_for_client_name get_uuid_for_client_name_ptr;
     jacksym_get_client_name_by_uuid get_client_name_by_uuid_ptr;
 
+    jacksym_uuid_parse uuid_parse_ptr;
+
     jacksym_activate activate_ptr;
     jacksym_deactivate deactivate_ptr;
     jacksym_is_realtime is_realtime_ptr;
@@ -301,8 +307,10 @@ struct JackBridge {
           client_close_ptr(nullptr),
           client_name_size_ptr(nullptr),
           get_client_name_ptr(nullptr),
+          client_get_uuid_ptr(nullptr),
           get_uuid_for_client_name_ptr(nullptr),
           get_client_name_by_uuid_ptr(nullptr),
+          uuid_parse_ptr(nullptr),
           activate_ptr(nullptr),
           deactivate_ptr(nullptr),
           is_realtime_ptr(nullptr),
@@ -422,9 +430,12 @@ struct JackBridge {
         LIB_SYMBOL(client_name_size)
         LIB_SYMBOL(get_client_name)
 
+        LIB_SYMBOL(client_get_uuid)
         LIB_SYMBOL(get_uuid_for_client_name)
         LIB_SYMBOL(get_client_name_by_uuid)
 
+        LIB_SYMBOL(uuid_parse)
+
         LIB_SYMBOL(activate)
         LIB_SYMBOL(deactivate)
         LIB_SYMBOL(is_realtime)
@@ -873,6 +884,18 @@ char* jackbridge_get_client_name(jack_client_t* client)
 
 // -----------------------------------------------------------------------------
 
+char* jackbridge_client_get_uuid(jack_client_t* client)
+{
+#if defined(JACKBRIDGE_DUMMY)
+#elif defined(JACKBRIDGE_DIRECT)
+    return jack_client_get_uuid(client);
+#else
+    if (const jacksym_client_get_uuid func = getBridgeInstance().client_get_uuid_ptr)
+        return func(client);
+#endif
+    return nullptr;
+}
+
 char* jackbridge_get_uuid_for_client_name(jack_client_t* client, const char* name)
 {
 #if defined(JACKBRIDGE_DUMMY)
@@ -899,6 +922,20 @@ char* jackbridge_get_client_name_by_uuid(jack_client_t* client, const char* uuid
 
 // -----------------------------------------------------------------------------
 
+bool jackbridge_uuid_parse(const char* buf, jack_uuid_t* uuid)
+{
+#if defined(JACKBRIDGE_DUMMY)
+#elif defined(JACKBRIDGE_DIRECT)
+    return (jack_uuid_parse(buf, uuid) == 0);
+#else
+    if (const jacksym_uuid_parse func = getBridgeInstance().uuid_parse_ptr)
+        return (func(buf, uuid) == 0);
+#endif
+    return false;
+}
+
+// -----------------------------------------------------------------------------
+
 bool jackbridge_activate(jack_client_t* client)
 {
 #if defined(JACKBRIDGE_DUMMY)
@@ -1274,7 +1311,9 @@ jack_port_t* jackbridge_port_register(jack_client_t* client, const char* port_na
     return jack_port_register(client, port_name, port_type, flags, buffer_size);
 #else
     if (getBridgeInstance().port_register_ptr != nullptr)
-        return getBridgeInstance().port_register_ptr(client, port_name, port_type, flags, buffer_size);
+        return getBridgeInstance().port_register_ptr(client, port_name, port_type,
+                                                     static_cast<ulong>(flags),
+                                                     static_cast<ulong>(buffer_size));
 #endif
     return nullptr;
 }
@@ -1600,7 +1639,7 @@ uint32_t jackbridge_port_type_get_buffer_size(jack_client_t* client, const char*
 {
 #if defined(JACKBRIDGE_DUMMY)
 #elif defined(JACKBRIDGE_DIRECT)
-    return jack_port_type_get_buffer_size(client, port_type);
+    return static_cast<uint32_t>(jack_port_type_get_buffer_size(client, port_type));
 #else
     if (getBridgeInstance().port_type_get_buffer_size_ptr != nullptr)
         return static_cast<uint32_t>(getBridgeInstance().port_type_get_buffer_size_ptr(client, port_type));
@@ -1617,7 +1656,9 @@ void jackbridge_port_get_latency_range(jack_port_t* port, uint32_t mode, jack_la
     return jack_port_get_latency_range(port, static_cast<jack_latency_callback_mode_t>(mode), range);
 #else
     if (getBridgeInstance().port_get_latency_range_ptr != nullptr)
-        return getBridgeInstance().port_get_latency_range_ptr(port, static_cast<jack_latency_callback_mode_t>(mode), range);
+        return getBridgeInstance().port_get_latency_range_ptr(port,
+                                                              static_cast<jack_latency_callback_mode_t>(mode),
+                                                              range);
 #endif
     range->min = 0;
     range->max = 0;
@@ -1630,7 +1671,9 @@ void jackbridge_port_set_latency_range(jack_port_t* port, uint32_t mode, jack_la
     jack_port_set_latency_range(port, static_cast<jack_latency_callback_mode_t>(mode), range);
 #else
     if (getBridgeInstance().port_set_latency_range_ptr != nullptr)
-        getBridgeInstance().port_set_latency_range_ptr(port, static_cast<jack_latency_callback_mode_t>(mode), range);
+        getBridgeInstance().port_set_latency_range_ptr(port,
+                                                       static_cast<jack_latency_callback_mode_t>(mode),
+                                                       range);
 #endif
 }
 
@@ -1655,7 +1698,8 @@ const char** jackbridge_get_ports(jack_client_t* client, const char* port_name_p
     return jack_get_ports(client, port_name_pattern, type_name_pattern, flags);
 #else
     if (getBridgeInstance().get_ports_ptr != nullptr)
-        return getBridgeInstance().get_ports_ptr(client, port_name_pattern, type_name_pattern, flags);
+        return getBridgeInstance().get_ports_ptr(client, port_name_pattern, type_name_pattern,
+                                                 static_cast<ulong>(flags));
 #endif
     return nullptr;
 }
