@@ -1,6 +1,6 @@
 /*
  * Carla JACK API for external applications
- * Copyright (C) 2016-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2016-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,22 +18,39 @@
 #include "libjack.hpp"
 
 #include "CarlaThread.hpp"
+#include "CarlaJuceUtils.hpp"
 
 #include <signal.h>
 #include <sys/prctl.h>
 #include <sys/time.h>
 
-typedef int (*CarlaInterposedCallback)(int, void*);
+// ---------------------------------------------------------------------------------------------------------------------
 
 CARLA_EXPORT
-int jack_carla_interposed_action(int, int, void*)
+int jack_carla_interposed_action(uint, uint, void*)
 {
-    carla_stderr2("Non-export jack_carla_interposed_action called, this should not happen!!");
+    static bool printWarning = true;
+
+    if (printWarning)
+    {
+        printWarning = false;
+        carla_stderr2("Non-exported jack_carla_interposed_action called, this should not happen!!");
+        carla_stderr("Printing some info:");
+        carla_stderr("\tLD_LIBRARY_PATH: '%s'", std::getenv("LD_LIBRARY_PATH"));
+        carla_stderr("\tLD_PRELOAD:      '%s'", std::getenv("LD_PRELOAD"));
+        std::fflush(stderr);
+    }
+
+    // ::kill(::getpid(), SIGKILL);
     return 1337;
 }
 
+// ---------------------------------------------------------------------------------------------------------------------
+
 CARLA_BACKEND_START_NAMESPACE
 
+// ---------------------------------------------------------------------------------------------------------------------
+
 static int64_t getCurrentTimeMilliseconds() noexcept
 {
     struct timeval tv;
@@ -41,6 +58,8 @@ static int64_t getCurrentTimeMilliseconds() noexcept
     return ((int64_t) tv.tv_sec) * 1000 + tv.tv_usec / 1000;
 }
 
+static int carla_interposed_callback(int, void*);
+
 // ---------------------------------------------------------------------------------------------------------------------
 
 class CarlaJackRealtimeThread : public CarlaThread
@@ -95,8 +114,6 @@ private:
     CARLA_DECLARE_NON_COPY_CLASS(CarlaJackNonRealtimeThread)
 };
 
-static int carla_interposed_callback(int, void*);
-
 // ---------------------------------------------------------------------------------------------------------------------
 
 class CarlaJackAppClient : public CarlaJackRealtimeThread::Callback,
@@ -105,18 +122,20 @@ class CarlaJackAppClient : public CarlaJackRealtimeThread::Callback,
 public:
     JackServerState fServer;
     LinkedList<JackClientState*> fClients;
+    LinkedList<JackClientState*> fNewClients;
 
     CarlaJackAppClient()
         : fServer(this),
           fClients(),
+          fNewClients(),
           fShmAudioPool(),
           fShmRtClientControl(),
           fShmNonRtClientControl(),
           fShmNonRtServerControl(),
           fAudioPoolCopy(nullptr),
           fAudioTmpBuf(nullptr),
-          fDummyMidiInBuffer(true, "ignored"),
-          fDummyMidiOutBuffer(false, "ignored"),
+          fDummyMidiInBuffer(true),
+          fDummyMidiOutBuffer(false),
           fMidiInBuffers(nullptr),
           fMidiOutBuffers(nullptr),
           fIsOffline(false),
@@ -125,15 +144,19 @@ public:
           fSessionManager(0),
           fSetupHints(0),
           fRealtimeThread(this),
-          fNonRealtimeThread(this)
+          fNonRealtimeThread(this),
+          fRealtimeThreadMutex()
+#ifdef DEBUG
+          ,leakDetector_CarlaJackAppClient()
+#endif
     {
         carla_debug("CarlaJackAppClient::CarlaJackAppClient()");
 
         const char* const shmIds(std::getenv("CARLA_SHM_IDS"));
-        CARLA_SAFE_ASSERT_RETURN(shmIds != nullptr && std::strlen(shmIds) == 6*4,);
+        CARLA_SAFE_ASSERT_INT2_RETURN(shmIds != nullptr && std::strlen(shmIds) == 6*4, std::strlen(shmIds), 6*4,);
 
         const char* const libjackSetup(std::getenv("CARLA_LIBJACK_SETUP"));
-        CARLA_SAFE_ASSERT_RETURN(libjackSetup != nullptr && std::strlen(libjackSetup) == 6,);
+        CARLA_SAFE_ASSERT_RETURN(libjackSetup != nullptr && std::strlen(libjackSetup) >= 6,);
 
         // make sure we don't get loaded again
         carla_unsetenv("CARLA_SHM_IDS");
@@ -144,9 +167,8 @@ public:
         for (int i=4; --i >= 0;) {
             CARLA_SAFE_ASSERT_RETURN(libjackSetup[i] >= '0' && libjackSetup[i] <= '0'+64,);
         }
-        for (int i=6; --i >= 4;) {
-            CARLA_SAFE_ASSERT_RETURN(libjackSetup[i] >= '0' && libjackSetup[i] < '0'+0x4f,);
-        }
+        CARLA_SAFE_ASSERT_RETURN(libjackSetup[4] >= '0' && libjackSetup[4] <= '0' + LIBJACK_SESSION_MANAGER_NSM,);
+        CARLA_SAFE_ASSERT_RETURN(libjackSetup[5] >= '0' && libjackSetup[5] < '0'+0x4f,);
 
         std::memcpy(fBaseNameAudioPool,          shmIds+6*0, 6);
         std::memcpy(fBaseNameRtClientControl,    shmIds+6*1, 6);
@@ -163,11 +185,19 @@ public:
         fServer.numMidiIns   = static_cast<uint8_t>(libjackSetup[2] - '0');
         fServer.numMidiOuts  = static_cast<uint8_t>(libjackSetup[3] - '0');
 
-        fSessionManager = libjackSetup[4] - '0';
-        fSetupHints     = libjackSetup[5] - '0';
+        fSessionManager = static_cast<uint>(libjackSetup[4] - '0');
+        fSetupHints     = static_cast<uint>(libjackSetup[5] - '0');
 
-        jack_carla_interposed_action(1, fSetupHints, (void*)carla_interposed_callback);
-        jack_carla_interposed_action(2, fSessionManager, nullptr);
+        if (fSetupHints & LIBJACK_FLAG_MIDI_OUTPUT_CHANNEL_MIXDOWN)
+            fServer.numMidiOuts = 16;
+
+        jack_carla_interposed_action(LIBJACK_INTERPOSER_ACTION_SET_HINTS_AND_CALLBACK,
+                                     fSetupHints,
+                                     (void*)carla_interposed_callback);
+
+        jack_carla_interposed_action(LIBJACK_INTERPOSER_ACTION_SET_SESSION_MANAGER,
+                                     fSessionManager,
+                                     nullptr);
 
         fNonRealtimeThread.startThread(false);
     }
@@ -217,6 +247,11 @@ public:
 
         if (! fClients.append(jclient))
             return false;
+        if (! fNewClients.append(jclient))
+        {
+            fClients.removeOne(jclient);
+            return false;
+        }
 
         jclient->activated = true;
         jclient->deactivated = false;
@@ -246,7 +281,7 @@ public:
 
         switch (cb_action)
         {
-        case 1: {
+        case LIBJACK_INTERPOSER_CALLBACK_UI_HIDE: {
             const CarlaMutexLocker cml(fShmNonRtServerControl.mutex);
             fShmNonRtServerControl.writeOpcode(kPluginBridgeNonRtServerUiClosed);
             fShmNonRtServerControl.commitWrite();
@@ -281,10 +316,10 @@ private:
     float* fAudioPoolCopy;
     float* fAudioTmpBuf;
 
-    JackMidiPortBuffer fDummyMidiInBuffer;
-    JackMidiPortBuffer fDummyMidiOutBuffer;
-    JackMidiPortBuffer* fMidiInBuffers;
-    JackMidiPortBuffer* fMidiOutBuffers;
+    JackMidiPortBufferDummy fDummyMidiInBuffer;
+    JackMidiPortBufferDummy fDummyMidiOutBuffer;
+    JackMidiPortBufferOnStack* fMidiInBuffers;
+    JackMidiPortBufferOnStack* fMidiOutBuffers;
 
     char fBaseNameAudioPool[6+1];
     char fBaseNameRtClientControl[6+1];
@@ -295,8 +330,8 @@ private:
     volatile bool fIsReady;
     int64_t fLastPingTime;
 
-    int fSessionManager;
-    int fSetupHints;
+    uint fSessionManager;
+    uint fSetupHints;
 
     CarlaJackRealtimeThread    fRealtimeThread;
     CarlaJackNonRealtimeThread fNonRealtimeThread;
@@ -364,7 +399,7 @@ bool CarlaJackAppClient::initSharedMemmory()
     CARLA_SAFE_ASSERT_RETURN(opcode == kPluginBridgeNonRtClientVersion, false);
 
     const uint32_t apiVersion = fShmNonRtClientControl.readUInt();
-    CARLA_SAFE_ASSERT_RETURN(apiVersion == CARLA_PLUGIN_BRIDGE_API_VERSION, false);
+    CARLA_SAFE_ASSERT_RETURN(apiVersion == CARLA_PLUGIN_BRIDGE_API_VERSION_CURRENT, false);
 
     const uint32_t shmRtClientDataSize = fShmNonRtClientControl.readUInt();
     CARLA_SAFE_ASSERT_INT2(shmRtClientDataSize == sizeof(BridgeRtClientData), shmRtClientDataSize, sizeof(BridgeRtClientData));
@@ -451,6 +486,20 @@ void CarlaJackAppClient::clearSharedMemory() noexcept
 
 bool CarlaJackAppClient::handleRtData()
 {
+    if (fNewClients.count() != 0)
+    {
+        for (LinkedList<JackClientState*>::Itenerator it = fNewClients.begin2(); it.valid(); it.next())
+        {
+            JackClientState* const jclient(it.getValue(nullptr));
+            CARLA_SAFE_ASSERT_CONTINUE(jclient != nullptr);
+
+            if (jclient->threadInitCb != nullptr)
+                jclient->threadInitCb(jclient->threadInitCbPtr);
+        }
+
+        fNewClients.clear();
+    }
+
     const BridgeRtClientControl::WaitHelper helper(fShmRtClientControl);
 
     if (! helper.ok)
@@ -506,7 +555,8 @@ bool CarlaJackAppClient::handleRtData()
                         JackClientState* const jclient(it.getValue(nullptr));
                         CARLA_SAFE_ASSERT_CONTINUE(jclient != nullptr);
 
-                        jclient->bufferSizeCb(fServer.bufferSize, jclient->bufferSizeCbPtr);
+                        if (jclient->bufferSizeCb != nullptr)
+                            jclient->bufferSizeCb(fServer.bufferSize, jclient->bufferSizeCbPtr);
                     }
 
                     delete[] fAudioTmpBuf;
@@ -530,16 +580,31 @@ bool CarlaJackAppClient::handleRtData()
                     JackClientState* const jclient(it.getValue(nullptr));
                     CARLA_SAFE_ASSERT_CONTINUE(jclient != nullptr);
 
-                    jclient->sampleRateCb(static_cast<uint32_t>(fServer.sampleRate), jclient->sampleRateCbPtr);
+                    if (jclient->sampleRateCb != nullptr)
+                        jclient->sampleRateCb(static_cast<uint32_t>(fServer.sampleRate), jclient->sampleRateCbPtr);
                 }
             }
         }   break;
 
-        case kPluginBridgeRtClientSetOnline:
-            // TODO inform changes
-            fIsOffline = fShmRtClientControl.readBool();
-            //offlineModeChanged(fIsOffline);
-            break;
+        case kPluginBridgeRtClientSetOnline: {
+            const bool offline = fShmRtClientControl.readBool();
+
+            if (fIsOffline != offline)
+            {
+                const CarlaMutexLocker cml(fRealtimeThreadMutex);
+
+                fIsOffline = offline;
+
+                for (LinkedList<JackClientState*>::Itenerator it = fClients.begin2(); it.valid(); it.next())
+                {
+                    JackClientState* const jclient(it.getValue(nullptr));
+                    CARLA_SAFE_ASSERT_CONTINUE(jclient != nullptr);
+
+                    if (jclient->freewheelCb != nullptr)
+                        jclient->freewheelCb(offline ? 1 : 0, jclient->freewheelCbPtr);
+                }
+            }
+        }   break;
 
         case kPluginBridgeRtClientControlEventParameter:
         case kPluginBridgeRtClientControlEventMidiBank:
@@ -554,17 +619,17 @@ bool CarlaJackAppClient::handleRtData()
             const uint8_t  size(fShmRtClientControl.readByte());
             CARLA_SAFE_ASSERT_BREAK(size > 0);
 
-            if (port >= fServer.numMidiIns || size > JackMidiPortBuffer::kMaxEventSize || ! fRealtimeThreadMutex.tryLock())
+            if (port >= fServer.numMidiIns || size > JackMidiPortBufferBase::kMaxEventSize || ! fRealtimeThreadMutex.tryLock())
             {
                 for (uint8_t i=0; i<size; ++i)
                     fShmRtClientControl.readByte();
                 break;
             }
 
-            JackMidiPortBuffer& midiPortBuf(fMidiInBuffers[port]);
+            JackMidiPortBufferOnStack& midiPortBuf(fMidiInBuffers[port]);
 
-            if (midiPortBuf.count < JackMidiPortBuffer::kMaxEventCount &&
-                midiPortBuf.bufferPoolPos + size < JackMidiPortBuffer::kBufferPoolSize)
+            if (midiPortBuf.count < JackMidiPortBufferBase::kMaxEventCount &&
+                midiPortBuf.bufferPoolPos + size < JackMidiPortBufferBase::kBufferPoolSize)
             {
                 jack_midi_event_t& ev(midiPortBuf.events[midiPortBuf.count++]);
 
@@ -587,15 +652,16 @@ bool CarlaJackAppClient::handleRtData()
 
             // TODO tell client of xrun in case buffersize does not match
 
-            // FIXME - lock if offline
-            const CarlaMutexTryLocker cmtl(fRealtimeThreadMutex);
+            const CarlaMutexTryLocker cmtl(fRealtimeThreadMutex, fIsOffline);
 
             if (cmtl.wasLocked())
             {
                 CARLA_SAFE_ASSERT_BREAK(fShmAudioPool.data != nullptr);
 
                 // mixdown is default, do buffer addition (for multiple clients) if requested
-                const bool doBufferAddition = fSetupHints & 0x10;
+                const bool doBufferAddition  = fSetupHints & LIBJACK_FLAG_AUDIO_BUFFERS_ADDITION;
+                // mixdown midi outputs based on channel if requested
+                const bool doMidiChanMixdown = fSetupHints & LIBJACK_FLAG_MIDI_OUTPUT_CHANNEL_MIXDOWN;
 
                 // location to start of audio outputs (shm buffer)
                 float* const fdataRealOuts = fShmAudioPool.data+(fServer.bufferSize*fServer.numAudioIns);
@@ -605,13 +671,18 @@ bool CarlaJackAppClient::handleRtData()
 
                 if (! fClients.isEmpty())
                 {
-                    // save tranport for all clients
+                    // save transport for all clients
                     const BridgeTimeInfo& bridgeTimeInfo(fShmRtClientControl.data->timeInfo);
 
+                    const bool transportChanged = fServer.playing != bridgeTimeInfo.playing ||
+                                                  (bridgeTimeInfo.playing && fServer.position.frame + frames != bridgeTimeInfo.frame);
+
                     fServer.playing        = bridgeTimeInfo.playing;
-                    fServer.position.frame = bridgeTimeInfo.frame;
+                    fServer.position.frame = static_cast<jack_nframes_t>(bridgeTimeInfo.frame);
                     fServer.position.usecs = bridgeTimeInfo.usecs;
 
+                    fServer.position.frame_rate = static_cast<jack_nframes_t>(fServer.sampleRate);
+
                     if (bridgeTimeInfo.validFlags & kPluginBridgeTimeInfoValidBBT)
                     {
                         fServer.position.valid = JackPositionBBT;
@@ -640,8 +711,7 @@ bool CarlaJackAppClient::handleRtData()
                         JackClientState* const jclient(it.getValue(nullptr));
                         CARLA_SAFE_ASSERT_CONTINUE(jclient != nullptr);
 
-                        // FIXME - lock if offline
-                        const CarlaMutexTryLocker cmtl2(jclient->mutex);
+                        const CarlaMutexTryLocker cmtl2(jclient->mutex, fIsOffline);
 
                         // check if we can process
                         if (cmtl2.wasNotLocked() || jclient->processCb == nullptr || ! jclient->activated)
@@ -654,6 +724,14 @@ bool CarlaJackAppClient::handleRtData()
                         }
                         else
                         {
+                            // report transport sync changes if needed
+                            if (transportChanged && jclient->syncCb != nullptr)
+                            {
+                                jclient->syncCb(fServer.playing ? JackTransportRolling : JackTransportStopped,
+                                                &fServer.position,
+                                                jclient->syncCbPtr);
+                            }
+
                             uint8_t i;
                             // direct access to shm buffer, used only for inputs
                             float* fdataReal = fShmAudioPool.data;
@@ -687,7 +765,7 @@ bool CarlaJackAppClient::handleRtData()
                             }
                             if (i < fServer.numAudioIns)
                             {
-                                const std::size_t remainingBufferSize = fServer.bufferSize * (fServer.numAudioIns - i);
+                                const std::size_t remainingBufferSize = fServer.bufferSize * static_cast<uint8_t>(fServer.numAudioIns - i);
                                 //fdataReal += remainingBufferSize;
                                 fdataCopy += remainingBufferSize;
                             }
@@ -695,7 +773,7 @@ bool CarlaJackAppClient::handleRtData()
                             // location to start of audio outputs
                             float* const fdataCopyOuts = fdataCopy;
 
-                            // set audio ouputs
+                            // set audio outputs
                             i = 0;
                             for (LinkedList<JackPortState*>::Itenerator it2 = jclient->audioOuts.begin2(); it2.valid(); it2.next())
                             {
@@ -715,7 +793,7 @@ bool CarlaJackAppClient::handleRtData()
                             }
                             if (i < fServer.numAudioOuts)
                             {
-                                const std::size_t remainingBufferSize = fServer.bufferSize * (fServer.numAudioOuts - i);
+                                const std::size_t remainingBufferSize = fServer.bufferSize * static_cast<uint8_t>(fServer.numAudioOuts - i);
                                 carla_zeroFloats(fdataCopy, remainingBufferSize);
                                 //fdataCopy += remainingBufferSize;
                             }
@@ -809,13 +887,13 @@ bool CarlaJackAppClient::handleRtData()
 
                 if (fServer.numMidiOuts > 0)
                 {
-                    uint8_t* midiData(fShmRtClientControl.data->midiOut);
+                    uint8_t* midiData = fShmRtClientControl.data->midiOut;
                     carla_zeroBytes(midiData, kBridgeBaseMidiOutHeaderSize);
                     std::size_t curMidiDataPos = 0;
 
                     for (uint8_t i=0; i<fServer.numMidiOuts; ++i)
                     {
-                        JackMidiPortBuffer& midiPortBuf(fMidiOutBuffers[i]);
+                        JackMidiPortBufferOnStack& midiPortBuf(fMidiOutBuffers[i]);
 
                         for (uint16_t j=0; j<midiPortBuf.count; ++j)
                         {
@@ -824,12 +902,16 @@ bool CarlaJackAppClient::handleRtData()
                             if (curMidiDataPos + kBridgeBaseMidiOutHeaderSize + jmevent.size >= kBridgeRtClientDataMidiOutSize)
                                 break;
 
+                            if (doMidiChanMixdown && MIDI_IS_CHANNEL_MESSAGE(jmevent.buffer[0]))
+                                jmevent.buffer[0] = static_cast<jack_midi_data_t>(
+                                                    (jmevent.buffer[0] & MIDI_STATUS_BIT) | (i & MIDI_CHANNEL_BIT));
+
                             // set time
                             *(uint32_t*)midiData = jmevent.time;
                             midiData += 4;
 
                             // set port
-                            *midiData++ = i;
+                            *midiData++ = doMidiChanMixdown ? 0 : i;
 
                             // set size
                             *midiData++ = static_cast<uint8_t>(jmevent.size);
@@ -842,16 +924,63 @@ bool CarlaJackAppClient::handleRtData()
                         }
                     }
 
+                    // make last event null, so server stops when reaching it
                     if (curMidiDataPos != 0 &&
                         curMidiDataPos + kBridgeBaseMidiOutHeaderSize < kBridgeRtClientDataMidiOutSize)
+                    {
                         carla_zeroBytes(midiData, kBridgeBaseMidiOutHeaderSize);
 
+                        // sort events in case of mixdown
+                        if (doMidiChanMixdown)
+                        {
+                            uint32_t time;
+                            uint8_t size, *midiDataPtr;
+                            uint8_t tmp[kBridgeBaseMidiOutHeaderSize + JackMidiPortBufferBase::kMaxEventSize];
+                            bool wasSorted = true;
+
+                            for (; wasSorted;)
+                            {
+                                midiDataPtr = fShmRtClientControl.data->midiOut;
+                                uint8_t* prevData = midiDataPtr;
+                                uint32_t prevTime = *(uint32_t*)midiDataPtr;
+                                uint8_t prevSize = *(midiDataPtr + 5);
+                                wasSorted = false;
+
+                                for (;;)
+                                {
+                                    time = *(uint32_t*)midiDataPtr;
+                                    size = *(midiDataPtr + 5); // time and port
+
+                                    if (size == 0)
+                                        break;
+
+                                    if (prevTime > time)
+                                    {
+                                        // copy previous data to a temporary place
+                                        std::memcpy(tmp, prevData, kBridgeBaseMidiOutHeaderSize + prevSize);
+                                        // override previous data with new one (shifting left)
+                                        std::memcpy(prevData, midiDataPtr, kBridgeBaseMidiOutHeaderSize + size);
+                                        // override new data with old one
+                                        std::memcpy(midiDataPtr, tmp, kBridgeBaseMidiOutHeaderSize + prevSize);
+                                        // done swapping, flag it
+                                        wasSorted = true;
+                                    }
+
+                                    prevTime = time;
+                                    prevSize = size;
+                                    prevData = midiDataPtr;
+                                    midiDataPtr += 6 + size;
+                                }
+                            }
+                        }
+                    }
                 }
             }
             else
             {
                 carla_stderr2("CarlaJackAppClient: fRealtimeThreadMutex tryLock failed");
             }
+            fServer.monotonic_frame += frames;
             break;
         }
 
@@ -893,7 +1022,8 @@ bool CarlaJackAppClient::handleNonRtData()
 
         case kPluginBridgeNonRtClientVersion: {
             const uint apiVersion = fShmNonRtServerControl.readUInt();
-            CARLA_SAFE_ASSERT_UINT2(apiVersion == CARLA_PLUGIN_BRIDGE_API_VERSION, apiVersion, CARLA_PLUGIN_BRIDGE_API_VERSION);
+            CARLA_SAFE_ASSERT_UINT2(apiVersion == CARLA_PLUGIN_BRIDGE_API_VERSION_CURRENT,
+                                    apiVersion, CARLA_PLUGIN_BRIDGE_API_VERSION_CURRENT);
         }   break;
 
         case kPluginBridgeNonRtClientPing: {
@@ -921,7 +1051,8 @@ bool CarlaJackAppClient::handleNonRtData()
 
         case kPluginBridgeNonRtClientSetParameterValue:
         case kPluginBridgeNonRtClientSetParameterMidiChannel:
-        case kPluginBridgeNonRtClientSetParameterMidiCC:
+        case kPluginBridgeNonRtClientSetParameterMappedControlIndex:
+        case kPluginBridgeNonRtClientSetParameterMappedRange:
         case kPluginBridgeNonRtClientSetProgram:
         case kPluginBridgeNonRtClientSetMidiProgram:
         case kPluginBridgeNonRtClientSetCustomData:
@@ -933,6 +1064,10 @@ bool CarlaJackAppClient::handleNonRtData()
             fShmNonRtClientControl.readBool();
             break;
 
+        case kPluginBridgeNonRtClientSetOptions:
+            fShmNonRtClientControl.readUInt();
+            break;
+
         case kPluginBridgeNonRtClientSetCtrlChannel:
             fShmNonRtClientControl.readShort();
             break;
@@ -943,7 +1078,7 @@ bool CarlaJackAppClient::handleNonRtData()
 
         case kPluginBridgeNonRtClientPrepareForSave:
             {
-                if (fSessionManager == 1) // auto
+                if (fSessionManager == LIBJACK_SESSION_MANAGER_AUTO && std::getenv("NSM_URL") == nullptr)
                 {
                     struct sigaction sig;
                     carla_zeroStruct(sig);
@@ -951,10 +1086,10 @@ bool CarlaJackAppClient::handleNonRtData()
                     sigaction(SIGUSR1, nullptr, &sig);
 
                     if (sig.sa_handler != nullptr)
-                        fSessionManager = 3;
+                        fSessionManager = LIBJACK_SESSION_MANAGER_LADISH;
                 }
 
-                if (fSessionManager == 3)
+                if (fSessionManager == LIBJACK_SESSION_MANAGER_LADISH)
                     ::kill(::getpid(), SIGUSR1);
 
                 const CarlaMutexLocker _cml(fShmNonRtServerControl.mutex);
@@ -968,7 +1103,7 @@ bool CarlaJackAppClient::handleNonRtData()
             break;
 
         case kPluginBridgeNonRtClientShowUI:
-            if (jack_carla_interposed_action(3, 1, nullptr) == 1337)
+            if (jack_carla_interposed_action(LIBJACK_INTERPOSER_ACTION_SHOW_HIDE_GUI, 1, nullptr) == 1337)
             {
                 // failed, LD_PRELOAD did not work?
                 const char* const message("Cannot show UI, LD_PRELOAD not working?");
@@ -987,7 +1122,7 @@ bool CarlaJackAppClient::handleNonRtData()
             break;
 
         case kPluginBridgeNonRtClientHideUI:
-            jack_carla_interposed_action(3, 0, nullptr);
+            jack_carla_interposed_action(LIBJACK_INTERPOSER_ACTION_SHOW_HIDE_GUI, 0, nullptr);
             break;
 
         case kPluginBridgeNonRtClientUiParameterChange:
@@ -1047,7 +1182,7 @@ void CarlaJackAppClient::runNonRealtimeThread()
 
     if (fServer.numMidiIns > 0)
     {
-        fMidiInBuffers = new JackMidiPortBuffer[fServer.numMidiIns];
+        fMidiInBuffers = new JackMidiPortBufferOnStack[fServer.numMidiIns];
 
         for (uint8_t i=0; i<fServer.numMidiIns; ++i)
             fMidiInBuffers[i].isInput = true;
@@ -1055,7 +1190,7 @@ void CarlaJackAppClient::runNonRealtimeThread()
 
     if (fServer.numMidiOuts > 0)
     {
-        fMidiOutBuffers = new JackMidiPortBuffer[fServer.numMidiOuts];
+        fMidiOutBuffers = new JackMidiPortBufferOnStack[fServer.numMidiOuts];
 
         for (uint8_t i=0; i<fServer.numMidiOuts; ++i)
             fMidiOutBuffers[i].isInput = false;
@@ -1092,7 +1227,7 @@ void CarlaJackAppClient::runNonRealtimeThread()
         */
     }
 
-    //callback(ENGINE_CALLBACK_ENGINE_STOPPED, 0, 0, 0, 0.0f, nullptr);
+    //callback(true, true, ENGINE_CALLBACK_ENGINE_STOPPED, 0, 0, 0, 0.0f, nullptr);
 
     if (quitReceived)
     {
@@ -1158,15 +1293,24 @@ void CarlaJackAppClient::runNonRealtimeThread()
     carla_debug("CarlaJackAppClient runNonRealtimeThread FINISHED");
 }
 
+CARLA_BACKEND_END_NAMESPACE
+
 // ---------------------------------------------------------------------------------------------------------------------
 
+using CarlaBackend::CarlaJackAppClient;
+using CarlaBackend::JackClientState;
+
 static CarlaJackAppClient gClient;
 
+CARLA_BACKEND_START_NAMESPACE
+
 static int carla_interposed_callback(int cb_action, void* ptr)
 {
     return gClient.handleInterposerCallback(cb_action, ptr);
 }
 
+CARLA_BACKEND_END_NAMESPACE
+
 // ---------------------------------------------------------------------------------------------------------------------
 
 CARLA_EXPORT
@@ -1247,8 +1391,6 @@ pthread_t jack_client_thread_id(jack_client_t* client)
     return jackAppPtr->getRealtimeThreadId();
 }
 
-CARLA_BACKEND_END_NAMESPACE
-
 // ---------------------------------------------------------------------------------------------------------------------
 
 #include "jackbridge/JackBridge2.cpp"
