@@ -3,7 +3,7 @@
 
    This file is part of the Water library.
    Copyright (c) 2016 ROLI Ltd.
-   Copyright (C) 2017 Filipe Coelho <falktx@falktx.com>
+   Copyright (C) 2017-2019 Filipe Coelho <falktx@falktx.com>
 
    Permission is granted to use this software under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license/
@@ -52,7 +52,7 @@ namespace water {
 
     @see OwnedArray, ReferenceCountedArray, StringArray, CriticalSection
 */
-template <typename ElementType, int minimumAllocatedSize = 0>
+template <typename ElementType, size_t minimumAllocatedSize = 0>
 class Array
 {
 private:
@@ -61,7 +61,9 @@ private:
 public:
     //==============================================================================
     /** Creates an empty array. */
-    Array() noexcept   : numUsed (0)
+    Array() noexcept
+        : data(),
+          numUsed(0)
     {
     }
 
@@ -69,7 +71,8 @@ public:
         @param other    the array to copy
     */
     Array (const Array<ElementType>& other) noexcept
-        : numUsed (0)
+        : data(),
+          numUsed(0)
     {
         CARLA_SAFE_ASSERT_RETURN(data.setAllocatedSize (other.numUsed),);
         numUsed = other.numUsed;
@@ -226,7 +229,7 @@ public:
     {
         if (isPositiveAndBelow (index, numUsed))
         {
-            jassert (data.elements != nullptr);
+            wassert (data.elements != nullptr);
             return data.elements [index];
         }
 
@@ -244,7 +247,7 @@ public:
     */
     inline ElementType getUnchecked (const int index) const
     {
-        jassert (isPositiveAndBelow (index, numUsed) && data.elements != nullptr);
+        wassert (isPositiveAndBelow (index, numUsed) && data.elements != nullptr);
         return data.elements [index];
     }
 
@@ -259,7 +262,7 @@ public:
     */
     inline ElementType& getReference (const int index) const noexcept
     {
-        jassert (isPositiveAndBelow (index, numUsed) && data.elements != nullptr);
+        wassert (isPositiveAndBelow (index, numUsed) && data.elements != nullptr);
         return data.elements [index];
     }
 
@@ -271,7 +274,7 @@ public:
     {
         if (numUsed > 0)
         {
-            jassert (data.elements != nullptr);
+            wassert (data.elements != nullptr);
             return data.elements[0];
         }
 
@@ -286,7 +289,7 @@ public:
     {
         if (numUsed > 0)
         {
-            jassert (data.elements != nullptr);
+            wassert (data.elements != nullptr);
             return data.elements[numUsed - 1];
         }
 
@@ -370,7 +373,7 @@ public:
     */
     bool add (const ElementType& newElement) noexcept
     {
-        if (! data.ensureAllocatedSize (numUsed + 1))
+        if (! data.ensureAllocatedSize (static_cast<size_t>(numUsed + 1)))
             return false;
 
         new (data.elements + numUsed++) ElementType (newElement);
@@ -385,7 +388,7 @@ public:
     */
     bool add (ElementType&& newElement) noexcept
     {
-        if (! data.ensureAllocatedSize (numUsed + 1))
+        if (! data.ensureAllocatedSize (static_cast<size_t>(numUsed + 1)))
             return false;
 
         new (data.elements + numUsed++) ElementType (static_cast<ElementType&&> (newElement));
@@ -416,7 +419,7 @@ public:
             const int numberToMove = numUsed - indexToInsertAt;
 
             if (numberToMove > 0)
-                std::memmove (insertPos + 1, insertPos, ((size_t) numberToMove) * sizeof (ElementType));
+                data.moveMemory (insertPos + 1, insertPos, numberToMove);
 
             new (insertPos) ElementType (newElement);
             ++numUsed;
@@ -441,19 +444,21 @@ public:
         @param numberOfTimesToInsertIt  how many copies of the value to insert
         @see insert, add, addSorted, set
     */
-    void insertMultiple (int indexToInsertAt, ParameterType newElement,
+    bool insertMultiple (int indexToInsertAt, ParameterType newElement,
                          int numberOfTimesToInsertIt)
     {
         if (numberOfTimesToInsertIt > 0)
         {
-            data.ensureAllocatedSize (numUsed + numberOfTimesToInsertIt);
+            if (! data.ensureAllocatedSize (numUsed + numberOfTimesToInsertIt))
+                return false;
+
             ElementType* insertPos;
 
             if (isPositiveAndBelow (indexToInsertAt, numUsed))
             {
                 insertPos = data.elements + indexToInsertAt;
                 const int numberToMove = numUsed - indexToInsertAt;
-                memmove (insertPos + numberOfTimesToInsertIt, insertPos, ((size_t) numberToMove) * sizeof (ElementType));
+                data.moveMemory (insertPos + numberOfTimesToInsertIt, insertPos, numberToMove);
             }
             else
             {
@@ -469,8 +474,11 @@ public:
                              // new statement to avoid a compiler bug in VS2014
             }
         }
+
+        return true;
     }
 
+#if 0
     /** Inserts an array of values into this array at a given position.
 
         If the index is less than 0 or greater than the size of the array, the
@@ -483,20 +491,22 @@ public:
         @param numberOfElements     how many items are in the array
         @see insert, add, addSorted, set
     */
-    void insertArray (int indexToInsertAt,
+    bool insertArray (int indexToInsertAt,
                       const ElementType* newElements,
                       int numberOfElements)
     {
         if (numberOfElements > 0)
         {
-            data.ensureAllocatedSize (numUsed + numberOfElements);
+            if (! data.ensureAllocatedSize (numUsed + numberOfElements))
+                return false;
+
             ElementType* insertPos = data.elements;
 
             if (isPositiveAndBelow (indexToInsertAt, numUsed))
             {
                 insertPos += indexToInsertAt;
                 const int numberToMove = numUsed - indexToInsertAt;
-                memmove (insertPos + numberOfElements, insertPos, (size_t) numberToMove * sizeof (ElementType));
+                std::memmove (insertPos + numberOfElements, insertPos, (size_t) numberToMove * sizeof (ElementType));
             }
             else
             {
@@ -508,7 +518,10 @@ public:
             while (--numberOfElements >= 0)
                 new (insertPos++) ElementType (*newElements++);
         }
+
+        return true;
     }
+#endif
 
     /** Appends a new element at the end of the array as long as the array doesn't
         already contain it.
@@ -538,11 +551,11 @@ public:
     */
     void set (const int indexToChange, ParameterType newValue)
     {
-        jassert (indexToChange >= 0);
+        wassert (indexToChange >= 0);
 
         if (isPositiveAndBelow (indexToChange, numUsed))
         {
-            jassert (data.elements != nullptr);
+            wassert (data.elements != nullptr);
             data.elements [indexToChange] = newValue;
         }
         else if (indexToChange >= 0)
@@ -563,7 +576,7 @@ public:
     */
     void setUnchecked (const int indexToChange, ParameterType newValue)
     {
-        jassert (isPositiveAndBelow (indexToChange, numUsed));
+        wassert (isPositiveAndBelow (indexToChange, numUsed));
         data.elements [indexToChange] = newValue;
     }
 
@@ -633,7 +646,7 @@ public:
     {
         if (startIndex < 0)
         {
-            jassertfalse;
+            wassertfalse;
             startIndex = 0;
         }
 
@@ -653,7 +666,7 @@ public:
     */
     void resize (const int targetNumItems)
     {
-        jassert (targetNumItems >= 0);
+        wassert (targetNumItems >= 0);
 
         const int numToAdd = targetNumItems - numUsed;
         if (numToAdd > 0)
@@ -748,7 +761,7 @@ public:
     {
         if (isPositiveAndBelow (indexToRemove, numUsed))
         {
-            jassert (data.elements != nullptr);
+            wassert (data.elements != nullptr);
             removeInternal (indexToRemove);
         }
     }
@@ -767,7 +780,7 @@ public:
     {
         if (isPositiveAndBelow (indexToRemove, numUsed))
         {
-            jassert (data.elements != nullptr);
+            wassert (data.elements != nullptr);
             ElementType removed (data.elements[indexToRemove]);
             removeInternal (indexToRemove);
             return removed;
@@ -788,13 +801,13 @@ public:
     */
     void remove (const ElementType* elementToRemove)
     {
-        jassert (elementToRemove != nullptr);
-        jassert (data.elements != nullptr);
+        wassert (elementToRemove != nullptr);
+        wassert (data.elements != nullptr);
         const int indexToRemove = int (elementToRemove - data.elements);
 
         if (! isPositiveAndBelow (indexToRemove, numUsed))
         {
-            jassertfalse;
+            wassertfalse;
             return;
         }
 
@@ -903,7 +916,7 @@ public:
 
             const int numToShift = numUsed - endIndex;
             if (numToShift > 0)
-                memmove (e, e + numberToRemove, ((size_t) numToShift) * sizeof (ElementType));
+                data.moveMemory (e, e + numberToRemove, numToShift);
 
             numUsed -= numberToRemove;
             minimiseStorageAfterRemoval();
@@ -994,50 +1007,6 @@ public:
         }
     }
 
-    /** Moves one of the values to a different position.
-
-        This will move the value to a specified index, shuffling along
-        any intervening elements as required.
-
-        So for example, if you have the array { 0, 1, 2, 3, 4, 5 } then calling
-        move (2, 4) would result in { 0, 1, 3, 4, 2, 5 }.
-
-        @param currentIndex     the index of the value to be moved. If this isn't a
-                                valid index, then nothing will be done
-        @param newIndex         the index at which you'd like this value to end up. If this
-                                is less than zero, the value will be moved to the end
-                                of the array
-    */
-    void move (const int currentIndex, int newIndex) noexcept
-    {
-        if (currentIndex != newIndex)
-        {
-            if (isPositiveAndBelow (currentIndex, numUsed))
-            {
-                if (! isPositiveAndBelow (newIndex, numUsed))
-                    newIndex = numUsed - 1;
-
-                char tempCopy [sizeof (ElementType)];
-                memcpy (tempCopy, data.elements + currentIndex, sizeof (ElementType));
-
-                if (newIndex > currentIndex)
-                {
-                    memmove (data.elements + currentIndex,
-                             data.elements + currentIndex + 1,
-                             sizeof (ElementType) * (size_t) (newIndex - currentIndex));
-                }
-                else
-                {
-                    memmove (data.elements + newIndex + 1,
-                             data.elements + newIndex,
-                             sizeof (ElementType) * (size_t) (currentIndex - newIndex));
-                }
-
-                memcpy (data.elements + newIndex, tempCopy, sizeof (ElementType));
-            }
-        }
-    }
-
     //==============================================================================
     /** Reduces the amount of storage being used by the array.
 
@@ -1120,7 +1089,7 @@ private:
         const int numberToShift = numUsed - indexToRemove;
 
         if (numberToShift > 0)
-            memmove (e, e + 1, ((size_t) numberToShift) * sizeof (ElementType));
+            data.moveMemory (e, e + 1, static_cast<size_t>(numberToShift));
 
         minimiseStorageAfterRemoval();
     }
@@ -1133,8 +1102,12 @@ private:
 
     void minimiseStorageAfterRemoval()
     {
-        if (data.numAllocated > jmax (minimumAllocatedSize, numUsed * 2))
-            data.shrinkToNoMoreThan (jmax (numUsed, jmax (minimumAllocatedSize, 64 / (int) sizeof (ElementType))));
+        CARLA_SAFE_ASSERT_RETURN(numUsed >= 0,);
+
+        const size_t snumUsed = static_cast<size_t>(numUsed);
+
+        if (data.numAllocated > jmax (minimumAllocatedSize, snumUsed * 2U))
+            data.shrinkToNoMoreThan (jmax (snumUsed, jmax (minimumAllocatedSize, 64U / sizeof (ElementType))));
     }
 };
 
