@@ -1,6 +1,6 @@
 /*
  * Carla Native Plugins
- * Copyright (C) 2012-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2020 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,18 +19,54 @@
 #include "CarlaMIDI.h"
 #include "CarlaUtils.hpp"
 
-#undef DESCFUNCS
-#define DESCFUNCS \
+#undef DESCFUNCS_WITHCV
+#undef DESCFUNCS_WITHOUTCV
+#define DESCFUNCS_WITHCV \
     nullptr, nullptr, nullptr, nullptr, nullptr, \
     nullptr, nullptr, nullptr, nullptr, nullptr, \
     nullptr, nullptr, nullptr, nullptr, nullptr, \
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr
+    nullptr, nullptr, nullptr, nullptr, nullptr, \
+    nullptr, nullptr
+#define DESCFUNCS_WITHOUTCV \
+    DESCFUNCS_WITHCV, 0, 0, nullptr, nullptr
 
 static const NativePluginDescriptor sNativePluginDescriptors[] = {
 
 // --------------------------------------------------------------------------------------------------------------------
 // Simple plugins
 
+{
+    /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
+    /* hints     */ NATIVE_PLUGIN_IS_RTSAFE,
+    /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
+    /* audioIns  */ 1,
+    /* audioOuts */ 1,
+    /* midiIns   */ 0,
+    /* midiOuts  */ 0,
+    /* paramIns  */ 1,
+    /* paramOuts */ 0,
+    /* name      */ "Audio Gain (Mono)",
+    /* label     */ "audiogain",
+    /* maker     */ "falkTX",
+    /* copyright */ "GNU GPL v2+",
+    DESCFUNCS_WITHOUTCV
+},
+{
+    /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
+    /* hints     */ NATIVE_PLUGIN_IS_RTSAFE,
+    /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
+    /* audioIns  */ 2,
+    /* audioOuts */ 2,
+    /* midiIns   */ 0,
+    /* midiOuts  */ 0,
+    /* paramIns  */ 3,
+    /* paramOuts */ 0,
+    /* name      */ "Audio Gain (Stereo)",
+    /* label     */ "audiogain_s",
+    /* maker     */ "falkTX",
+    /* copyright */ "GNU GPL v2+",
+    DESCFUNCS_WITHOUTCV
+},
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_NONE,
     /* hints     */ NATIVE_PLUGIN_IS_RTSAFE,
@@ -45,7 +81,27 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "bypass",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
+},
+{
+    /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
+    /* hints     */ NATIVE_PLUGIN_IS_RTSAFE,
+    /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
+    /* audioIns  */ 0,
+    /* audioOuts */ 1,
+    /* midiIns   */ 0,
+    /* midiOuts  */ 0,
+    /* paramIns  */ 1,
+    /* paramOuts */ 0,
+    /* name      */ "CV to Audio",
+    /* label     */ "cv2audio",
+    /* maker     */ "falkTX",
+    /* copyright */ "GNU GPL v2+",
+    DESCFUNCS_WITHCV,
+    /* cvIns      */ 1,
+    /* cvOuts     */ 0,
+    /* bufnamefn  */ nullptr,
+    /* bufrangefn */ nullptr
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -61,7 +117,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "lfo",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -77,7 +133,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "midichanfilter",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -93,7 +149,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "midichanab",
     /* maker     */ "Milk Brewster",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -109,7 +165,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "midigain",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -125,7 +181,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "midijoin",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -141,7 +197,27 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "midisplit",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
+},
+{
+    /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
+    /* hints     */ NATIVE_PLUGIN_IS_RTSAFE,
+    /* supports  */ NATIVE_PLUGIN_SUPPORTS_ALL_SOUND_OFF,
+    /* audioIns  */ 0,
+    /* audioOuts */ 0,
+    /* midiIns   */ 1,
+    /* midiOuts  */ 0,
+    /* paramIns  */ 4,
+    /* paramOuts */ 0,
+    /* name      */ "MIDI to CV",
+    /* label     */ "midi2cv",
+    /* maker     */ "falkTX, Bram Giesen, Jarno Verheesen",
+    /* copyright */ "GNU GPL v2+",
+    DESCFUNCS_WITHCV,
+    /* cvIns      */ 0,
+    /* cvOuts     */ 3,
+    /* bufnamefn  */ nullptr,
+    /* bufrangefn */ nullptr
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -157,7 +233,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "midithrough",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -173,7 +249,23 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "miditranspose",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
+},
+{
+    /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
+    /* hints     */ NATIVE_PLUGIN_IS_RTSAFE,
+    /* supports  */ NATIVE_PLUGIN_SUPPORTS_EVERYTHING,
+    /* audioIns  */ 0,
+    /* audioOuts */ 0,
+    /* midiIns   */ 1,
+    /* midiOuts  */ 1,
+    /* paramIns  */ 1,
+    /* paramOuts */ 0,
+    /* name      */ "MIDI Channelize",
+    /* label     */ "midichannelize",
+    /* maker     */ "falkTX",
+    /* copyright */ "GNU GPL v2+",
+    DESCFUNCS_WITHOUTCV
 },
 
 // --------------------------------------------------------------------------------------------------------------------
@@ -182,8 +274,10 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
     /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_RTSAFE
+                                                  |NATIVE_PLUGIN_HAS_INLINE_DISPLAY
                                                   |NATIVE_PLUGIN_HAS_UI
                                                   |NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE
+                                                  |NATIVE_PLUGIN_REQUESTS_IDLE
                                                   |NATIVE_PLUGIN_USES_TIME),
     /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
     /* audioIns  */ 0,
@@ -196,7 +290,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "audiofile",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 
 // --------------------------------------------------------------------------------------------------------------------
@@ -207,6 +301,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_RTSAFE
                                                   |NATIVE_PLUGIN_HAS_UI
                                                   |NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE
+                                                  |NATIVE_PLUGIN_REQUESTS_IDLE
                                                   |NATIVE_PLUGIN_USES_STATE
                                                   |NATIVE_PLUGIN_USES_TIME),
     /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
@@ -220,7 +315,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "midifile",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 #ifdef HAVE_PYQT
 {
@@ -240,7 +335,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "midipattern",
     /* maker     */ "falkTX, tatch",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 #endif
 
@@ -260,13 +355,13 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* audioOuts */ 2,
     /* midiIns   */ 1,
     /* midiOuts  */ 1,
-    /* paramIns  */ 0,
-    /* paramOuts */ 0,
+    /* paramIns  */ 100,
+    /* paramOuts */ 10,
     /* name      */ "Carla-Rack",
     /* label     */ "carlarack",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_OTHER,
@@ -280,13 +375,13 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* audioOuts */ 2,
     /* midiIns   */ 1,
     /* midiOuts  */ 0,
-    /* paramIns  */ 0,
-    /* paramOuts */ 0,
+    /* paramIns  */ 100,
+    /* paramOuts */ 10,
     /* name      */ "Carla-Rack (no midi out)",
     /* label     */ "carlarack-nomidiout",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_OTHER,
@@ -300,13 +395,13 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* audioOuts */ 2,
     /* midiIns   */ 1,
     /* midiOuts  */ 1,
-    /* paramIns  */ 0,
-    /* paramOuts */ 0,
+    /* paramIns  */ 100,
+    /* paramOuts */ 10,
     /* name      */ "Carla-Patchbay",
     /* label     */ "carlapatchbay",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_OTHER,
@@ -320,13 +415,13 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* audioOuts */ 2,
     /* midiIns   */ 1,
     /* midiOuts  */ 1,
-    /* paramIns  */ 0,
-    /* paramOuts */ 0,
+    /* paramIns  */ 100,
+    /* paramOuts */ 10,
     /* name      */ "Carla-Patchbay (sidechain)",
     /* label     */ "carlapatchbay3s",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_OTHER,
@@ -340,13 +435,13 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* audioOuts */ 16,
     /* midiIns   */ 1,
     /* midiOuts  */ 1,
-    /* paramIns  */ 0,
-    /* paramOuts */ 0,
+    /* paramIns  */ 100,
+    /* paramOuts */ 10,
     /* name      */ "Carla-Patchbay (16chan)",
     /* label     */ "carlapatchbay16",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_OTHER,
@@ -360,13 +455,58 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* audioOuts */ 32,
     /* midiIns   */ 1,
     /* midiOuts  */ 1,
-    /* paramIns  */ 0,
-    /* paramOuts */ 0,
+    /* paramIns  */ 100,
+    /* paramOuts */ 10,
     /* name      */ "Carla-Patchbay (32chan)",
     /* label     */ "carlapatchbay32",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
+},
+{
+    /* category  */ NATIVE_PLUGIN_CATEGORY_OTHER,
+    /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_SYNTH
+                                                  |NATIVE_PLUGIN_HAS_UI
+                                                  |NATIVE_PLUGIN_NEEDS_UI_MAIN_THREAD
+                                                  |NATIVE_PLUGIN_USES_STATE
+                                                  |NATIVE_PLUGIN_USES_TIME),
+    /* supports  */ static_cast<NativePluginSupports>(NATIVE_PLUGIN_SUPPORTS_EVERYTHING),
+    /* audioIns  */ 64,
+    /* audioOuts */ 64,
+    /* midiIns   */ 1,
+    /* midiOuts  */ 1,
+    /* paramIns  */ 100,
+    /* paramOuts */ 10,
+    /* name      */ "Carla-Patchbay (64chan)",
+    /* label     */ "carlapatchbay64",
+    /* maker     */ "falkTX",
+    /* copyright */ "GNU GPL v2+",
+    DESCFUNCS_WITHOUTCV
+},
+{
+    /* category  */ NATIVE_PLUGIN_CATEGORY_OTHER,
+    /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_SYNTH
+                                                  |NATIVE_PLUGIN_HAS_UI
+                                                  |NATIVE_PLUGIN_NEEDS_UI_MAIN_THREAD
+                                                  |NATIVE_PLUGIN_USES_CONTROL_VOLTAGE
+                                                  |NATIVE_PLUGIN_USES_STATE
+                                                  |NATIVE_PLUGIN_USES_TIME),
+    /* supports  */ static_cast<NativePluginSupports>(NATIVE_PLUGIN_SUPPORTS_EVERYTHING),
+    /* audioIns  */ 2,
+    /* audioOuts */ 2,
+    /* midiIns   */ 1,
+    /* midiOuts  */ 1,
+    /* paramIns  */ 100,
+    /* paramOuts */ 10,
+    /* name      */ "Carla-Patchbay (CV)",
+    /* label     */ "carlapatchbaycv",
+    /* maker     */ "falkTX",
+    /* copyright */ "GNU GPL v2+",
+    DESCFUNCS_WITHCV,
+    /* cvIns      */ 5,
+    /* cvOuts     */ 5,
+    /* bufnamefn  */ nullptr,
+    /* bufrangefn */ nullptr
 },
 #endif
 
@@ -377,6 +517,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
     /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_RTSAFE
+                                                  |NATIVE_PLUGIN_HAS_INLINE_DISPLAY
                                                   |NATIVE_PLUGIN_HAS_UI
                                                   |NATIVE_PLUGIN_NEEDS_FIXED_BUFFERS),
     /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
@@ -390,7 +531,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "bigmeter",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -407,7 +548,7 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* label     */ "notes",
     /* maker     */ "falkTX",
     /* copyright */ "GNU GPL v2+",
-    DESCFUNCS
+    DESCFUNCS_WITHOUTCV
 },
 #endif
 
@@ -418,7 +559,8 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
 
 };
 
-#undef DESCFUNCS
+#undef DESCFUNCS_WITHCV
+#undef DESCFUNCS_WITHOUTCV
 
 // --------------------------------------------------------------------------------------------------------------------
 
