@@ -1,6 +1,6 @@
 /*
  * Carla Native Plugins
- * Copyright (C) 2012-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -15,23 +15,40 @@
  * For a full copy of the GNU General Public License see the doc/GPL.txt file.
  */
 
-#include "CarlaNative.hpp"
+#include "CarlaNativePrograms.hpp"
 #include "midi-base.hpp"
 
 #include "water/files/FileInputStream.h"
 #include "water/midi/MidiFile.h"
 
+#ifndef HAVE_PYQT
+# define process2 process
+#endif
+
 // -----------------------------------------------------------------------
 
+#ifdef HAVE_PYQT
+class MidiFilePlugin : public NativePluginWithMidiPrograms<FileMIDI>,
+#else
 class MidiFilePlugin : public NativePluginClass,
+#endif
                        public AbstractMidiPlayer
 {
 public:
     MidiFilePlugin(const NativeHostDescriptor* const host)
+#ifdef HAVE_PYQT
+        : NativePluginWithMidiPrograms<FileMIDI>(host, fPrograms, 0),
+#else
         : NativePluginClass(host),
+#endif
           fMidiOut(this),
           fNeedsAllNotesOff(false),
-          fWasPlayingBefore(false) {}
+          fWasPlayingBefore(false)
+#ifdef HAVE_PYQT
+        , fPrograms(hostGetFilePath("midi"), "*.mid;*.midi")
+#endif
+    {
+    }
 
 protected:
     // -------------------------------------------------------------------
@@ -45,16 +62,22 @@ protected:
         if (std::strcmp(key, "file") != 0)
             return;
 
+#ifdef HAVE_PYQT
+        invalidateNextFilename();
+#endif
         _loadMidiFile(value);
     }
 
     // -------------------------------------------------------------------
     // Plugin process calls
 
-    void process(float**, float**, const uint32_t frames, const NativeMidiEvent* const, const uint32_t) override
+    void process2(const float* const*, float**, const uint32_t frames, const NativeMidiEvent* const, const uint32_t) override
     {
         const NativeTimeInfo* const timePos(getTimeInfo());
 
+        if (timePos == nullptr)
+            return;
+
         if (fWasPlayingBefore != timePos->playing)
         {
             fNeedsAllNotesOff = true;
@@ -113,6 +136,13 @@ protected:
         fMidiOut.setState(data);
     }
 
+#ifdef HAVE_PYQT
+    void setStateFromFile(const char* const filename) override
+    {
+        _loadMidiFile(filename);
+    }
+#endif
+
     // -------------------------------------------------------------------
     // AbstractMidiPlayer calls
 
@@ -137,6 +167,9 @@ private:
     MidiPattern fMidiOut;
     bool fNeedsAllNotesOff;
     bool fWasPlayingBefore;
+#ifdef HAVE_PYQT
+    NativeMidiPrograms fPrograms;
+#endif
 
     void _loadMidiFile(const char* const filename)
     {
@@ -160,7 +193,7 @@ private:
 
         const double sampleRate(getSampleRate());
 
-        for (int i=0, numTracks = midiFile.getNumTracks(); i<numTracks; ++i)
+        for (size_t i=0, numTracks = midiFile.getNumTracks(); i<numTracks; ++i)
         {
             const MidiMessageSequence* const track(midiFile.getTrack(i));
             CARLA_SAFE_ASSERT_CONTINUE(track != nullptr);
@@ -220,6 +253,9 @@ static const NativePluginDescriptor midifileDesc = {
     /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_RTSAFE
                                                   |NATIVE_PLUGIN_HAS_UI
                                                   |NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE
+#ifdef HAVE_PYQT
+                                                  |NATIVE_PLUGIN_REQUESTS_IDLE
+#endif
                                                   |NATIVE_PLUGIN_USES_STATE
                                                   |NATIVE_PLUGIN_USES_TIME),
     /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
@@ -248,3 +284,7 @@ void carla_register_native_plugin_midifile()
 }
 
 // -----------------------------------------------------------------------
+
+#ifndef HAVE_PYQT
+# undef process2
+#endif
