@@ -1,6 +1,6 @@
 /*
  * Carla Native Plugins
- * Copyright (C) 2013-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2013-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,22 @@
 #include "CarlaPipeUtils.hpp"
 #include "CarlaString.hpp"
 
+#ifdef USING_JUCE
+# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wconversion"
+#  pragma GCC diagnostic ignored "-Weffc++"
+#  pragma GCC diagnostic ignored "-Wsign-conversion"
+#  pragma GCC diagnostic ignored "-Wundef"
+#  pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+# endif
+# include "AppConfig.h"
+# include "juce_events/juce_events.h"
+# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic pop
+# endif
+#endif
+
 #include "water/files/File.h"
 
 template<>
@@ -51,10 +67,18 @@ public:
 #ifdef CARLA_PROPER_CPP11_SUPPORT
           fProgramDesc({0, 0, nullptr}),
 #endif
+          kIgnoreParameters(std::strncmp(desc->label, "carla", 5) == 0),
           fMidiEventCount(0),
+#ifdef USING_JUCE
+          fJuceInitialiser(),
+#endif
+          fLoadedFile(),
           fWorkerUISignal(0)
     {
         carla_zeroStruct(fHost);
+#ifndef CARLA_PROPER_CPP11_SUPPORT
+        carla_zeroStruct(fProgramDesc);
+#endif
 
         if (! loadedInProperHost())
             return;
@@ -121,13 +145,16 @@ public:
         fPorts.usesTime     = fDescriptor->hints & NATIVE_PLUGIN_USES_TIME;
         fPorts.numAudioIns  = fDescriptor->audioIns;
         fPorts.numAudioOuts = fDescriptor->audioOuts;
+        fPorts.numCVIns     = fDescriptor->cvIns;
+        fPorts.numCVOuts    = fDescriptor->cvOuts;
         fPorts.numMidiIns   = fDescriptor->midiIns;
         fPorts.numMidiOuts  = fDescriptor->midiOuts;
 
         if (fDescriptor->get_parameter_count != nullptr &&
             fDescriptor->get_parameter_info  != nullptr &&
             fDescriptor->get_parameter_value != nullptr &&
-            fDescriptor->set_parameter_value != nullptr)
+            fDescriptor->set_parameter_value != nullptr &&
+            ! kIgnoreParameters)
         {
             fPorts.numParams = fDescriptor->get_parameter_count(fHandle);
         }
@@ -225,13 +252,15 @@ public:
 
                     if (event->body.type == fURIs.uiEvents && fWorkerUISignal != -1)
                     {
+                        CARLA_SAFE_ASSERT_CONTINUE((fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE) == 0);
+
                         if (fWorker != nullptr)
                         {
                             // worker is supported by the host, we can continue
                             fWorkerUISignal = 1;
                             const char* const msg((const char*)(event + 1));
                             const size_t msgSize = std::strlen(msg);
-                            fWorker->schedule_work(fWorker->handle, msgSize+1, msg);
+                            fWorker->schedule_work(fWorker->handle, static_cast<uint32_t>(msgSize + 1U), msg);
                         }
                         else
                         {
@@ -241,6 +270,43 @@ public:
                         continue;
                     }
 
+                    if (event->body.type == fURIs.atomObject)
+                    {
+                        const LV2_Atom_Object* const obj = (const LV2_Atom_Object*)(&event->body);
+
+                        if (obj->body.otype == fURIs.patchSet) {
+                            // Get property URI.
+                            const LV2_Atom* property = NULL;
+                            lv2_atom_object_get(obj, fURIs.patchProperty, &property, 0);
+                            CARLA_SAFE_ASSERT_CONTINUE(property != nullptr);
+                            CARLA_SAFE_ASSERT_CONTINUE(property->type == fURIs.atomURID);
+
+                            const LV2_URID urid = ((const LV2_Atom_URID*)property)->body;
+
+                            /*  */ if (std::strcmp(fDescriptor->label, "audiofile") == 0) {
+                                CARLA_SAFE_ASSERT_CONTINUE(urid == fURIs.carlaFileAudio);
+                            } else if (std::strcmp(fDescriptor->label, "midifile") == 0) {
+                                CARLA_SAFE_ASSERT_CONTINUE(urid == fURIs.carlaFileMIDI);
+                            } else {
+                                CARLA_SAFE_ASSERT_CONTINUE(urid == fURIs.carlaFile);
+                            }
+
+                            // Get value.
+                            const LV2_Atom* fileobj = NULL;
+                            lv2_atom_object_get(obj, fURIs.patchValue, &fileobj, 0);
+                            CARLA_SAFE_ASSERT_CONTINUE(fileobj != nullptr);
+                            CARLA_SAFE_ASSERT_CONTINUE(fileobj->type == fURIs.atomPath);
+
+                            const char* const filepath((const char*)(fileobj + 1));
+
+                            fWorker->schedule_work(fWorker->handle,
+                                                   static_cast<uint32_t>(std::strlen(filepath) + 1U),
+                                                   filepath);
+                        }
+
+                        continue;
+                    }
+
                     if (event->body.type != fURIs.midiEvent)
                         continue;
                     if (event->body.size > 4)
@@ -268,10 +334,7 @@ public:
             }
         }
 
-        // FIXME
-        fDescriptor->process(fHandle,
-                             const_cast<float**>(fPorts.audioIns), fPorts.audioOuts, frames,
-                             fMidiEvents, fMidiEventCount);
+        fDescriptor->process(fHandle, fPorts.audioCVIns, fPorts.audioCVOuts, frames, fMidiEvents, fMidiEventCount);
 
         if (fWorkerUISignal == -1 && fPorts.hasUI)
         {
@@ -350,6 +413,17 @@ public:
     LV2_State_Status lv2_save(const LV2_State_Store_Function store, const LV2_State_Handle handle,
                               const uint32_t /*flags*/, const LV2_Feature* const* const /*features*/) const
     {
+        if (fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE)
+        {
+            store(handle,
+                  fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/file"),
+                  fLoadedFile.buffer(),
+                  fLoadedFile.length()+1,
+                  fURIs.atomPath,
+                  LV2_STATE_IS_POD);
+            return LV2_STATE_SUCCESS;
+        }
+
         if ((fDescriptor->hints & NATIVE_PLUGIN_USES_STATE) == 0 || fDescriptor->get_state == nullptr)
             return LV2_STATE_ERR_NO_FEATURE;
 
@@ -364,13 +438,31 @@ public:
     }
 
     LV2_State_Status lv2_restore(const LV2_State_Retrieve_Function retrieve, const LV2_State_Handle handle,
-                                 uint32_t flags, const LV2_Feature* const* const /*features*/) const
+                                 uint32_t flags, const LV2_Feature* const* const /*features*/)
     {
+        size_t   size = 0;
+        uint32_t type = 0;
+
+        if (fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE)
+        {
+            size = type = 0;
+            const void* const data = retrieve(handle,
+                                              fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/file"),
+                                              &size, &type, &flags);
+
+            CARLA_SAFE_ASSERT_RETURN(type == fURIs.atomPath, LV2_STATE_ERR_UNKNOWN);
+
+            const char* const filename = (const char*)data;
+
+            fLoadedFile = filename;
+            fDescriptor->set_custom_data(fHandle, "file", filename);
+            return LV2_STATE_SUCCESS;
+        }
+
         if ((fDescriptor->hints & NATIVE_PLUGIN_USES_STATE) == 0 || fDescriptor->set_state == nullptr)
             return LV2_STATE_ERR_NO_FEATURE;
 
-        size_t   size = 0;
-        uint32_t type = 0;
+        size = type = 0;
         const void* const data = retrieve(handle, fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/chunk"), &size, &type, &flags);
 
         if (size == 0)
@@ -393,6 +485,13 @@ public:
     {
         const char* const msg = (const char*)data;
 
+        if (fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE)
+        {
+            fLoadedFile = msg;
+            fDescriptor->set_custom_data(fHandle, "file", msg);
+            return LV2_WORKER_SUCCESS;
+        }
+
         /**/ if (std::strncmp(msg, "control ", 8) == 0)
         {
             if (fDescriptor->ui_set_parameter_value == nullptr)
@@ -406,19 +505,19 @@ public:
 
                 char strBufIndex[8];
                 carla_zeroChars(strBufIndex, 8);
-                std::strncpy(strBufIndex, msgIndex, msgSplit-msgIndex);
+                std::strncpy(strBufIndex, msgIndex, static_cast<size_t>(msgSplit - msgIndex));
 
-                const int index = std::atoi(msgIndex) - fPorts.indexOffset;
+                const int index = std::atoi(msgIndex) - static_cast<int>(fPorts.indexOffset);
                 CARLA_SAFE_ASSERT_RETURN(index >= 0, LV2_WORKER_ERR_UNKNOWN);
 
-                double value;
+                float value;
 
                 {
-                    const ScopedLocale csl;
-                    value = std::atof(msgSplit+1);
+                    const CarlaScopedLocale csl;
+                    value = static_cast<float>(std::atof(msgSplit+1));
                 }
 
-                fDescriptor->ui_set_parameter_value(fHandle, index, value);
+                fDescriptor->ui_set_parameter_value(fHandle, static_cast<uint32_t>(index), value);
             }
         }
         else if (std::strcmp(msg, "show") == 0)
@@ -454,11 +553,10 @@ public:
     // ----------------------------------------------------------------------------------------------------------------
 
     void lv2ui_instantiate(LV2UI_Write_Function writeFunction, LV2UI_Controller controller,
-                           LV2UI_Widget* widget, const LV2_Feature* const* features, const bool isEmbed)
+                           LV2UI_Widget* widget, const LV2_Feature* const* features)
     {
         fUI.writeFunction = writeFunction;
         fUI.controller = controller;
-        fUI.isEmbed = isEmbed;
 
         if (fHost.uiName != nullptr)
         {
@@ -466,60 +564,6 @@ public:
             fHost.uiName = nullptr;
         }
 
-#if defined(CARLA_OS_LINUX) && defined(HAVE_X11)
-        // ---------------------------------------------------------------
-        // show embed UI if needed
-
-        if (isEmbed)
-        {
-            intptr_t parentId = 0;
-            const LV2UI_Resize* uiResize = nullptr;
-
-            for (int i=0; features[i] != nullptr; ++i)
-            {
-                if (std::strcmp(features[i]->URI, LV2_UI__parent) == 0)
-                {
-                    parentId = (intptr_t)features[i]->data;
-                }
-                else if (std::strcmp(features[i]->URI, LV2_UI__resize) == 0)
-                {
-                    uiResize = (const LV2UI_Resize*)features[i]->data;
-                }
-            }
-
-            // -----------------------------------------------------------
-            // see if the host can really embed the UI
-
-            if (parentId != 0)
-            {
-                // wait for remote side to be ready
-                fDescriptor->dispatcher(fHandle, NATIVE_PLUGIN_OPCODE_NULL, (int32_t)0xDEADF00D, 0xC0C0B00B, nullptr, 0.0f);
-
-                if (uiResize && uiResize->ui_resize != nullptr)
-                    uiResize->ui_resize(uiResize->handle, 740, 512);
-
-                fHost.uiName = carla_strdup(fDescriptor->name);
-                fUI.isVisible = true;
-
-                char strBuf[0xff+1];
-                strBuf[0xff] = '\0';
-                std::snprintf(strBuf, 0xff, P_INTPTR, parentId);
-
-                carla_setenv("CARLA_PLUGIN_EMBED_WINID", strBuf);
-
-                fDescriptor->ui_show(fHandle, true);
-
-                carla_setenv("CARLA_PLUGIN_EMBED_WINID", "0");
-
-                const intptr_t winId(fDescriptor->dispatcher(fHandle, NATIVE_PLUGIN_OPCODE_NULL, (int32_t)0xDEADF00D, 0xC0C0B00B, nullptr, 0.0f));
-                CARLA_SAFE_ASSERT_RETURN(winId != 0,);
-
-                *widget = (LV2UI_Widget)winId;
-                return;
-            }
-        }
-#endif
-
         // ---------------------------------------------------------------
         // see if the host supports external-ui
 
@@ -531,6 +575,11 @@ public:
                 fUI.host = (const LV2_External_UI_Host*)features[i]->data;
                 break;
             }
+            if (std::strcmp(features[i]->URI, LV2_UI__touch) == 0)
+            {
+                fUI.touch = (const LV2UI_Touch*)features[i]->data;
+                break;
+            }
         }
 
         if (fUI.host != nullptr)
@@ -570,9 +619,6 @@ public:
 
         *widget = nullptr;
         return;
-
-        // maybe be unused
-        (void)isEmbed;
     }
 
     void lv2ui_port_event(uint32_t portIndex, uint32_t bufferSize, uint32_t format, const void* buffer) const
@@ -683,13 +729,22 @@ protected:
 
     void handleUiParameterChanged(const uint32_t index, const float value) const
     {
-        if (fWorkerUISignal)
+        if (kIgnoreParameters || fWorkerUISignal)
             return;
 
         if (fUI.writeFunction != nullptr && fUI.controller != nullptr)
             fUI.writeFunction(fUI.controller, index+fPorts.indexOffset, sizeof(float), 0, &value);
     }
 
+    void handleUiParameterTouch(const uint32_t index, const bool touch) const
+    {
+        if (kIgnoreParameters)
+            return;
+
+        if (fUI.touch != nullptr && fUI.touch->touch != nullptr)
+            fUI.touch->touch(fUI.touch->handle, index+fPorts.indexOffset, touch);
+    }
+
     void handleUiCustomDataChanged(const char* const key, const char* const value) const
     {
         carla_stdout("TODO: handleUiCustomDataChanged %s %s", key, value);
@@ -710,6 +765,7 @@ protected:
             fUI.host->ui_closed(fUI.controller);
 
         fUI.host = nullptr;
+        fUI.touch = nullptr;
         fUI.writeFunction = nullptr;
         fUI.controller = nullptr;
     }
@@ -728,7 +784,8 @@ protected:
 
     intptr_t handleDispatcher(const NativeHostDispatcherOpcode opcode, const int32_t index, const intptr_t value, void* const ptr, const float opt)
     {
-        carla_debug("NativePlugin::handleDispatcher(%i, %i, " P_INTPTR ", %p, %f)", opcode, index, value, ptr, opt);
+        carla_debug("NativePlugin::handleDispatcher(%i, %i, " P_INTPTR ", %p, %f)",
+                    opcode, index, value, ptr, static_cast<double>(opt));
 
         intptr_t ret = 0;
 
@@ -742,11 +799,20 @@ protected:
         case NATIVE_HOST_OPCODE_RELOAD_ALL:
         case NATIVE_HOST_OPCODE_HOST_IDLE:
         case NATIVE_HOST_OPCODE_INTERNAL_PLUGIN:
+        case NATIVE_HOST_OPCODE_QUEUE_INLINE_DISPLAY:
+        case NATIVE_HOST_OPCODE_REQUEST_IDLE:
+        case NATIVE_HOST_OPCODE_GET_FILE_PATH:
             // nothing
             break;
+
         case NATIVE_HOST_OPCODE_UI_UNAVAILABLE:
             handleUiClosed();
             break;
+
+        case NATIVE_HOST_OPCODE_UI_TOUCH_PARAMETER:
+            CARLA_SAFE_ASSERT_RETURN(index >= 0, 0);
+            handleUiParameterTouch(static_cast<uint32_t>(index), value != 0);
+            break;
         }
 
         return ret;
@@ -783,9 +849,17 @@ private:
     const NativePluginDescriptor* const fDescriptor;
     LV2_Program_Descriptor              fProgramDesc;
 
+    // carla as plugin does not implement lv2 parameter API yet, needed for feedback
+    const bool kIgnoreParameters;
+
     uint32_t        fMidiEventCount;
     NativeMidiEvent fMidiEvents[kMaxMidiEvents];
 
+#ifdef USING_JUCE
+    juce::SharedResourcePointer<juce::ScopedJuceInitialiser_GUI> fJuceInitialiser;
+#endif
+
+    CarlaString fLoadedFile;
     int fWorkerUISignal;
 
     // -------------------------------------------------------------------
@@ -822,6 +896,11 @@ private:
         handlePtr->handleUiParameterChanged(index, value);
     }
 
+    static void host_ui_parameter_touch(NativeHostHandle handle, uint32_t index, bool touch)
+    {
+        handlePtr->handleUiParameterTouch(index, touch);
+    }
+
     static void host_ui_custom_data_changed(NativeHostHandle handle, const char* key, const char* value)
     {
         handlePtr->handleUiCustomDataChanged(key, value);
@@ -978,7 +1057,7 @@ static LV2_Worker_Status lv2_work(LV2_Handle instance, LV2_Worker_Respond_Functi
     return instancePtr->lv2_work(respond, handle, size, data);
 }
 
-LV2_Worker_Status lv2_work_resp(LV2_Handle instance, uint32_t size, const void* body)
+static LV2_Worker_Status lv2_work_resp(LV2_Handle instance, uint32_t size, const void* body)
 {
     carla_debug("work_resp(%p, %u, %p)", instance, size, body);
     return instancePtr->lv2_work_resp(size, body);
@@ -1011,13 +1090,11 @@ static const void* lv2_extension_data(const char* uri)
 // -----------------------------------------------------------------------
 // LV2 UI descriptor functions
 
-static LV2UI_Handle lv2ui_instantiate(LV2UI_Write_Function writeFunction, LV2UI_Controller controller,
-                                      LV2UI_Widget* widget, const LV2_Feature* const* features, const bool isEmbed)
+static LV2UI_Handle lv2ui_instantiate(const LV2UI_Descriptor*, const char*, const char*,
+                                            LV2UI_Write_Function writeFunction, LV2UI_Controller controller,
+                                            LV2UI_Widget* widget, const LV2_Feature* const* features)
 {
     carla_debug("lv2ui_instantiate(..., %p, %p, %p)", writeFunction, controller, widget, features);
-# if ! (defined(CARLA_OS_LINUX) && defined(HAVE_X11))
-    CARLA_SAFE_ASSERT_RETURN(! isEmbed, nullptr);
-# endif
 
     NativePlugin* plugin = nullptr;
 
@@ -1036,27 +1113,11 @@ static LV2UI_Handle lv2ui_instantiate(LV2UI_Write_Function writeFunction, LV2UI_
         return nullptr;
     }
 
-    plugin->lv2ui_instantiate(writeFunction, controller, widget, features, isEmbed);
+    plugin->lv2ui_instantiate(writeFunction, controller, widget, features);
 
     return (LV2UI_Handle)plugin;
 }
 
-# if defined(CARLA_OS_LINUX) && defined(HAVE_X11)
-static LV2UI_Handle lv2ui_instantiate_embed(const LV2UI_Descriptor*, const char*, const char*,
-                                            LV2UI_Write_Function writeFunction, LV2UI_Controller controller,
-                                            LV2UI_Widget* widget, const LV2_Feature* const* features)
-{
-    return lv2ui_instantiate(writeFunction, controller, widget, features, true);
-}
-# endif
-
-static LV2UI_Handle lv2ui_instantiate_external(const LV2UI_Descriptor*, const char*, const char*,
-                                               LV2UI_Write_Function writeFunction, LV2UI_Controller controller,
-                                               LV2UI_Widget* widget, const LV2_Feature* const* features)
-{
-    return lv2ui_instantiate(writeFunction, controller, widget, features, false);
-}
-
 #define uiPtr ((NativePlugin*)ui)
 
 static void lv2ui_port_event(LV2UI_Handle ui, uint32_t portIndex, uint32_t bufferSize, uint32_t format, const void* buffer)
@@ -1174,24 +1235,9 @@ const LV2UI_Descriptor* lv2ui_descriptor(uint32_t index)
 {
     carla_debug("lv2ui_descriptor(%i)", index);
 
-#if defined(CARLA_OS_LINUX) && defined(HAVE_X11)
-    static const LV2UI_Descriptor lv2UiEmbedDesc = {
-    /* URI            */ "http://kxstudio.sf.net/carla/ui-embed",
-    /* instantiate    */ lv2ui_instantiate_embed,
-    /* cleanup        */ lv2ui_cleanup,
-    /* port_event     */ lv2ui_port_event,
-    /* extension_data */ lv2ui_extension_data
-    };
-
-    if (index == 0)
-        return &lv2UiEmbedDesc;
-    else
-        --index;
-#endif
-
     static const LV2UI_Descriptor lv2UiExtDesc = {
     /* URI            */ "http://kxstudio.sf.net/carla/ui-ext",
-    /* instantiate    */ lv2ui_instantiate_external,
+    /* instantiate    */ lv2ui_instantiate,
     /* cleanup        */ lv2ui_cleanup,
     /* port_event     */ lv2ui_port_event,
     /* extension_data */ lv2ui_extension_data
