@@ -1,6 +1,6 @@
 /*
  * Carla Native Plugins
- * Copyright (C) 2013-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2013-2020 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -15,35 +15,67 @@
  * For a full copy of the GNU General Public License see the doc/GPL.txt file.
  */
 
+#ifdef __WINE__
+  #error This file is not supposed to be built with wine!
+#endif
+#ifndef CARLA_PLUGIN_SYNTH
+  #error CARLA_PLUGIN_SYNTH undefined
+#endif
+
 #ifndef CARLA_VST_SHELL
   #ifndef CARLA_PLUGIN_PATCHBAY
     #error CARLA_PLUGIN_PATCHBAY undefined
   #endif
-  #ifndef CARLA_PLUGIN_SYNTH
-    #error CARLA_PLUGIN_SYNTH undefined
-  #endif
-  #if CARLA_PLUGIN_32CH || CARLA_PLUGIN_16CH
+  #if defined(CARLA_PLUGIN_64CH) || defined(CARLA_PLUGIN_32CH) || defined(CARLA_PLUGIN_16CH)
     #if ! CARLA_PLUGIN_SYNTH
-      #error CARLA_PLUGIN_16/32CH requires CARLA_PLUGIN_SYNTH
+      #error CARLA_PLUGIN_16/32/64CH requires CARLA_PLUGIN_SYNTH
     #endif
   #endif
 #endif
 
 #define CARLA_NATIVE_PLUGIN_VST
 #include "carla-base.cpp"
+#include "carla-vst.hpp"
 
 #include "water/files/File.h"
 
 #include "CarlaMathUtils.hpp"
 #include "CarlaVstUtils.hpp"
 
+#ifdef USING_JUCE
+# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wconversion"
+#  pragma GCC diagnostic ignored "-Weffc++"
+#  pragma GCC diagnostic ignored "-Wsign-conversion"
+#  pragma GCC diagnostic ignored "-Wundef"
+#  pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+# endif
+# include "AppConfig.h"
+# include "juce_events/juce_events.h"
+# if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic pop
+# endif
+#endif
+
 static uint32_t d_lastBufferSize = 0;
 static double   d_lastSampleRate = 0.0;
 
 static const int32_t kBaseUniqueID = CCONST('C', 'r', 'l', 'a');
-static const int32_t kShellUniqueID = CCONST('C', 'r', 'l', 's');
 static const int32_t kVstMidiEventSize = static_cast<int32_t>(sizeof(VstMidiEvent));
 
+#ifdef CARLA_VST_SHELL
+# if CARLA_PLUGIN_SYNTH
+static const int32_t kShellUniqueID = CCONST('C', 'r', 'l', 's');
+# else
+static const int32_t kShellUniqueID = CCONST('C', 'r', 'l', 'F');
+# endif
+#else
+static const int32_t kNumParameters = 100;
+#endif
+
+static const bool kIsUsingUILauncher = isUsingUILauncher();
+
 // --------------------------------------------------------------------------------------------------------------------
 // Carla Internal Plugin API exposed as VST plugin
 
@@ -52,9 +84,8 @@ class NativePlugin
 public:
     static const uint32_t kMaxMidiEvents = 512;
 
-    NativePlugin(const audioMasterCallback audioMaster, AEffect* const effect, const NativePluginDescriptor* desc)
-        : fAudioMaster(audioMaster),
-          fEffect(effect),
+    NativePlugin(AEffect* const effect, const NativePluginDescriptor* desc)
+        : fEffect(effect),
           fHandle(nullptr),
           fHost(),
           fDescriptor(desc),
@@ -64,22 +95,35 @@ public:
           fMidiEventCount(0),
           fTimeInfo(),
           fVstRect(),
+          fUiLauncher(nullptr),
           fHostType(kHostTypeNull),
           fMidiOutEvents(),
+#ifdef USING_JUCE
+          fJuceInitialiser(),
+#endif
           fStateChunk(nullptr)
     {
         fHost.handle      = this;
         fHost.uiName      = carla_strdup("CarlaVST");
         fHost.uiParentId  = 0;
 
+        std::memset(fProgramName, 0, sizeof(fProgramName));
+        std::strcpy(fProgramName, "Default");
+
         // find resource dir
         using water::File;
         using water::String;
 
         File curExe = File::getSpecialLocation(File::currentExecutableFile).getLinkedTarget();
         File resDir = curExe.getSiblingFile("resources");
+
+#ifndef CARLA_OS_MAC
+        // FIXME: proper fallback path for other OSes
         if (! resDir.exists())
-            resDir = File("/usr/share/carla/resources/");
+            resDir = File("/usr/local/share/carla/resources");
+        if (! resDir.exists())
+            resDir = File("/usr/share/carla/resources");
+#endif
 
         // find host type
         const String hostFilename(File::getSpecialLocation(File::hostApplicationPath).getFileName());
@@ -103,10 +147,19 @@ public:
         fHost.ui_save_file           = host_ui_save_file;
         fHost.dispatcher             = host_dispatcher;
 
-        fVstRect.top    = 0;
-        fVstRect.left   = 0;
-        fVstRect.bottom = 712;
-        fVstRect.right  = 1024;
+        fVstRect.top  = 0;
+        fVstRect.left = 0;
+
+        if (kIsUsingUILauncher)
+        {
+            fVstRect.bottom = ui_launcher_res::carla_uiHeight;
+            fVstRect.right  = ui_launcher_res::carla_uiWidth;
+        }
+        else
+        {
+            fVstRect.bottom = 712;
+            fVstRect.right  = 1024;
+        }
 
         init();
     }
@@ -170,7 +223,8 @@ public:
 
     // -------------------------------------------------------------------
 
-    intptr_t vst_dispatcher(const int32_t opcode, const int32_t /*index*/, const intptr_t value, void* const ptr, const float opt)
+    intptr_t vst_dispatcher(const int32_t opcode,
+                            const int32_t index, const intptr_t value, void* const ptr, const float opt)
     {
         CARLA_SAFE_ASSERT_RETURN(fHandle != nullptr, 0);
 
@@ -178,6 +232,116 @@ public:
 
         switch (opcode)
         {
+        case effGetProgram:
+            return 0;
+
+        case effSetProgramName:
+            if (char* const programName = (char*)ptr)
+            {
+                std::strncpy(fProgramName, programName, 32);
+                return 1;
+            }
+            break;
+
+        case effGetProgramName:
+            if (char* const programName = (char*)ptr)
+            {
+                std::strncpy(programName, fProgramName, 23);
+                programName[23] = '\0';
+                return 1;
+            }
+            break;
+
+        case effGetProgramNameIndexed:
+            if (char* const programName = (char*)ptr)
+            {
+                std::strncpy(programName, fProgramName, 23);
+                programName[23] = '\0';
+                return 1;
+            }
+            break;
+
+        case effGetParamDisplay:
+            CARLA_SAFE_ASSERT_RETURN(index >= 0, 0);
+#ifndef CARLA_VST_SHELL
+            CARLA_SAFE_ASSERT_RETURN(index < kNumParameters, 0);
+#endif
+
+            if (char* const cptr = (char*)ptr)
+            {
+                const uint32_t uindex = static_cast<uint32_t>(index);
+                CARLA_SAFE_ASSERT_RETURN(uindex < fDescriptor->paramIns, 0);
+
+                const NativeParameter* const param = fDescriptor->get_parameter_info(fHandle, uindex);
+                CARLA_SAFE_ASSERT_RETURN(param != nullptr, 0);
+
+                float paramValue = fDescriptor->get_parameter_value(fHandle, uindex);
+
+                if (param->hints & NATIVE_PARAMETER_IS_BOOLEAN)
+                {
+                    const NativeParameterRanges& ranges(param->ranges);
+                    const float midRange = ranges.min + (ranges.max - ranges.min) / 2.0f;
+
+                    paramValue = paramValue > midRange ? ranges.max : ranges.min;
+                }
+                else if (param->hints & NATIVE_PARAMETER_IS_INTEGER)
+                {
+                    paramValue = std::round(paramValue);
+                }
+
+                for (uint32_t i = 0; i < param->scalePointCount; ++i)
+                {
+                    const NativeParameterScalePoint& scalePoint(param->scalePoints[uindex]);
+
+                    if (carla_isNotEqual(paramValue, scalePoint.value))
+                        continue;
+
+                    std::strncpy(cptr, scalePoint.label, 23);
+                    cptr[23] = '\0';
+                    return 1;
+                }
+
+                if (param->hints & NATIVE_PARAMETER_IS_INTEGER)
+                {
+                    std::snprintf(cptr, 23, "%d%s%s",
+                                  static_cast<int>(paramValue),
+                                  param->unit != nullptr && param->unit[0] != '\0' ? " " : "",
+                                  param->unit != nullptr && param->unit[0] != '\0' ? param->unit : "");
+                    cptr[23] = '\0';
+                }
+                else
+                {
+                    std::snprintf(cptr, 23, "%.12g%s%s",
+                                  static_cast<double>(paramValue),
+                                  param->unit != nullptr && param->unit[0] != '\0' ? " " : "",
+                                  param->unit != nullptr && param->unit[0] != '\0' ? param->unit : "");
+                    cptr[23] = '\0';
+                }
+
+                return 1;
+            }
+            break;
+
+        case effGetParamName:
+            CARLA_SAFE_ASSERT_RETURN(index >= 0, 0);
+#ifndef CARLA_VST_SHELL
+            CARLA_SAFE_ASSERT_RETURN(index < kNumParameters, 0);
+#endif
+
+            if (char* const cptr = (char*)ptr)
+            {
+                const uint32_t uindex = static_cast<uint32_t>(index);
+                CARLA_SAFE_ASSERT_RETURN(uindex < fDescriptor->paramIns, 0);
+
+                const NativeParameter* const param = fDescriptor->get_parameter_info(fHandle, uindex);
+                CARLA_SAFE_ASSERT_RETURN(param != nullptr, 0);
+
+                std::strncpy(cptr, param->name, 15);
+                cptr[15] = '\0';
+                return 1;
+            }
+            return 0;
+
         case effSetSampleRate:
             CARLA_SAFE_ASSERT_RETURN(opt > 0.0f, 0);
 
@@ -260,46 +424,26 @@ public:
         case effEditOpen:
             if (fDescriptor->ui_show != nullptr)
             {
-                char strBuf[0xff+1];
-                strBuf[0xff] = '\0';
-                std::snprintf(strBuf, 0xff, P_INTPTR, (intptr_t)ptr);
-
-                // set CARLA_PLUGIN_EMBED_WINID for external process
-                carla_setenv("CARLA_PLUGIN_EMBED_WINID", strBuf);
-
-                // check if vst host is an old version of tracktion
-                bool oldTracktionWorkaround = false;
-
-                // tracktion doesn't export its version properly, use env var instead
-                if (std::getenv("CARLA_TRACKTION_WORKAROUND") != nullptr)
+                if (kIsUsingUILauncher)
                 {
-                    carla_zeroChars(strBuf, 0xff+1);
-                    hostCallback(audioMasterGetProductString, 0, 0, strBuf, 0.0f);
-                    oldTracktionWorkaround = (std::strcmp(strBuf, "Tracktion") == 0);
-
-                    // if vst host is old tracktion, delay UI appearance for a bit (part 1)
-                    if (oldTracktionWorkaround)
-                        fDescriptor->dispatcher(fHandle, NATIVE_PLUGIN_OPCODE_NULL, (int32_t)0xDEADF00D, 0xC0C0B00B, nullptr, 0.0f);
+                    destoryUILauncher(fUiLauncher);
+                    fUiLauncher = createUILauncher((intptr_t)ptr, fDescriptor, fHandle);
                 }
-
-                // show UI now
-                fDescriptor->ui_show(fHandle, true);
-
-                // reset CARLA_PLUGIN_EMBED_WINID just in case
-                carla_setenv("CARLA_PLUGIN_EMBED_WINID", "0");
-
-                // if vst host is old tracktion, delay UI appearance for a bit (part 2)
-                if (oldTracktionWorkaround)
+                else
                 {
-                    carla_stdout("Old Tracktion detected, delaying UI appearance so it works properly...");
+                    char strBuf[0xff+1];
+                    std::snprintf(strBuf, 0xff, P_INTPTR, (intptr_t)ptr);
+                    strBuf[0xff] = '\0';
 
-                    for (int x=10; --x>=0;)
-                    {
-                        hostCallback(audioMasterIdle);
-                        carla_msleep(25);
-                    }
+                    // set CARLA_PLUGIN_EMBED_WINID for external process
+                    carla_setenv("CARLA_PLUGIN_EMBED_WINID", strBuf);
+
+                    // show UI now
+                    fDescriptor->ui_show(fHandle, true);
+
+                    // reset CARLA_PLUGIN_EMBED_WINID just in case
+                    carla_setenv("CARLA_PLUGIN_EMBED_WINID", "0");
                 }
-
                 ret = 1;
             }
             break;
@@ -307,12 +451,22 @@ public:
         case effEditClose:
             if (fDescriptor->ui_show != nullptr)
             {
-                fDescriptor->ui_show(fHandle, false);
+                if (kIsUsingUILauncher)
+                {
+                    destoryUILauncher(fUiLauncher);
+                    fUiLauncher = nullptr;
+                }
+                else
+                {
+                    fDescriptor->ui_show(fHandle, false);
+                }
                 ret = 1;
             }
             break;
 
         case effEditIdle:
+            if (fUiLauncher != nullptr)
+                idleUILauncher(fUiLauncher);
             if (fDescriptor->ui_idle != nullptr)
                 fDescriptor->ui_idle(fHandle);
             break;
@@ -381,6 +535,10 @@ public:
             }
             break;
 
+        case effCanBeAutomated:
+            ret = 1;
+            break;
+
         case effCanDo:
             if (const char* const canDo = (const char*)ptr)
             {
@@ -405,16 +563,50 @@ public:
         return ret;
     }
 
-    float vst_getParameter(const int32_t /*index*/)
+    float vst_getParameter(const int32_t index)
     {
-        return 0.0f;
+        CARLA_SAFE_ASSERT_RETURN(index >= 0, 0.0f);
+
+        const uint32_t uindex = static_cast<uint32_t>(index);
+        CARLA_SAFE_ASSERT_RETURN(uindex < fDescriptor->paramIns, 0.0f);
+
+        const NativeParameter* const param = fDescriptor->get_parameter_info(fHandle, uindex);
+        CARLA_SAFE_ASSERT_RETURN(param != nullptr, 0);
+
+        const float realValue = fDescriptor->get_parameter_value(fHandle, uindex);
+
+        return (realValue - param->ranges.min) / (param->ranges.max - param->ranges.min);
     }
 
-    void vst_setParameter(const int32_t /*index*/, const float /*value*/)
+    void vst_setParameter(const int32_t index, const float value)
     {
+        CARLA_SAFE_ASSERT_RETURN(index >= 0,);
+
+        const uint32_t uindex = static_cast<uint32_t>(index);
+        CARLA_SAFE_ASSERT_RETURN(uindex < fDescriptor->paramIns,);
+
+        const NativeParameter* const param = fDescriptor->get_parameter_info(fHandle, uindex);
+        CARLA_SAFE_ASSERT_RETURN(param != nullptr,);
+
+        float realValue;
+
+        if (param->hints & NATIVE_PARAMETER_IS_BOOLEAN)
+        {
+            realValue = value > 0.5f ? param->ranges.max : param->ranges.min;
+        }
+        else
+        {
+            realValue = param->ranges.min + ((param->ranges.max - param->ranges.min) * value);
+
+            if (param->hints & NATIVE_PARAMETER_IS_INTEGER)
+                realValue = std::round(realValue);
+        }
+
+        fDescriptor->set_parameter_value(fHandle, uindex, realValue);
     }
 
-    void vst_processReplacing(const float** const inputs, float** const outputs, const int32_t sampleFrames)
+    // FIXME for v3.0, use const for the input buffer
+    void vst_processReplacing(float** const inputs, float** const outputs, const int32_t sampleFrames)
     {
         if (sampleFrames <= 0)
             return;
@@ -443,7 +635,7 @@ public:
             vst_dispatcher(effMainsChanged, 0, 1, nullptr, 0.0f);
         }
 
-        static const int kWantVstTimeFlags(kVstTransportPlaying|kVstPpqPosValid|kVstTempoValid|kVstTimeSigValid);
+        static const int kWantVstTimeFlags = kVstTransportPlaying|kVstPpqPosValid|kVstTempoValid|kVstTimeSigValid;
 
         if (const VstTimeInfo* const vstTimeInfo = (const VstTimeInfo*)hostCallback(audioMasterGetTime, 0, kWantVstTimeFlags))
         {
@@ -461,15 +653,23 @@ public:
 
             if (vstTimeInfo->flags & (kVstPpqPosValid|kVstTimeSigValid))
             {
+                const double ppqPos    = std::abs(vstTimeInfo->ppqPos);
                 const int    ppqPerBar = vstTimeInfo->timeSigNumerator * 4 / vstTimeInfo->timeSigDenominator;
-                const double barBeats  = (std::fmod(vstTimeInfo->ppqPos, ppqPerBar) / ppqPerBar) * vstTimeInfo->timeSigDenominator;
+                const double barBeats  = (std::fmod(ppqPos, ppqPerBar) / ppqPerBar) * vstTimeInfo->timeSigNumerator;
                 const double rest      =  std::fmod(barBeats, 1.0);
 
-                fTimeInfo.bbt.bar         = static_cast<int32_t>(vstTimeInfo->ppqPos)/ppqPerBar + 1;
-                fTimeInfo.bbt.beat        = static_cast<int32_t>(barBeats-rest+1.0);
-                fTimeInfo.bbt.tick        = static_cast<int32_t>(rest*fTimeInfo.bbt.ticksPerBeat+0.5);
+                fTimeInfo.bbt.bar         = static_cast<int32_t>(ppqPos) / ppqPerBar + 1;
+                fTimeInfo.bbt.beat        = static_cast<int32_t>(barBeats - rest + 0.5) + 1;
+                fTimeInfo.bbt.tick        = static_cast<int32_t>(rest * fTimeInfo.bbt.ticksPerBeat + 0.5);
                 fTimeInfo.bbt.beatsPerBar = static_cast<float>(vstTimeInfo->timeSigNumerator);
                 fTimeInfo.bbt.beatType    = static_cast<float>(vstTimeInfo->timeSigDenominator);
+
+                if (vstTimeInfo->ppqPos < 0.0)
+                {
+                    --fTimeInfo.bbt.bar;
+                    fTimeInfo.bbt.beat = vstTimeInfo->timeSigNumerator - fTimeInfo.bbt.beat + 1;
+                    fTimeInfo.bbt.tick = fTimeInfo.bbt.ticksPerBeat - fTimeInfo.bbt.tick - 1;
+                }
             }
             else
             {
@@ -480,14 +680,17 @@ public:
                 fTimeInfo.bbt.beatType    = 4.0f;
             }
 
-            fTimeInfo.bbt.barStartTick = fTimeInfo.bbt.ticksPerBeat*fTimeInfo.bbt.beatsPerBar*(fTimeInfo.bbt.bar-1);
+            fTimeInfo.bbt.barStartTick = fTimeInfo.bbt.ticksPerBeat *
+                                         static_cast<double>(fTimeInfo.bbt.beatsPerBar) *
+                                         (fTimeInfo.bbt.bar - 1);
         }
 
         fMidiOutEvents.numEvents = 0;
 
         if (fHandle != nullptr)
-            // FIXME
-            fDescriptor->process(fHandle, const_cast<float**>(inputs), outputs, static_cast<uint32_t>(sampleFrames), fMidiEvents, fMidiEventCount);
+            fDescriptor->process(fHandle,
+                                 inputs, outputs, static_cast<uint32_t>(sampleFrames),
+                                 fMidiEvents, fMidiEventCount);
 
         fMidiEventCount = 0;
 
@@ -527,8 +730,19 @@ protected:
         return true;
     }
 
-    void handleUiParameterChanged(const uint32_t /*index*/, const float /*value*/) const
+    void handleUiParameterChanged(const uint32_t index, const float value) const
     {
+        const NativeParameter* const param = fDescriptor->get_parameter_info(fHandle, index);
+        CARLA_SAFE_ASSERT_RETURN(param != nullptr,);
+
+        const float normalizedValue = (value - param->ranges.min) / (param->ranges.max - param->ranges.min);
+
+        hostCallback(audioMasterAutomate, static_cast<int32_t>(index), 0, nullptr, normalizedValue);
+    }
+
+    void handleUiParameterTouch(const uint32_t index, const bool touch) const
+    {
+        hostCallback(touch ? audioMasterBeginEdit : audioMasterEndEdit, static_cast<int32_t>(index));
     }
 
     void handleUiCustomDataChanged(const char* const /*key*/, const char* const /*value*/) const
@@ -553,7 +767,8 @@ protected:
 
     intptr_t handleDispatcher(const NativeHostDispatcherOpcode opcode, const int32_t index, const intptr_t value, void* const ptr, const float opt)
     {
-        carla_debug("NativePlugin::handleDispatcher(%i, %i, " P_INTPTR ", %p, %f)", opcode, index, value, ptr, opt);
+        carla_debug("NativePlugin::handleDispatcher(%i, %i, " P_INTPTR ", %p, %f)",
+                    opcode, index, value, ptr, static_cast<double>(opt));
 
         switch (opcode)
         {
@@ -562,25 +777,36 @@ protected:
         case NATIVE_HOST_OPCODE_UPDATE_MIDI_PROGRAM:
         case NATIVE_HOST_OPCODE_RELOAD_PARAMETERS:
         case NATIVE_HOST_OPCODE_RELOAD_MIDI_PROGRAMS:
-        case NATIVE_HOST_OPCODE_RELOAD_ALL:
         case NATIVE_HOST_OPCODE_UI_UNAVAILABLE:
         case NATIVE_HOST_OPCODE_INTERNAL_PLUGIN:
+        case NATIVE_HOST_OPCODE_QUEUE_INLINE_DISPLAY:
+        case NATIVE_HOST_OPCODE_REQUEST_IDLE:
+        case NATIVE_HOST_OPCODE_GET_FILE_PATH:
+            // nothing
+            break;
+
+        case NATIVE_HOST_OPCODE_RELOAD_ALL:
+            hostCallback(audioMasterUpdateDisplay);
             break;
 
         case NATIVE_HOST_OPCODE_HOST_IDLE:
             hostCallback(audioMasterIdle);
             break;
+
+        case NATIVE_HOST_OPCODE_UI_TOUCH_PARAMETER:
+            CARLA_SAFE_ASSERT_RETURN(index >= 0, 0);
+            handleUiParameterTouch(static_cast<uint32_t>(index), value != 0);
+            break;
         }
 
         // unused for now
         return 0;
 
-        (void)index; (void)value; (void)ptr; (void)opt;
+        (void)ptr; (void)opt;
     }
 
 private:
     // VST stuff
-    const audioMasterCallback fAudioMaster;
     AEffect* const fEffect;
 
     // Native data
@@ -596,9 +822,13 @@ private:
     bool            fIsActive;
     uint32_t        fMidiEventCount;
     NativeMidiEvent fMidiEvents[kMaxMidiEvents];
+    char            fProgramName[32+1];
     NativeTimeInfo  fTimeInfo;
     ERect           fVstRect;
 
+    // UI button
+    CarlaUILauncher* fUiLauncher;
+
     // Host data
     enum HostType {
         kHostTypeNull = 0,
@@ -612,9 +842,9 @@ private:
                           const int32_t index = 0,
                           const intptr_t value = 0,
                           void* const ptr = nullptr,
-                          const float opt = 0.0f)
+                          const float opt = 0.0f) const
     {
-        return fAudioMaster(fEffect, opcode, index, value, ptr, opt);
+        return VSTAudioMaster(fEffect, opcode, index, value, ptr, opt);
     }
 
     struct FixedVstEvents {
@@ -635,6 +865,10 @@ private:
         CARLA_DECLARE_NON_COPY_STRUCT(FixedVstEvents);
     } fMidiOutEvents;
 
+#ifdef USING_JUCE
+    juce::SharedResourcePointer<juce::ScopedJuceInitialiser_GUI> fJuceInitialiser;
+#endif
+
     char* fStateChunk;
 
     // -------------------------------------------------------------------
@@ -704,17 +938,12 @@ private:
 
 // -----------------------------------------------------------------------
 
-struct VstObject {
-    audioMasterCallback audioMaster;
-    NativePlugin* plugin;
-};
-
 #define validObject  effect != nullptr && effect->object != nullptr
 #define validPlugin  effect != nullptr && effect->object != nullptr && ((VstObject*)effect->object)->plugin != nullptr
 #define vstObjectPtr (VstObject*)effect->object
 #define pluginPtr    (vstObjectPtr)->plugin
 
-static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t index, intptr_t value, void* ptr, float opt)
+intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t index, intptr_t value, void* ptr, float opt)
 {
     // handle base opcodes
     switch (opcode)
@@ -728,10 +957,8 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
             // some hosts call effOpen twice
             CARLA_SAFE_ASSERT_RETURN(obj->plugin == nullptr, 1);
 
-            const audioMasterCallback audioMaster = (audioMasterCallback)obj->audioMaster;
-
-            d_lastBufferSize = static_cast<uint32_t>(audioMaster(effect, audioMasterGetBlockSize, 0, 0, nullptr, 0.0f));
-            d_lastSampleRate = static_cast<double>(audioMaster(effect, audioMasterGetSampleRate, 0, 0, nullptr, 0.0f));
+            d_lastBufferSize = static_cast<uint32_t>(VSTAudioMaster(effect, audioMasterGetBlockSize, 0, 0, nullptr, 0.0f));
+            d_lastSampleRate = static_cast<double>(VSTAudioMaster(effect, audioMasterGetSampleRate, 0, 0, nullptr, 0.0f));
 
             // some hosts are not ready at this point or return 0 buffersize/samplerate
             if (d_lastBufferSize == 0)
@@ -742,7 +969,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
             const NativePluginDescriptor* pluginDesc = nullptr;
             PluginListManager& plm(PluginListManager::getInstance());
 
-           #if CARLA_VST_SHELL
+           #ifdef CARLA_VST_SHELL
             if (effect->uniqueID == 0)
                 effect->uniqueID = kShellUniqueID;
 
@@ -759,19 +986,21 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
             const int32_t plugIndex = effect->uniqueID - kShellUniqueID - 1;
             CARLA_SAFE_ASSERT_RETURN(plugIndex >= 0, 0);
 
-            pluginDesc = plm.descs.getAt(plugIndex, nullptr);
+            pluginDesc = plm.descs.getAt(static_cast<size_t>(plugIndex), nullptr);
 
            #else // CARLA_VST_SHELL
 
-           #  if CARLA_PLUGIN_32CH
+           # if defined(CARLA_PLUGIN_64CH)
+            const char* const pluginLabel = "carlapatchbay64";
+           # elif defined(CARLA_PLUGIN_32CH)
             const char* const pluginLabel = "carlapatchbay32";
-           #  elif CARLA_PLUGIN_16CH
+           # elif defined(CARLA_PLUGIN_16CH)
             const char* const pluginLabel = "carlapatchbay16";
-           #  elif CARLA_PLUGIN_PATCHBAY
+           # elif CARLA_PLUGIN_PATCHBAY
             const char* const pluginLabel = "carlapatchbay";
-           #  else
+           # else
             const char* const pluginLabel = "carlarack";
-           #  endif
+           # endif
 
             for (LinkedList<const NativePluginDescriptor*>::Itenerator it = plm.descs.begin2(); it.valid(); it.next())
             {
@@ -788,26 +1017,31 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
 
             CARLA_SAFE_ASSERT_RETURN(pluginDesc != nullptr, 0);
 
-           #if CARLA_VST_SHELL
-            effect->numParams   = 0;
-            effect->numPrograms = 0;
-            effect->numInputs   = pluginDesc->audioIns;
-            effect->numOutputs  = pluginDesc->audioOuts;
+           #ifdef CARLA_VST_SHELL
+            effect->numPrograms = 1;
+            effect->numParams   = static_cast<int>(pluginDesc->paramIns);
+            effect->numInputs   = static_cast<int>(pluginDesc->audioIns);
+            effect->numOutputs  = static_cast<int>(pluginDesc->audioOuts);
 
-            /*
             if (pluginDesc->hints & NATIVE_PLUGIN_HAS_UI)
                 effect->flags |= effFlagsHasEditor;
             else
                 effect->flags &= ~effFlagsHasEditor;
-            */
 
+            /* carla as plugin always has NATIVE_PLUGIN_IS_SYNTH set
             if (pluginDesc->hints & NATIVE_PLUGIN_IS_SYNTH)
                 effect->flags |= effFlagsIsSynth;
             else
                 effect->flags &= ~effFlagsIsSynth;
+            */
            #endif // CARLA_VST_SHELL
 
-            obj->plugin = new NativePlugin(audioMaster, effect, pluginDesc);
+           #if CARLA_PLUGIN_SYNTH
+            // override if requested
+            effect->flags |= effFlagsIsSynth;
+           #endif
+
+            obj->plugin = new NativePlugin(effect, pluginDesc);
             return 1;
         }
         return 0;
@@ -837,11 +1071,15 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
         return 0;
 
     case effGetPlugCategory:
-#if CARLA_VST_SHELL
+#ifdef CARLA_VST_SHELL
         if (validPlugin)
         {
+           #if CARLA_PLUGIN_SYNTH
+            return kPlugCategSynth;
+           #else
             const NativePluginDescriptor* const desc = pluginPtr->getDescriptor();
             return desc->category == NATIVE_PLUGIN_CATEGORY_SYNTH ? kPlugCategSynth : kPlugCategEffect;
+           #endif
         }
 
         return kPlugCategShell;
@@ -854,19 +1092,25 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetEffectName:
         if (char* const cptr = (char*)ptr)
         {
-           #if CARLA_VST_SHELL
+           #if defined(CARLA_VST_SHELL)
             if (validPlugin)
             {
                 const NativePluginDescriptor* const desc = pluginPtr->getDescriptor();
+               #if CARLA_PLUGIN_SYNTH
                 std::strncpy(cptr, desc->name, 32);
+               #else
+                std::snprintf(cptr, 32, "%s FX", desc->name);
+               #endif
             }
             else
             {
                 std::strncpy(cptr, "Carla-VstShell", 32);
             }
-           #elif CARLA_PLUGIN_32CH
+           #elif defined(CARLA_PLUGIN_64CH)
+            std::strncpy(cptr, "Carla-Patchbay64", 32);
+           #elif defined(CARLA_PLUGIN_32CH)
             std::strncpy(cptr, "Carla-Patchbay32", 32);
-           #elif CARLA_PLUGIN_16CH
+           #elif defined(CARLA_PLUGIN_16CH)
             std::strncpy(cptr, "Carla-Patchbay16", 32);
            #elif CARLA_PLUGIN_PATCHBAY
            #  if CARLA_PLUGIN_SYNTH
@@ -888,7 +1132,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetVendorString:
         if (char* const cptr = (char*)ptr)
         {
-           #if CARLA_VST_SHELL
+           #ifdef CARLA_VST_SHELL
             if (validPlugin)
             {
                 const NativePluginDescriptor* const desc = pluginPtr->getDescriptor();
@@ -904,19 +1148,25 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetProductString:
         if (char* const cptr = (char*)ptr)
         {
-           #if CARLA_VST_SHELL
+           #if defined(CARLA_VST_SHELL)
             if (validPlugin)
             {
                 const NativePluginDescriptor* const desc = pluginPtr->getDescriptor();
+               #if CARLA_PLUGIN_SYNTH
                 std::strncpy(cptr, desc->label, 32);
+               #else
+                std::snprintf(cptr, 32, "%sFX", desc->label);
+               #endif
             }
             else
             {
                 std::strncpy(cptr, "CarlaVstShell", 32);
             }
-           #elif CARLA_PLUGIN_32CH
+           #elif defined(CARLA_PLUGIN_64CH)
+            std::strncpy(cptr, "CarlaPatchbay64", 32);
+           #elif defined(CARLA_PLUGIN_32CH)
             std::strncpy(cptr, "CarlaPatchbay32", 32);
-           #elif CARLA_PLUGIN_16CH
+           #elif defined(CARLA_PLUGIN_16CH)
             std::strncpy(cptr, "CarlaPatchbay16", 32);
            #elif CARLA_PLUGIN_PATCHBAY
            #  if CARLA_PLUGIN_SYNTH
@@ -941,7 +1191,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetVstVersion:
         return kVstVersion;
 
-#if CARLA_VST_SHELL
+#ifdef CARLA_VST_SHELL
     case effShellGetNextPlugin:
         if (char* const cptr = (char*)ptr)
         {
@@ -952,27 +1202,28 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
 
             for (;;)
             {
-                const uint index = effect->uniqueID - kShellUniqueID;
+                const uint index2 = static_cast<uint>(effect->uniqueID - kShellUniqueID);
 
-                if (index >= plm.descs.count())
+                if (index2 >= plm.descs.count())
                 {
                     effect->uniqueID = kShellUniqueID;
                     return 0;
                 }
 
-                const NativePluginDescriptor* const desc = plm.descs.getAt(index, nullptr);
+                const NativePluginDescriptor* const desc = plm.descs.getAt(index2, nullptr);
                 CARLA_SAFE_ASSERT_RETURN(desc != nullptr, 0);
 
                 ++effect->uniqueID;
 
                 if (desc->midiIns > 1 || desc->midiOuts > 1)
                     continue;
-                if (std::strncmp(desc->label, "carlarack", 9) == 0)
-                    continue;
-                if (std::strncmp(desc->label, "carlapatchbay", 13) == 0)
-                    continue;
 
+               #if CARLA_PLUGIN_SYNTH
                 std::strncpy(cptr, desc->label, 32);
+               #else
+                std::snprintf(cptr, 32, "%sFX", desc->label);
+               #endif
+
                 return effect->uniqueID;
             }
         }
@@ -986,29 +1237,29 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     return 0;
 }
 
-static float vst_getParameterCallback(AEffect* effect, int32_t index)
+float vst_getParameterCallback(AEffect* effect, int32_t index)
 {
     if (validPlugin)
         return pluginPtr->vst_getParameter(index);
     return 0.0f;
 }
 
-static void vst_setParameterCallback(AEffect* effect, int32_t index, float value)
+void vst_setParameterCallback(AEffect* effect, int32_t index, float value)
 {
     if (validPlugin)
         pluginPtr->vst_setParameter(index, value);
 }
 
-static void vst_processCallback(AEffect* effect, float** inputs, float** outputs, int32_t sampleFrames)
+void vst_processCallback(AEffect* effect, float** inputs, float** outputs, int32_t sampleFrames)
 {
     if (validPlugin)
-        pluginPtr->vst_processReplacing(const_cast<const float**>(inputs), outputs, sampleFrames);
+        pluginPtr->vst_processReplacing(inputs, outputs, sampleFrames);
 }
 
-static void vst_processReplacingCallback(AEffect* effect, float** inputs, float** outputs, int32_t sampleFrames)
+void vst_processReplacingCallback(AEffect* effect, float** inputs, float** outputs, int32_t sampleFrames)
 {
     if (validPlugin)
-        pluginPtr->vst_processReplacing(const_cast<const float**>(inputs), outputs, sampleFrames);
+        pluginPtr->vst_processReplacing(inputs, outputs, sampleFrames);
 }
 
 #undef pluginPtr
@@ -1018,35 +1269,18 @@ static void vst_processReplacingCallback(AEffect* effect, float** inputs, float*
 
 // -----------------------------------------------------------------------
 
-CARLA_EXPORT
-#if defined(CARLA_OS_WIN) || defined(CARLA_OS_MAC)
-const AEffect* VSTPluginMain(audioMasterCallback audioMaster);
-#else
-const AEffect* VSTPluginMain(audioMasterCallback audioMaster) asm ("main");
-#endif
-
-CARLA_EXPORT
-const AEffect* VSTPluginMain(audioMasterCallback audioMaster)
+const AEffect* VSTPluginMainInit(AEffect* const effect)
 {
-    // old version
-    if (audioMaster(nullptr, audioMasterVersion, 0, 0, nullptr, 0.0f) == 0)
-        return nullptr;
-
-    AEffect* const effect(new AEffect);
-    std::memset(effect, 0, sizeof(AEffect));
-
-    // vst fields
-    effect->magic   = kEffectMagic;
-    effect->version = CARLA_VERSION_HEX;
-
-   #if CARLA_VST_SHELL
-    if (const intptr_t uniqueID = audioMaster(nullptr, audioMasterCurrentId, 0, 0, nullptr, 0.0f))
-        effect->uniqueID = uniqueID;
+   #if defined(CARLA_VST_SHELL)
+    if (const intptr_t uniqueID = VSTAudioMaster(effect, audioMasterCurrentId, 0, 0, nullptr, 0.0f))
+        effect->uniqueID = static_cast<int>(uniqueID);
     else
         effect->uniqueID = kShellUniqueID;
-   #elif CARLA_PLUGIN_32CH
+   #elif defined(CARLA_PLUGIN_64CH)
+    effect->uniqueID = kBaseUniqueID+7;
+   #elif defined(CARLA_PLUGIN_32CH)
     effect->uniqueID = kBaseUniqueID+6;
-   #elif CARLA_PLUGIN_16CH
+   #elif defined(CARLA_PLUGIN_16CH)
     effect->uniqueID = kBaseUniqueID+5;
    #elif CARLA_PLUGIN_PATCHBAY
    #  if CARLA_PLUGIN_SYNTH
@@ -1063,46 +1297,33 @@ const AEffect* VSTPluginMain(audioMasterCallback audioMaster)
    #endif
 
     // plugin fields
-    effect->numParams   = 0;
-    effect->numPrograms = 0;
-#if CARLA_VST_SHELL
-    effect->numInputs   = 0;
-    effect->numOutputs  = 0;
-#elif CARLA_PLUGIN_32CH
+#ifndef CARLA_VST_SHELL
+    effect->numParams   = kNumParameters;
+    effect->numPrograms = 1;
+# if defined(CARLA_PLUGIN_64CH)
+    effect->numInputs   = 64;
+    effect->numOutputs  = 64;
+# elif defined(CARLA_PLUGIN_32CH)
     effect->numInputs   = 32;
     effect->numOutputs  = 32;
-#elif CARLA_PLUGIN_16CH
+# elif defined(CARLA_PLUGIN_16CH)
     effect->numInputs   = 16;
     effect->numOutputs  = 16;
-#else
+# else
     effect->numInputs   = 2;
     effect->numOutputs  = 2;
+# endif
 #endif
 
     // plugin flags
     effect->flags |= effFlagsCanReplacing;
     effect->flags |= effFlagsProgramChunks;
-#if ! CARLA_VST_SHELL
+#ifndef CARLA_VST_SHELL
     effect->flags |= effFlagsHasEditor;
-#  if CARLA_PLUGIN_SYNTH
-    effect->flags |= effFlagsIsSynth;
-#  endif
 #endif
-
-    // static calls
-    effect->dispatcher   = vst_dispatcherCallback;
-    effect->process      = vst_processCallback;
-    effect->getParameter = vst_getParameterCallback;
-    effect->setParameter = vst_setParameterCallback;
-    effect->processReplacing = vst_processReplacingCallback;
-
-    // pointers
-    VstObject* const obj(new VstObject());
-    obj->audioMaster = audioMaster;
-    obj->plugin      = nullptr;
-
-    // done
-    effect->object = obj;
+#if CARLA_PLUGIN_SYNTH
+    effect->flags |= effFlagsIsSynth;
+#endif
 
     return effect;
 }
