@@ -1,6 +1,6 @@
 /*
  * Carla Pipe Utilities
- * Copyright (C) 2013-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2013-2019 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,7 +32,6 @@
 # include "lv2/lv2plug.in/ns/ext/atom/util.h"
 #endif
 
-#include <clocale>
 #include <fcntl.h>
 
 #include "water/misc/Time.h"
@@ -321,8 +320,8 @@ bool waitForClientConnect(const HANDLE pipe, const HANDLE event, const HANDLE pr
 static inline
 bool startProcess(const char* const argv[], pid_t& pidinst) noexcept
 {
-    const ScopedEnvVar sev1("LD_LIBRARY_PATH", nullptr);
-    const ScopedEnvVar sev2("LD_PRELOAD", nullptr);
+    const CarlaScopedEnvVar sev1("LD_LIBRARY_PATH", nullptr);
+    const CarlaScopedEnvVar sev2("LD_PRELOAD", nullptr);
 
     const pid_t ret = pidinst = vfork();
 
@@ -597,7 +596,7 @@ struct CarlaPipeCommon::PrivateData {
     CarlaMutex writeLock;
 
     // temporary buffers for _readline()
-    mutable char        tmpBuf[0xff+1];
+    mutable char        tmpBuf[0xffff];
     mutable CarlaString tmpStr;
 
     PrivateData() noexcept
@@ -626,7 +625,7 @@ struct CarlaPipeCommon::PrivateData {
         ovSend = ::CreateEvent(nullptr, FALSE, FALSE, nullptr);
 #endif
 
-        carla_zeroChars(tmpBuf, 0xff+1);
+        carla_zeroChars(tmpBuf, 0xffff);
     }
 
     CARLA_DECLARE_NON_COPY_STRUCT(PrivateData)
@@ -656,20 +655,17 @@ bool CarlaPipeCommon::isPipeRunning() const noexcept
 
 void CarlaPipeCommon::idlePipe(const bool onlyOnce) noexcept
 {
-    const char* locale = nullptr;
+    bool readSucess;
 
     for (;;)
     {
-        const char* const msg(_readline());
+        readSucess = false;
+        const char* const msg = _readline(true, 0, readSucess);
 
-        if (msg == nullptr)
+        if (! readSucess)
             break;
-
-        if (locale == nullptr && ! onlyOnce)
-        {
-            locale = carla_strdup_safe(::setlocale(LC_NUMERIC, nullptr));
-            ::setlocale(LC_NUMERIC, "C");
-        }
+        if (msg == nullptr)
+            continue;
 
         pData->isReading = true;
 
@@ -691,12 +687,6 @@ void CarlaPipeCommon::idlePipe(const bool onlyOnce) noexcept
         if (onlyOnce || pData->pipeRecv == INVALID_PIPE_VALUE)
             break;
     }
-
-    if (locale != nullptr)
-    {
-        ::setlocale(LC_NUMERIC, locale);
-        delete[] locale;
-    }
 }
 
 // -------------------------------------------------------------------
@@ -727,10 +717,9 @@ bool CarlaPipeCommon::readNextLineAsBool(bool& value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (const char* const msg = _readlineblock(false))
     {
         value = (std::strcmp(msg, "true") == 0);
-        delete[] msg;
         return true;
     }
 
@@ -741,14 +730,13 @@ bool CarlaPipeCommon::readNextLineAsByte(uint8_t& value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (const char* const msg = _readlineblock(false))
     {
-        int tmp = std::atoi(msg);
-        delete[] msg;
+        const int asint = std::atoi(msg);
 
-        if (tmp >= 0 && tmp <= 0xFF)
+        if (asint >= 0 && asint <= 0xFF)
         {
-            value = static_cast<uint8_t>(tmp);
+            value = static_cast<uint8_t>(asint);
             return true;
         }
     }
@@ -760,10 +748,9 @@ bool CarlaPipeCommon::readNextLineAsInt(int32_t& value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (const char* const msg = _readlineblock(false))
     {
         value = std::atoi(msg);
-        delete[] msg;
         return true;
     }
 
@@ -774,14 +761,13 @@ bool CarlaPipeCommon::readNextLineAsUInt(uint32_t& value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (const char* const msg = _readlineblock(false))
     {
-        int32_t tmp = std::atoi(msg);
-        delete[] msg;
+        const long aslong = std::atol(msg);
 
-        if (tmp >= 0)
+        if (aslong >= 0)
         {
-            value = static_cast<uint32_t>(tmp);
+            value = static_cast<uint32_t>(aslong);
             return true;
         }
     }
@@ -793,10 +779,9 @@ bool CarlaPipeCommon::readNextLineAsLong(int64_t& value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (const char* const msg = _readlineblock(false))
     {
         value = std::atol(msg);
-        delete[] msg;
         return true;
     }
 
@@ -807,14 +792,13 @@ bool CarlaPipeCommon::readNextLineAsULong(uint64_t& value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (const char* const msg = _readlineblock(false))
     {
-        int64_t tmp = std::atol(msg);
-        delete[] msg;
+        const int64_t asint64 = std::atol(msg);
 
-        if (tmp >= 0)
+        if (asint64 >= 0)
         {
-            value = static_cast<uint64_t>(tmp);
+            value = static_cast<uint64_t>(asint64);
             return true;
         }
     }
@@ -826,10 +810,12 @@ bool CarlaPipeCommon::readNextLineAsFloat(float& value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (const char* const msg = _readlineblock(false))
     {
-        value = static_cast<float>(std::atof(msg));
-        delete[] msg;
+        {
+            const CarlaScopedLocale csl;
+            value = static_cast<float>(std::atof(msg));
+        }
         return true;
     }
 
@@ -840,21 +826,26 @@ bool CarlaPipeCommon::readNextLineAsDouble(double& value) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (const char* const msg = _readlineblock(false))
     {
-        value = std::atof(msg);
-        delete[] msg;
+        {
+            const CarlaScopedLocale csl;
+            value = std::atof(msg);
+        }
         return true;
     }
 
     return false;
 }
 
-bool CarlaPipeCommon::readNextLineAsString(const char*& value) const noexcept
+bool CarlaPipeCommon::readNextLineAsString(const char*& value, const bool allocateString, uint32_t size) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->isReading, false);
 
-    if (const char* const msg = _readlineblock())
+    if (size >= 0xffff)
+        size = 0;
+
+    if (const char* const msg = _readlineblock(allocateString, static_cast<uint16_t>(size)))
     {
         value = msg;
         return true;
@@ -934,15 +925,23 @@ bool CarlaPipeCommon::writeAndFixMessage(const char* const msg) const noexcept
     return _writeMsgBuffer(fixedMsg, size+1);
 }
 
+bool CarlaPipeCommon::writeEmptyMessage() const noexcept
+{
+    if (pData->pipeClosed)
+        return false;
+
+    return _writeMsgBuffer("\n", 1);
+}
+
 bool CarlaPipeCommon::flushMessages() const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->pipeSend != INVALID_PIPE_VALUE, false);
 
 #if defined(CARLA_OS_LINUX) || defined(CARLA_OS_GNU_HURD)
-#  if defined(__GLIBC__) && (__GLIBC__ * 1000 + __GLIBC_MINOR__) >= 2014
+# if defined(__GLIBC__) && (__GLIBC__ * 1000 + __GLIBC_MINOR__) >= 2014
     // the only call that seems to do something
     return ::syncfs(pData->pipeSend) == 0;
-#  endif
+# endif
 #elif 0 // defined(CARLA_OS_WIN)
     // FIXME causes issues
     try {
@@ -971,21 +970,21 @@ void CarlaPipeCommon::writeErrorMessage(const char* const error) const noexcept
 
 void CarlaPipeCommon::writeControlMessage(const uint32_t index, const float value) const noexcept
 {
-    char tmpBuf[0xff+1];
-    tmpBuf[0xff] = '\0';
+    char tmpBuf[0xff];
+    tmpBuf[0xfe] = '\0';
 
     const CarlaMutexLocker cml(pData->writeLock);
 
     if (! _writeMsgBuffer("control\n", 8))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", index);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", index);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
     {
-        const ScopedLocale csl;
-        std::snprintf(tmpBuf, 0xff, "%f\n", value);
+        const CarlaScopedLocale csl;
+        std::snprintf(tmpBuf, 0xfe, "%.12g\n", static_cast<double>(value));
     }
 
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
@@ -1013,15 +1012,15 @@ void CarlaPipeCommon::writeConfigureMessage(const char* const key, const char* c
 
 void CarlaPipeCommon::writeProgramMessage(const uint32_t index) const noexcept
 {
-    char tmpBuf[0xff+1];
-    tmpBuf[0xff] = '\0';
+    char tmpBuf[0xff];
+    tmpBuf[0xfe] = '\0';
 
     const CarlaMutexLocker cml(pData->writeLock);
 
     if (! _writeMsgBuffer("program\n", 8))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", index);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", index);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
@@ -1030,23 +1029,23 @@ void CarlaPipeCommon::writeProgramMessage(const uint32_t index) const noexcept
 
 void CarlaPipeCommon::writeProgramMessage(const uint8_t channel, const uint32_t bank, const uint32_t program) const noexcept
 {
-    char tmpBuf[0xff+1];
-    tmpBuf[0xff] = '\0';
+    char tmpBuf[0xff];
+    tmpBuf[0xfe] = '\0';
 
     const CarlaMutexLocker cml(pData->writeLock);
 
     if (! _writeMsgBuffer("program\n", 8))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", channel);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", channel);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", bank);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", bank);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", program);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", program);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
@@ -1055,19 +1054,19 @@ void CarlaPipeCommon::writeProgramMessage(const uint8_t channel, const uint32_t
 
 void CarlaPipeCommon::writeMidiProgramMessage(const uint32_t bank, const uint32_t program) const noexcept
 {
-    char tmpBuf[0xff+1];
-    tmpBuf[0xff] = '\0';
+    char tmpBuf[0xff];
+    tmpBuf[0xfe] = '\0';
 
     const CarlaMutexLocker cml(pData->writeLock);
 
     if (! _writeMsgBuffer("midiprogram\n", 12))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", bank);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", bank);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", program);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", program);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
@@ -1076,15 +1075,15 @@ void CarlaPipeCommon::writeMidiProgramMessage(const uint32_t bank, const uint32_
 
 void CarlaPipeCommon::writeReloadProgramsMessage(const int32_t index) const noexcept
 {
-    char tmpBuf[0xff+1];
-    tmpBuf[0xff] = '\0';
+    char tmpBuf[0xff];
+    tmpBuf[0xfe] = '\0';
 
     const CarlaMutexLocker cml(pData->writeLock);
 
     if (! _writeMsgBuffer("reloadprograms\n", 15))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", index);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", index);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
@@ -1097,27 +1096,27 @@ void CarlaPipeCommon::writeMidiNoteMessage(const bool onOff, const uint8_t chann
     CARLA_SAFE_ASSERT_RETURN(note < MAX_MIDI_NOTE,);
     CARLA_SAFE_ASSERT_RETURN(velocity < MAX_MIDI_VALUE,);
 
-    char tmpBuf[0xff+1];
-    tmpBuf[0xff] = '\0';
+    char tmpBuf[0xff];
+    tmpBuf[0xfe] = '\0';
 
     const CarlaMutexLocker cml(pData->writeLock);
 
     if (! _writeMsgBuffer("note\n", 5))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%s\n", bool2str(onOff));
+    std::snprintf(tmpBuf, 0xfe, "%s\n", bool2str(onOff));
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", channel);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", channel);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", note);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", note);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", velocity);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", velocity);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
@@ -1128,8 +1127,8 @@ void CarlaPipeCommon::writeLv2AtomMessage(const uint32_t index, const LV2_Atom*
 {
     CARLA_SAFE_ASSERT_RETURN(atom != nullptr,);
 
-    char tmpBuf[0xff+1];
-    tmpBuf[0xff] = '\0';
+    char tmpBuf[0xff];
+    tmpBuf[0xfe] = '\0';
 
     const uint32_t atomTotalSize(lv2_atom_total_size(atom));
     CarlaString base64atom(CarlaString::asBase64(atom, atomTotalSize));
@@ -1139,11 +1138,15 @@ void CarlaPipeCommon::writeLv2AtomMessage(const uint32_t index, const LV2_Atom*
     if (! _writeMsgBuffer("atom\n", 5))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", index);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", index);
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", atomTotalSize);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", atomTotalSize);
+    if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
+        return;
+
+    std::snprintf(tmpBuf, 0xfe, "%lu\n", static_cast<long unsigned>(base64atom.length()));
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
@@ -1158,15 +1161,19 @@ void CarlaPipeCommon::writeLv2UridMessage(const uint32_t urid, const char* const
     CARLA_SAFE_ASSERT_RETURN(urid != 0,);
     CARLA_SAFE_ASSERT_RETURN(uri != nullptr && uri[0] != '\0',);
 
-    char tmpBuf[0xff+1];
-    tmpBuf[0xff] = '\0';
+    char tmpBuf[0xff];
+    tmpBuf[0xfe] = '\0';
 
     const CarlaMutexLocker cml(pData->writeLock);
 
     if (! _writeMsgBuffer("urid\n", 5))
         return;
 
-    std::snprintf(tmpBuf, 0xff, "%i\n", urid);
+    std::snprintf(tmpBuf, 0xfe, "%i\n", urid);
+    if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
+        return;
+
+    std::snprintf(tmpBuf, 0xfe, "%lu\n", static_cast<long unsigned>(std::strlen(uri)));
     if (! _writeMsgBuffer(tmpBuf, std::strlen(tmpBuf)))
         return;
 
@@ -1179,46 +1186,103 @@ void CarlaPipeCommon::writeLv2UridMessage(const uint32_t urid, const char* const
 // -------------------------------------------------------------------
 
 // internal
-const char* CarlaPipeCommon::_readline() const noexcept
+const char* CarlaPipeCommon::_readline(const bool allocReturn, const uint16_t size, bool& readSucess) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(pData->pipeRecv != INVALID_PIPE_VALUE, nullptr);
 
     char    c;
     char*   ptr = pData->tmpBuf;
     ssize_t ret = -1;
+    bool tooBig = false;
 
     pData->tmpStr.clear();
 
-    for (int i=0; i<0xff; ++i)
+    if (size == 0 || size == 1)
     {
-        try {
-#ifdef CARLA_OS_WIN
-            ret = ReadFileWin32(pData->pipeRecv, pData->ovRecv, &c, 1);
-#else
-            ret = ::read(pData->pipeRecv, &c, 1);
-#endif
-        } CARLA_SAFE_EXCEPTION_BREAK("CarlaPipeCommon::readline() - read");
-
-        if (ret != 1 || c == '\n')
-            break;
-
-        if (c == '\r')
-            c = '\n';
-
-        *ptr++ = c;
-
-        if (i+1 == 0xff)
+        for (int i=0; i<0xfffe; ++i)
         {
-            i = 0;
+            try {
+    #ifdef CARLA_OS_WIN
+                ret = ReadFileWin32(pData->pipeRecv, pData->ovRecv, &c, 1);
+    #else
+                ret = ::read(pData->pipeRecv, &c, 1);
+    #endif
+            } CARLA_SAFE_EXCEPTION_BREAK("CarlaPipeCommon::readline() - read");
+
+            if (ret != 1 || c == '\n')
+                break;
+
+            if (c == '\r')
+                c = '\n';
+
+            *ptr++ = c;
+
+            if (i+1 == 0xfffe)
+            {
+                i = 0;
+                *ptr = '\0';
+                tooBig = true;
+                pData->tmpStr += pData->tmpBuf;
+                ptr = pData->tmpBuf;
+            }
+        }
+    }
+    else
+    {
+        uint16_t remaining = size;
+        readSucess = false;
+
+        for (;;)
+        {
+            try {
+    #ifdef CARLA_OS_WIN
+                ret = ReadFileWin32(pData->pipeRecv, pData->ovRecv, ptr, remaining);
+    #else
+                ret = ::read(pData->pipeRecv, ptr, remaining);
+    #endif
+            } CARLA_SAFE_EXCEPTION_RETURN("CarlaPipeCommon::readline() - read", nullptr);
+
+            if (ret == -1 && errno == EAGAIN)
+                continue;
+
+            CARLA_SAFE_ASSERT_INT2_RETURN(ret > 0, ret, remaining, nullptr);
+            CARLA_SAFE_ASSERT_INT2_RETURN(ret <= (ssize_t)remaining, ret, remaining, nullptr);
+
+            for (ssize_t i=0; i<ret; ++i)
+            {
+                if (ptr[i] == '\r')
+                    ptr[i] = '\n';
+            }
+
+            ptr += ret;
             *ptr = '\0';
-            pData->tmpStr += pData->tmpBuf;
-            ptr = pData->tmpBuf;
+            remaining = static_cast<uint16_t>(remaining - ret);
+
+            if (remaining != 0)
+                continue;
+
+            readSucess = true;
+
+            if (allocReturn)
+            {
+                pData->tmpStr = pData->tmpBuf;
+                return pData->tmpStr.releaseBufferPointer();
+            }
+
+            return pData->tmpBuf;
         }
     }
 
     if (ptr != pData->tmpBuf)
     {
         *ptr = '\0';
+
+        if (! allocReturn && ! tooBig)
+        {
+            readSucess = true;
+            return pData->tmpBuf;
+        }
+
         pData->tmpStr += pData->tmpBuf;
     }
     else if (pData->tmpStr.isEmpty() && ret != 1)
@@ -1227,18 +1291,27 @@ const char* CarlaPipeCommon::_readline() const noexcept
         return nullptr;
     }
 
-    try {
-        return pData->tmpStr.dup();
-    } CARLA_SAFE_EXCEPTION_RETURN("CarlaPipeCommon::readline() - dup", nullptr);
+    readSucess = true;
+
+    if (! allocReturn && ! tooBig)
+        return pData->tmpBuf;
+
+    return allocReturn ? pData->tmpStr.releaseBufferPointer() : pData->tmpStr.buffer();
 }
 
-const char* CarlaPipeCommon::_readlineblock(const uint32_t timeOutMilliseconds) const noexcept
+const char* CarlaPipeCommon::_readlineblock(const bool allocReturn,
+                                            const uint16_t size,
+                                            const uint32_t timeOutMilliseconds) const noexcept
 {
-    const uint32_t timeoutEnd(water::Time::getMillisecondCounter() + timeOutMilliseconds);
+    const uint32_t timeoutEnd = water::Time::getMillisecondCounter() + timeOutMilliseconds;
+    bool readSucess;
 
     for (;;)
     {
-        if (const char* const msg = _readline())
+        readSucess = false;
+        const char* const msg = _readline(allocReturn, size, readSucess);
+
+        if (readSucess)
             return msg;
 
         if (water::Time::getMillisecondCounter() >= timeoutEnd)
@@ -1247,13 +1320,18 @@ const char* CarlaPipeCommon::_readlineblock(const uint32_t timeOutMilliseconds)
         carla_msleep(5);
     }
 
-    if (std::getenv("CARLA_VALGRIND_TEST") != nullptr)
+    static const bool testingForValgrind = std::getenv("CARLA_VALGRIND_TEST") != nullptr;
+
+    if (testingForValgrind)
     {
-        const uint32_t timeoutEnd2(water::Time::getMillisecondCounter() + 1000);
+        const uint32_t timeoutEnd2 = water::Time::getMillisecondCounter() + 1000;
 
         for (;;)
         {
-            if (const char* const msg = _readline())
+            readSucess = false;
+            const char* const msg = _readline(allocReturn, size, readSucess);
+
+            if (readSucess)
                 return msg;
 
             if (water::Time::getMillisecondCounter() >= timeoutEnd2)
@@ -1623,7 +1701,7 @@ void CarlaPipeServer::stopPipeServer(const uint32_t timeOutMilliseconds) noexcep
     {
         const CarlaMutexLocker cml(pData->writeLock);
 
-        if (pData->pipeSend != INVALID_PIPE_VALUE)
+        if (pData->pipeSend != INVALID_PIPE_VALUE && ! pData->pipeClosed)
         {
             if (_writeMsgBuffer("__carla-quit__\n", 15))
                 flushMessages();
@@ -1641,7 +1719,7 @@ void CarlaPipeServer::stopPipeServer(const uint32_t timeOutMilliseconds) noexcep
     {
         const CarlaMutexLocker cml(pData->writeLock);
 
-        if (pData->pipeSend != INVALID_PIPE_VALUE)
+        if (pData->pipeSend != INVALID_PIPE_VALUE && ! pData->pipeClosed)
         {
             if (_writeMsgBuffer("__carla-quit__\n", 15))
                 flushMessages();
@@ -1843,68 +1921,4 @@ void CarlaPipeClient::writeExitingMessageAndWait() noexcept
 
 // -----------------------------------------------------------------------
 
-ScopedEnvVar::ScopedEnvVar(const char* const key, const char* const value) noexcept
-    : fKey(nullptr),
-      fOrigValue(nullptr)
-{
-    CARLA_SAFE_ASSERT_RETURN(key != nullptr && key[0] != '\0',);
-
-    fKey = carla_strdup_safe(key);
-    CARLA_SAFE_ASSERT_RETURN(fKey != nullptr,);
-
-    if (const char* const origValue = std::getenv(key))
-    {
-        fOrigValue = carla_strdup_safe(origValue);
-        CARLA_SAFE_ASSERT_RETURN(fOrigValue != nullptr,);
-    }
-
-    if (value != nullptr)
-        carla_setenv(key, value);
-    else if (fOrigValue != nullptr)
-        carla_unsetenv(key);
-}
-
-ScopedEnvVar::~ScopedEnvVar() noexcept
-{
-    bool hasOrigValue = false;
-
-    if (fOrigValue != nullptr)
-    {
-        hasOrigValue = true;
-
-        carla_setenv(fKey, fOrigValue);
-
-        delete[] fOrigValue;
-        fOrigValue = nullptr;
-    }
-
-    if (fKey != nullptr)
-    {
-        if (! hasOrigValue)
-            carla_unsetenv(fKey);
-
-        delete[] fKey;
-        fKey = nullptr;
-    }
-}
-
-// -----------------------------------------------------------------------
-
-ScopedLocale::ScopedLocale() noexcept
-    : fLocale(carla_strdup_safe(::setlocale(LC_NUMERIC, nullptr)))
-{
-    ::setlocale(LC_NUMERIC, "C");
-}
-
-ScopedLocale::~ScopedLocale() noexcept
-{
-    if (fLocale != nullptr)
-    {
-        ::setlocale(LC_NUMERIC, fLocale);
-        delete[] fLocale;
-    }
-}
-
-// -----------------------------------------------------------------------
-
 #undef INVALID_PIPE_VALUE
