@@ -63,6 +63,7 @@ public:
           fDisplay(nullptr),
           fHostWindow(0),
           fChildWindow(0),
+          fChildWindowConfigured(false),
           fIsVisible(false),
           fFirstShow(true),
           fSetSizeCalledAtLeastOnce(false),
@@ -103,6 +104,16 @@ public:
         const Atom _nwi = XInternAtom(fDisplay, "_NET_WM_ICON", False);
         XChangeProperty(fDisplay, fHostWindow, _nwi, XA_CARDINAL, 32, PropModeReplace, (const uchar*)sCarlaX11Icon, sCarlaX11IconSize);
 
+        const Atom _wt = XInternAtom(fDisplay, "_NET_WM_WINDOW_TYPE", False);
+
+        // Setting the window to both dialog and normal will produce a decorated floating dialog
+        // Order is important: DIALOG needs to come before NORMAL
+        const Atom _wts[2] = {
+            XInternAtom(fDisplay, "_NET_WM_WINDOW_TYPE_DIALOG", False),
+            XInternAtom(fDisplay, "_NET_WM_WINDOW_TYPE_NORMAL", False)
+        };
+        XChangeProperty(fDisplay, fHostWindow, _wt, XA_ATOM, 32, PropModeReplace, (const uchar*)&_wts, 2);
+
         if (parentId != 0)
             setTransientWinId(parentId);
     }
@@ -145,7 +156,10 @@ public:
                     carla_zeroStruct(hints);
 
                     if (XGetNormalHints(fDisplay, childWindow, &hints) && hints.width > 0 && hints.height > 0)
-                        setSize(hints.width, hints.height, false);
+                    {
+                        setSize(static_cast<uint>(hints.width),
+                                static_cast<uint>(hints.height), false);
+                    }
                 }
 
                 const Atom _xevp = XInternAtom(fDisplay, "_XEventProc", False);
@@ -214,7 +228,18 @@ public:
                         const uint height = static_cast<uint>(event.xconfigure.height);
 
                         if (fChildWindow != 0)
+                        {
+                            XSizeHints sizeHints;
+                            carla_zeroStruct(sizeHints);
+
+                            if (!fChildWindowConfigured && XGetNormalHints(fDisplay, fChildWindow, &sizeHints))
+                            {
+                                fChildWindowConfigured = true;
+                                XSetNormalHints(fDisplay, fHostWindow, &sizeHints);
+                            }
+
                             XResizeWindow(fDisplay, fChildWindow, width, height);
+                        }
 
                         fCallback->handlePluginUIResized(width, height);
                     }
@@ -358,6 +383,7 @@ private:
     Display* fDisplay;
     Window   fHostWindow;
     Window   fChildWindow;
+    bool     fChildWindowConfigured;
     bool     fIsVisible;
     bool     fFirstShow;
     bool     fSetSizeCalledAtLeastOnce;
@@ -1070,6 +1096,19 @@ bool CarlaPluginUI::tryTransientWinIdMatch(const uintptr_t pid, const char* cons
             carla_stdout("Match found using pid and simple name");
             windowToMap = lastGoodWindowPID;
         }
+        else if (lastGoodWindowNameUTF8 != 0)
+        {
+            if (lastGoodWindowNameUTF8 == lastGoodWindowNameSimple)
+            {
+                carla_stdout("Match found using simple and UTF-8 name (ignoring pid)");
+                windowToMap = lastGoodWindowNameUTF8;
+            }
+            else
+            {
+                carla_stdout("Match found using UTF-8 name (ignoring pid)");
+                windowToMap = lastGoodWindowNameUTF8;
+            }
+        }
         else
         {
             carla_stdout("Match found using pid");
@@ -1085,7 +1124,7 @@ bool CarlaPluginUI::tryTransientWinIdMatch(const uintptr_t pid, const char* cons
         }
         else
         {
-            carla_stdout("Match found using simple and UTF-8 name");
+            carla_stdout("Match found using UTF-8 name");
             windowToMap = lastGoodWindowNameUTF8;
         }
     }
