@@ -124,8 +124,11 @@ CarlaStateSave::Parameter::Parameter() noexcept
       symbol(nullptr),
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
       value(0.0f),
+      mappedControlIndex(CONTROL_INDEX_NONE),
       midiChannel(0),
-      midiCC(-1) {}
+      mappedRangeValid(false),
+      mappedMinimum(0.0f),
+      mappedMaximum(1.0f) {}
 #else
       value(0.0f) {}
 #endif
@@ -304,7 +307,7 @@ bool CarlaStateSave::fillFromXmlElement(const XmlElement* const xmlElement)
                     type = xmlSafeStringCharDup(text, false);
                 else if (tag == "Name")
                     name = xmlSafeStringCharDup(text, false);
-                else if (tag == "Label" || tag == "URI" || tag == "Setup")
+                else if (tag == "Label" || tag == "URI" || tag == "Identifier" || tag == "Setup")
                     label = xmlSafeStringCharDup(text, false);
                 else if (tag == "Binary" || tag == "Filename")
                     binary = xmlSafeStringCharDup(text, false);
@@ -406,6 +409,9 @@ bool CarlaStateSave::fillFromXmlElement(const XmlElement* const xmlElement)
                 else if (tag == "Parameter")
                 {
                     Parameter* const stateParameter(new Parameter());
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+                    bool hasMappedMinimum = false, hasMappedMaximum = false;
+#endif
 
                     for (XmlElement* xmlSubData = xmlData->getFirstChildElement(); xmlSubData != nullptr; xmlSubData = xmlSubData->getNextElement())
                     {
@@ -441,12 +447,33 @@ bool CarlaStateSave::fillFromXmlElement(const XmlElement* const xmlElement)
                         else if (pTag == "MidiCC")
                         {
                             const int cc(pText.getIntValue());
-                            if (cc >= -1 && cc < MAX_MIDI_CONTROL)
-                                stateParameter->midiCC = static_cast<int16_t>(cc);
+                            if (cc > 0 && cc < MAX_MIDI_CONTROL)
+                                stateParameter->mappedControlIndex = static_cast<int16_t>(cc);
+                        }
+                        else if (pTag == "MappedControlIndex")
+                        {
+                            const int ctrl(pText.getIntValue());
+                            if (ctrl > CONTROL_INDEX_NONE && ctrl <= CONTROL_INDEX_MAX_ALLOWED)
+                                stateParameter->mappedControlIndex = static_cast<int16_t>(ctrl);
+                        }
+                        else if (pTag == "MappedMinimum")
+                        {
+                            hasMappedMinimum = true;
+                            stateParameter->mappedMinimum = pText.getFloatValue();
+                        }
+                        else if (pTag == "MappedMaximum")
+                        {
+                            hasMappedMaximum = true;
+                            stateParameter->mappedMaximum = pText.getFloatValue();
                         }
 #endif
                     }
 
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+                    if (hasMappedMinimum && hasMappedMaximum)
+                        stateParameter->mappedRangeValid = true;
+#endif
+
                     parameters.append(stateParameter);
                 }
 
@@ -525,6 +552,15 @@ void CarlaStateSave::dumpToMemoryStream(MemoryOutputStream& content) const
             infoXml << "   <Binary>"   << xmlSafeString(binary, true) << "</Binary>\n";
             infoXml << "   <UniqueID>" << water::int64(uniqueId)       << "</UniqueID>\n";
             break;
+        case PLUGIN_VST3:
+            infoXml << "   <Binary>"   << xmlSafeString(binary, true) << "</Binary>\n";
+            infoXml << "   <Label>"    << xmlSafeString(label, true)  << "</Label>\n";
+            break;
+        case PLUGIN_AU:
+            infoXml << "   <Identifier>" << xmlSafeString(label, true) << "</Identifier>\n";
+            break;
+        case PLUGIN_DLS:
+        case PLUGIN_GIG:
         case PLUGIN_SF2:
             infoXml << "   <Filename>"   << xmlSafeString(binary, true) << "</Filename>\n";
             infoXml << "   <Label>"      << xmlSafeString(label, true)  << "</Label>\n";
@@ -589,10 +625,20 @@ void CarlaStateSave::dumpToMemoryStream(MemoryOutputStream& content) const
             parameterXml << "    <Symbol>" << xmlSafeString(stateParameter->symbol, true) << "</Symbol>\n";
 
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
-        if (stateParameter->midiCC > 0)
+        if (stateParameter->mappedControlIndex > CONTROL_INDEX_NONE && stateParameter->mappedControlIndex <= CONTROL_INDEX_MAX_ALLOWED)
         {
-            parameterXml << "    <MidiCC>"      << stateParameter->midiCC        << "</MidiCC>\n";
-            parameterXml << "    <MidiChannel>" << stateParameter->midiChannel+1 << "</MidiChannel>\n";
+            parameterXml << "    <MidiChannel>"   << stateParameter->midiChannel+1 << "</MidiChannel>\n";
+            parameterXml << "    <MappedControlIndex>" << stateParameter->mappedControlIndex << "</MappedControlIndex>\n";
+
+            if (stateParameter->mappedRangeValid)
+            {
+                parameterXml << "    <MappedMinimum>" << String(stateParameter->mappedMinimum, 15) << "</MappedMinimum>\n";
+                parameterXml << "    <MappedMaximum>" << String(stateParameter->mappedMaximum, 15) << "</MappedMaximum>\n";
+            }
+
+            // backwards compatibility for older carla versions
+            if (stateParameter->mappedControlIndex > 0 && stateParameter->mappedControlIndex < MAX_MIDI_CONTROL)
+                parameterXml << "    <MidiCC>" << stateParameter->mappedControlIndex << "</MidiCC>\n";
         }
 #endif
 
